/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.datatypes;

import java.util.Date;
import java.util.concurrent.*;

public class AbstractDelayedQueueEntry<T> implements Delayed
   {
   protected long endOfDelay = 0;
   final protected Date requestTime = new Date();

   public int compareTo(Delayed d)
	{
	final AbstractDelayedQueueEntry other = (AbstractDelayedQueueEntry) d;

	if (this.endOfDelay < other.endOfDelay)
	   {
	   return -1;
	   }
	else if (this.endOfDelay > other.endOfDelay)
	   {
	   return 1;
	   }
	else
	   {
	   return this.requestTime.compareTo(other.requestTime);
	   }
	}

   public void enableDelay(long seconds)
	{
	endOfDelay = System.currentTimeMillis() + 1000L * seconds;
	}

   public long getDelay(java.util.concurrent.TimeUnit timeUnit)
	{
	if (endOfDelay == 0)
	   {
	   return 0;
	   }
	return timeUnit.convert(endOfDelay - System.currentTimeMillis(), TimeUnit.MILLISECONDS);
	}
   }
