package de.hillenbrand.datatypes;

import java.util.concurrent.DelayQueue;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Cron extends Thread
   {
   public static final int ONE_SECOND = 1;
   public static final int ONE_MINUTE = ONE_SECOND * 60;
   public static final int ONE_HOUR = ONE_MINUTE * 60;
   public static final int ONE_DAY = ONE_HOUR * 24;
   public static final int ONE_WEEK = ONE_DAY * 7;

   private class Data extends AbstractDelayedQueueEntry<CronJob>
	{
	CronJob cronJob;
	int seconds;
	boolean once = false;

	public Data(CronJob cronJob, int seconds, boolean once)
	   {
	   this.cronJob = cronJob;
	   this.seconds = seconds;
	   this.once = once;
	   }
	};
   private final DelayQueue<Data> queue = new DelayQueue<Data>();
   private static Cron instance = null;
   private boolean running = true;

   protected Cron()
	{
	setName("JavaGPE-Cron");
	setPriority(NORM_PRIORITY);
	}

   public static synchronized Cron getInstance()
	{
	if (instance == null)
	   {
	   instance = new Cron();
	   instance.start();
	   }
	return instance;
	}

   public synchronized void add(CronJob cronJob, int seconds)
	{
	add(cronJob, seconds, false);
	}

   public synchronized void add(CronJob cronJob, int seconds, boolean once)
	{
	final Data data = new Data(cronJob, seconds, once);
	data.enableDelay(seconds);
	queue.add(data);
	}

   public void run()
	{
	while (running)
	   {
	   try
		{
		final Data data = queue.take();
		try
		   {
		   data.cronJob.execute();
		   }
		catch (Exception x)
		   {
		   Logger.getLogger(Cron.class.getName()).log(Level.SEVERE, null, x);
		   }
		finally
		   {
		   if (data.once == false && data.cronJob.isFinished() == false)
			{
			data.enableDelay(data.seconds);
			queue.add(data);
			}
		   }
		}
	   catch (InterruptedException ex)
		{
		Logger.getLogger(Cron.class.getName()).log(Level.SEVERE, null, ex);
		}
	   }
	}

   public static void main(String[] args)
	{
	final CronJob r1 = new CronJob()
	   {
	   public void run()
		{
		System.out.println(getClass().getName());
		}
	   };
	final CronJob r2 = new CronJob()
	   {
	   public void run()
		{
		System.out.println(getClass().getName());
		}
	   };
	final CronJob r3 = new CronJob()
	   {
	   public void run()
		{
		System.out.println(getClass().getName());
		}
	   };
	final CronJob r4 = new CronJob()
	   {
	   public void run()
		{
		System.out.println(getClass().getName());
		}
	   };

	Cron.getInstance().add(r1, Cron.ONE_SECOND);
	Cron.getInstance().add(r2, Cron.ONE_SECOND * 5, true);
	Cron.getInstance().add(r3, Cron.ONE_SECOND * 10);
	Cron.getInstance().add(r4, Cron.ONE_SECOND * 15);
	}
   }
