package de.hillenbrand.datatypes;

import de.hillenbrand.swing.HPrivilegedAction;
import de.hillenbrand.util.ArrayUtils;
import java.lang.reflect.Method;
import java.security.AccessController;
import java.util.Properties;
import java.util.logging.Level;

/**
 * A simple cron job
 * @author hillenbr
 */
public abstract class CronJob implements Runnable
   {
   protected boolean finished = false;
   private final Properties props = new Properties();

   protected void setFinished()
	{
	finished = true;
	}

   protected boolean isFinished()
	{
	return finished;
	}

   protected void execute()
	{
	run();
	}

   public synchronized Object setProperty(String key, String value)
	{
	return props.setProperty(key, value);
	}

   public String getProperty(String key, String defaultValue)
	{
	return props.getProperty(key, defaultValue);
	}

   public String getProperty(String key)
	{
	return props.getProperty(key);
	}

   public static CronJob createCronJob(final Object parentClass, final String methodName)
	{
	try
	   {
	   parentClass.getClass().getDeclaredMethod(methodName);
	   }
	catch (NoSuchMethodException x)
	   {
	   java.util.logging.Logger.getLogger(Cron.class.getName()).log(Level.WARNING, "No such method: {0}.{1}()", ArrayUtils.toArray(parentClass.getClass().getName(), methodName));
	   }

	final CronJob result = new CronJob()
	   {
	   public void run()
		{
		try
		   {
		   final Object method = AccessController.doPrivileged(new HPrivilegedAction(parentClass.getClass(), methodName));
		   if (method != null)
			{
			((Method) method).invoke(parentClass);
			}
		   else
			{
			throw new NoSuchMethodException(parentClass.getClass().getName() + "." + methodName + "()");
			}
		   }
		catch (Exception x)
		   {
		   java.util.logging.Logger.getLogger(Cron.class.getName()).log(Level.WARNING, "Can't execute {0}", methodName);
		   }
		}
	   };
	return result;
	}
   }
