/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import de.hillenbrand.jreqtools.*;

public class FileRequesterDemoFrame extends HFrame
   {
   private final JButton button1 = new JButton("Show");
   private final JButton button2 = new JButton("Show");
   private final JButton button3 = new JButton("Show");
   private final JButton button4 = new JButton("Show");
   private final FileRequester req1 = new FileRequester();
   private final FileRequester req2 = new FileRequester();
   private final FileRequester req3 = new FileRequester();
   private final FileRequester req4 = new FileRequester();
   private final HFileString req5 = new HFileString("", HFileString.DIRECTORIES_ONLY);
   private final HFileString req6 = new HFileString("");
   private final HFileString req7 = new HFileString(HFileString.MULTIPLE_FILES);

   public FileRequesterDemoFrame()
	{
	super("JavaGPE: FileRequester Demo");

	// properties
	setDefaultCloseOperation(EXIT_ON_CLOSE);

	// init
	req1.setDirectoriesOnly(false);
	req2.setDirectoriesOnly(true);
	req3.setDirectoriesOnly(false);
	req4.setDirectoriesOnly(true);

	req1.setFileHidingEnabled(true);
	req2.setFileHidingEnabled(true);
	req3.setFileHidingEnabled(false);
	req4.setFileHidingEnabled(false);

	req5.setSaveMode();
	req6.setLoadMode();

	// listeners
	button1.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		req1.request(FileRequesterDemoFrame.this);
		System.out.println(req1.getFile());
		}
	   });
	button2.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		req2.request(FileRequesterDemoFrame.this);
		System.out.println(req2.getFile());
		}
	   });
	button3.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		req3.request(FileRequesterDemoFrame.this);
		System.out.println(req3.getFile());
		}
	   });
	button4.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		req4.request(FileRequesterDemoFrame.this);
		System.out.println(req4.getFile());
		}
	   });

	// create and set panel
	setContentPane(createPanel());

	// set bounds
	setBounds(100, 100, 800, 600);
	pack();
	}

   private JPanel createPanel()
	{
	// create panel
	final JPanel panel = new JPanel();
	panel.setBorder(UIPreferences.getFrameBorder());
	panel.setLayout(new GridBagLayout());

	// create constraints
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();

	// add components
	c.set(0, 0, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
	panel.add(new JLabel("directoriesOnly=false; fileHidingEnabled=true:", JLabel.LEFT), c);
	c.set(1, 0, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(button1, c);
	c.set(0, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
	panel.add(new JLabel("directoriesOnly=true; fileHidingEnabled=true:", JLabel.LEFT), c);
	c.set(1, 1, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(button2, c);
	c.set(0, 2, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
	panel.add(new JLabel("directoriesOnly=false; fileHidingEnabled=false:", JLabel.LEFT), c);
	c.set(1, 2, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(button3, c);
	c.set(0, 3, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL);
	panel.add(new JLabel("directoriesOnly=true; fileHidingEnabled=false:", JLabel.LEFT), c);
	c.set(1, 3, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(button4, c);
	c.set(0, 4, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(req5, c);
	c.set(0, 5, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(req6, c);
	c.set(0, 6, 2, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(req7, c);

	// return panel
	return panel;
	}

   public static void main(String[] args)
	{
	// init UI preferences
	UIPreferences.init();

	// create and open the frame
	final FileRequesterDemoFrame test = new FileRequesterDemoFrame();
	test.addWindowListener(new WindowAdapter()
	   {
	   public void windowClosing(WindowEvent e)
		{
		System.exit(0);
		}
	   });
	test.setVisible(true);
	}
   }
