/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.io;

import java.io.*;

/**
 * Provides easy access to files.
 */
public class FileAccess
   {
   public static final int READ = 1;
   public static final int WRITE = 2;
   FileReader reader;
   FileWriter writer;
   BufferedReader in;
   BufferedWriter out;
   String name;
   boolean isopen;
   int mode;
   IOException exception;

   public FileAccess(File f)
	{
	this(f, READ);
	}

   public FileAccess(String s)
	{
	this(s, READ);
	}

   public FileAccess(File f, int mode)
	{
	this.mode = mode;
	this.name = f.getAbsolutePath();
	this.isopen = false;

	open();
	}

   public FileAccess(String s, int mode)
	{
	this.mode = mode;
	this.name = s;
	this.isopen = false;

	open();
	}

   public void close()
	{
	try
	   {
	   if (mode == READ && isopen)
		{
		in.close();
		reader.close();
		}
	   else if (mode == WRITE && isopen)
		{
		out.close();
		writer.close();
		}
	   }
	catch (IOException ex)
	   {
	   }

	isopen = false;
	in = null;
	out = null;
	reader = null;
	writer = null;
	}

   /**
    * @exception	InterruptedException
    */
   public void finalize() throws InterruptedException
	{
	close();
	}

   public synchronized void newLine() throws IOException
	{
	if (!isopen)
	   {
	   throw exception;
	   }
	if (mode == READ)
	   {
	   throw new IOException("File is opened for reading only.");
	   }
	out.newLine();
	}

   private void open()
	{
	try
	   {
	   if (mode == READ)
		{
		reader = new FileReader(this.name);
		in = new BufferedReader(reader);
		isopen = true;
		}
	   else
		{
		writer = new FileWriter(this.name);
		out = new BufferedWriter(writer);
		isopen = true;
		}
	   }
	catch (IOException ex)
	   {
	   exception = ex;
	   }
	}

   public String readLine() throws IOException
	{
	if (!isopen)
	   {
	   throw exception;
	   }
	if (mode == WRITE)
	   {
	   throw new IOException("File is opened for writing only.");
	   }

	return in.readLine();
	}

   public String readLine(int number) throws IOException
	{
	close();
	open();

	if (!isopen)
	   {
	   throw exception;
	   }
	if (mode == WRITE)
	   {
	   throw new IOException("File is opened for writing only.");
	   }

	String result = null;
	for (int i = 0; i < number; i++)
	   {
	   result = in.readLine();
	   }
	return result;
	}

   public synchronized void write(String s) throws IOException
	{
	if (!isopen)
	   {
	   throw exception;
	   }
	if (mode == READ)
	   {
	   throw new IOException("File is opened for reading only.");
	   }

	out.write(s);
	}

   public void write(boolean c) throws IOException
	{
	String s = "" + c;
	write(s);
	}

   public void write(char c) throws IOException
	{
	String s = "" + c;
	write(s);
	}

   public void write(double c) throws IOException
	{
	String s = "" + c;
	write(s);
	}

   public void write(float c) throws IOException
	{
	String s = "" + c;
	write(s);
	}

   public void write(int c) throws IOException
	{
	String s = "" + c;
	write(s);
	}

   public void write(long c) throws IOException
	{
	String s = "" + c;
	write(s);
	}

   public void writeLine(String s) throws IOException
	{
	write(s);
	newLine();
	}

   public void writeLine(boolean c) throws IOException
	{
	String s = "" + c;
	write(s);
	newLine();
	}

   public void writeLine(char c) throws IOException
	{
	String s = "" + c;
	write(s);
	newLine();
	}

   public void writeLine(double c) throws IOException
	{
	String s = "" + c;
	write(s);
	newLine();
	}

   public void writeLine(float c) throws IOException
	{
	String s = "" + c;
	write(s);
	newLine();
	}

   public void writeLine(int c) throws IOException
	{
	String s = "" + c;
	write(s);
	newLine();
	}

   public void writeLine(long c) throws IOException
	{
	String s = "" + c;
	write(s);
	newLine();
	}
   }
