package de.hillenbrand.io;

import java.io.*;

/**
 * A buffered file reader that omits lines starting with a '#' and empty lines.
 * Additionally ll lines are trimmed by default.
 */
public class HConfigurationFileReader
   {
   final private BufferedReader in;
   final private File file;
   private long lineNumber = 0;

   public HConfigurationFileReader(String file) throws FileNotFoundException
	{
	this.file = new File(file);
	this.in = new BufferedReader(new FileReader(file));
	}

   public HConfigurationFileReader(File file) throws FileNotFoundException
	{
	this.file = file;
	this.in = new BufferedReader(new FileReader(file));
	}

   public HConfigurationFileReader(String dir, String file) throws FileNotFoundException
	{
	this(new File(dir, file));
	}

   public String readLine() throws IOException
	{
	String line;
	do
	   {
	   lineNumber++;
	   line = in.readLine();
	   if (line == null)
		{
		return null;
		}

	   line = line.trim();
	   if (line.startsWith("#"))
		{
		line = null;
		}
	   else if (line.trim().equals(""))
		{
		line = null;
		}
	   }
	while (line == null);

	return line;
	}

   public void close() throws IOException
	{
	in.close();
	}

   public long getCurrentLineNumber()
	{
	return lineNumber;
	}

   public String toString()
	{
	return getClass().getName() + "[file=" + file + ", currentLine=" + lineNumber + "]";
	}
   }
