/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.io;

import java.io.*;
import de.hillenbrand.util.*;

/**
 * Like PrintWriter, but writes only if data has changed and close() has been called.
 */
public class HPrintWriter implements HWriter
   {
   private static final boolean DEBUG = false;
   private final File file;
   private final StringBuffer toWrite = new StringBuffer(5000);

   public HPrintWriter(File file)
	{
	this.file = file;
	}

   public HPrintWriter(String filename)
	{
	this(new File(filename));
	}

   public HPrintWriter(File dir, String file)
	{
	this(new File(dir, file));
	}

   public HPrintWriter(String dir, String file)
	{
	this(new File(dir, file));
	}

   public synchronized void close()
	{
	if (isFileEqual())
	   {
	   if (DEBUG)
		{
		System.out.println("HPrintWriter - file stays the same: " + file);
		}
	   return;
	   }

	try
	   {
	   if (DEBUG)
		{
		System.out.println("HPrintWriter - writing file: " + file);
		}
	   final FileWriter out = new FileWriter(file);
	   out.write(toWrite.toString());
	   out.close();
	   }
	catch (Exception x)
	   {
	   throw new RuntimeException(x);
	   }
	}

   public void flush()
	{
	}

   private boolean isFileEqual()
	{
	try
	   {
	   if (!file.exists())
		{
		return false;
		}
	   if (file.isDirectory())
		{
		return false;
		}
	   final String written = StringUtils.readStringFromFile(file);
	   return written.equals(toWrite.toString());
	   }
	catch (IOException x)
	   {
	   return false;
	   }
	}

   public void print(Object o)
	{
	if (o != null)
	   {
	   println(o.toString());
	   }
	else
	   {
	   println("null");
	   }
	}

   public void print(String s)
	{
	toWrite.append(s);
	}

   public void println()
	{
	toWrite.append("\n");
	}

   public void println(Object o)
	{
	if (o != null)
	   {
	   println(o.toString());
	   }
	else
	   {
	   println("null");
	   }
	}

   public void println(String s)
	{
	toWrite.append(s);
	toWrite.append("\n");
	}
   }
