/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.io;

import java.io.*;

/**
 * A stream for HTML output, that converts non-HTML code to HTML.
 */
public class HtmlPrintWriter
   {
   private static final boolean DEBUG = false;
   private PrintWriter out = null;
   private final StringBuffer buffer = new StringBuffer(1024);

   public HtmlPrintWriter(Writer writer)
	{
	out = new PrintWriter(writer);
	}

   public void close()
	{
	out.close();
	out = null;
	}

   private final String convert(String s)
	{
	buffer.setLength(0);

	for (int i = 0; i < s.length(); ++i)
	   {
	   final int value = s.charAt(i);
	   if (value < 127)
		{
		buffer.append((char) value);
		}
	   else
		{
		switch (value)
		   {
		   case 'ä':
			buffer.append("&auml;");
			break;
		   case 'á':
			buffer.append("&aacute;");
			break;
		   case 'à':
			buffer.append("&agrave;");
			break;
		   case 'â':
			buffer.append("&acirc;");
			break;
		   case 'ã':
			buffer.append("&atilde;");
			break;
		   case 'å':
			buffer.append("&aring;");
			break;
		   case 'Ä':
			buffer.append("&Auml;");
			break;
		   case 'Á':
			buffer.append("&Aacute;");
			break;
		   case 'À':
			buffer.append("&Agrave;");
			break;
		   case 'Â':
			buffer.append("&Acirc;");
			break;
		   case 'Ã':
			buffer.append("&Atilde;");
			break;
		   case 'Å':
			buffer.append("&Aring;");
			break;

		   case 'é':
			buffer.append("&eacute;");
			break;
		   case 'è':
			buffer.append("&egrave;");
			break;
		   case 'ê':
			buffer.append("&ecirc;");
			break;
		   case 'É':
			buffer.append("&Eacute;");
			break;
		   case 'È':
			buffer.append("&Egrave;");
			break;
		   case 'Ê':
			buffer.append("&Ecirc;");
			break;

		   case 'í':
			buffer.append("&iacute;");
			break;
		   case 'ì':
			buffer.append("&igrave;");
			break;
		   case 'î':
			buffer.append("&icirc;");
			break;
		   case 'Í':
			buffer.append("&Iacute;");
			break;
		   case 'Ì':
			buffer.append("&Igrave;");
			break;
		   case 'Î':
			buffer.append("&Icirc;");
			break;

		   case 'ö':
			buffer.append("&ouml;");
			break;
		   case 'ó':
			buffer.append("&oacute;");
			break;
		   case 'ò':
			buffer.append("&ograve;");
			break;
		   case 'ô':
			buffer.append("&ocirc;");
			break;
		   case 'õ':
			buffer.append("&otilde;");
			break;
		   case 'ø':
			buffer.append("&oslash;");
			break;
		   case 'Ö':
			buffer.append("&Ouml;");
			break;
		   case 'Ó':
			buffer.append("&Oacute;");
			break;
		   case 'Ò':
			buffer.append("&Ograve;");
			break;
		   case 'Ô':
			buffer.append("&Ocirc;");
			break;
		   case 'Õ':
			buffer.append("&Otilde;");
			break;
		   case 'Ø':
			buffer.append("&Oslash;");
			break;

		   case 'ü':
			buffer.append("&uuml;");
			break;
		   case 'ú':
			buffer.append("&uacute;");
			break;
		   case 'ù':
			buffer.append("&ugrave;");
			break;
		   case 'û':
			buffer.append("&ucirc;");
			break;
		   case 'Ü':
			buffer.append("&Uuml;");
			break;
		   case 'Ú':
			buffer.append("&Uacute;");
			break;
		   case 'Ù':
			buffer.append("&Ugrave");
			break;
		   case 'Û':
			buffer.append("&Ucirc;");
			break;

		   case 'ß':
			buffer.append("&szlig;");
			break;

		   case 'ç':
			buffer.append("&ccedil;");
			break;
		   case 'Ç':
			buffer.append("&Ccedil;");
			break;

		   case 'ñ':
			buffer.append("&ntilde;");
			break;
		   case 'Ñ':
			buffer.append("&Ntilde;");
			break;

		   case 'ý':
			buffer.append("&yacute;");
			break;
		   case 'Ý':
			buffer.append("&Yacute;");
			break;
		   case 'ÿ':
			buffer.append("&yuml;");
			break;

		   case 166:
			buffer.append("&#x0160;");
			break;
		   case 235:
			buffer.append("&#x00EB;");
			break;
		   case 239:
			buffer.append("&#x00EF;");
			break;
		   case 352:
			buffer.append("&#x0161;");
			break;

		   case 230:
			buffer.append("&#x00E6;");
			break;

		   default:
			System.err.println("de.hillenbrand.io.HtmlPrintWriter.convert(): unbekanntes Zeichen " + s.charAt(i) + " [" + value + "]\n" + s.substring(i));
			System.exit(5);
		   }
		}
	   }

	return buffer.toString();
	}

   public void flush()
	{
	out.flush();
	}

   public void print(String s)
	{
	out.print(convert(s));
	}

   public void println(String s)
	{
	out.println(convert(s));
	}
   }
