/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import java.awt.*;
import javax.swing.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;

/**
 * A requester that allows a user to change his password, i.e. a requester with three text fields. The first
 * one is for the old password, and the second and third are for the new password.
 */
public class ChangePasswordRequester extends OkCancelRequester
   {
   // components
   protected final JPasswordField p1 = new HPasswordField();
   protected final JPasswordField p2 = new HPasswordField();
   protected final JPasswordField p3 = new HPasswordField();
   protected final JLabel msg = new JLabel(JReqTools.getResourceString("ChangePasswordRequester.NewPassword2Message"), JLabel.LEFT);

   /**
    * Constructs a new ChangePasswordRequester with a title and a default  message.
    * @param title
    */
   public ChangePasswordRequester(String title)
	{
	this(title, JReqTools.getResourceString("ChangePasswordRequester.Message"));
	}

   /**
    * Constructs a new ChangePasswordRequester with a title and a message.
    * @param title
    * @param message
    */
   public ChangePasswordRequester(String title, String message)
	{
	super(title, message);

	this.msg.setFont(UIPreferences.getAcceleratorFont());
	}

   /**
    * Returns the value of the newPassword property.
    * @return
    */
   public byte[] getNewPassword()
	{
	return new String(p2.getPassword()).getBytes();
	}

   protected JComponent getOkTriggerComponent()
	{
	return p3;
	}

   /**
    * Returns the value of the oldPassword property.
    * @return
    */
   public byte[] getOldPassword()
	{
	return new String(p1.getPassword()).getBytes();
	}

   protected JPanel getUserPanel()
	{
	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());
	final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// add components
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(new JLabel(JReqTools.getResourceString("ChangePasswordRequester.OldPassword"), JLabel.LEFT), c);
	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(p1, c);

	c.gridx = 1;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(msg, c);

	c.gridx = 0;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(new JLabel(JReqTools.getResourceString("ChangePasswordRequester.NewPassword1"), JLabel.LEFT), c);
	c.gridx = 1;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(p2, c);

	c.gridx = 0;
	c.gridy = 3;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(new JLabel(JReqTools.getResourceString("ChangePasswordRequester.NewPassword2"), JLabel.LEFT), c);
	c.gridx = 1;
	c.gridy = 3;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(p3, c);

	// return panel
	return panel;
	}

   protected void prepareForRequest()
	{
	p1.setText("");
	p2.setText("");
	p3.setText("");
	}

   protected String validateData()
	{
	final String s2 = new String(p2.getPassword());
	final String s3 = new String(p3.getPassword());

	if (s2.length() < 8)
	   {
	   return JReqTools.getResourceString("ChangePasswordRequester.Error1");
	   }
	if (!s2.equals(s3))
	   {
	   return JReqTools.getResourceString("ChangePasswordRequester.Error2");
	   }

	return null;
	}
   }
