/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;

/**
 * A requester for simple strings.
 *
 * @author	Markus Hillenbrand
 */
public class ColorRequester extends OkCancelRequester
   {
   // components
   private JColorChooser chooser = new JColorChooser();
   // Property: color
   protected Color color = null;

   /**
    * Constructs a new ColorRequester.
    *
    * @param	message - The message to be shown in the requester.
    * @param	defaultcolor - The initial color value.
    */
   public ColorRequester(String message, Color defaultcolor)
	{
	this("JReqTools: ColorRequester", message, defaultcolor);
	}

   /**
    * Constructs a new ColorRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	defaultcolor - The initial color value.
    */
   public ColorRequester(String title, String message, Color defaultcolor)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_REQUEST_COLOR));

	// init variables
	this.color = defaultcolor;

	// init chooser
	//chooser.setControlButtonsAreShown(false);
	chooser.setBorder(BorderFactory.createEmptyBorder());
	}

   /**
    * Returns the value of the color property.
    * @return
    */
   public Color getColor()
	{
	return color;
	}

   protected JPanel getUserPanel()
	{
	// get default constraints
	final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 1;
	c.fill = GridBagConstraints.BOTH;

	panel.add(chooser, c);

	return panel;
	}

   protected void ok()
	{
	// the result may be a ColorUIResource, so we create a new color object
	final Color c = chooser.getColor();
	color = new Color(c.getRed(), c.getGreen(), c.getBlue());
	}

   protected void prepareForRequest()
	{
	chooser.setColor(color);
	}

   /**
    * Assigns the value of the color property.
    * @param arg
    */
   public void setColor(Color arg)
	{
	color = arg;
	}
   }
