/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This is a requester for colors.
 *
 * @author	Markus Hillenbrand
 */
public class ColorRequester2 extends OkCancelRequester
   {
   private Color color = null;
   // components
   private JPanel colorfield = new JPanel();
   private JSlider hueslider = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
   private JSlider saturationslider = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
   private JSlider brightnessslider = new JSlider(JSlider.HORIZONTAL, 0, 100, 0);
   private JSlider redslider = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
   private JSlider greenslider = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
   private JSlider blueslider = new JSlider(JSlider.HORIZONTAL, 0, 255, 0);
   private HTextField redtext = new HTextField();
   private HTextField greentext = new HTextField();
   private HTextField bluetext = new HTextField();
   private HTextField huetext = new HTextField();
   private HTextField saturationtext = new HTextField();
   private HTextField brightnesstext = new HTextField();
   private boolean handleEvent = true;
   private int tabindex = 0;

   /**
    * Constructs a new ColorRequester.
    *
    * @param	message - The message to be shown in the requester.
    * @param	defaultcolor - The initial color value.
    */
   public ColorRequester2(String message, Color defaultcolor)
	{
	this("JReqTools: ColorRequester", message, defaultcolor);
	}

   /**
    * Constructs a new ColorRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	defaultcolor - The initial color value.
    */
   public ColorRequester2(String title, String message, Color defaultcolor)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_REQUEST_COLOR));

	// init variables
	this.color = defaultcolor;

	// init components
	colorfield.setMinimumSize(new Dimension(100, 100));
	colorfield.setMaximumSize(new Dimension(100, 100));
	colorfield.setPreferredSize(new Dimension(100, 100));
	colorfield.setBorder(BorderFactory.createLineBorder(Color.black));
	colorfield.setBackground(color);
	colorfield.setPreferredSize(new Dimension(100, 100));
	huetext.setEditable(false);
	saturationtext.setEditable(false);
	brightnesstext.setEditable(false);
	redtext.setEditable(false);
	greentext.setEditable(false);
	bluetext.setEditable(false);

	// listeners
	redslider.addChangeListener(new ChangeListener()
	   {
	   public void stateChanged(ChangeEvent e)
		{
		changedRed();
		}
	   });
	greenslider.addChangeListener(new ChangeListener()
	   {
	   public void stateChanged(ChangeEvent e)
		{
		changedGreen();
		}
	   });
	blueslider.addChangeListener(new ChangeListener()
	   {
	   public void stateChanged(ChangeEvent e)
		{
		changedBlue();
		}
	   });
	hueslider.addChangeListener(new ChangeListener()
	   {
	   public void stateChanged(ChangeEvent e)
		{
		changedHue();
		}
	   });
	saturationslider.addChangeListener(new ChangeListener()
	   {
	   public void stateChanged(ChangeEvent e)
		{
		changedSaturation();
		}
	   });
	brightnessslider.addChangeListener(new ChangeListener()
	   {
	   public void stateChanged(ChangeEvent e)
		{
		changedBrightness();
		}
	   });
	}

   private void changedBlue()
	{
	if (!handleEvent)
	   {
	   return;
	   }

	// Events nicht mehr behandeln wegen setValue()
	handleEvent = false;

	// Werte auslesen
	int red = redslider.getValue();
	int green = greenslider.getValue();
	int blue = blueslider.getValue();

	// Farbe ändern
	Color c = new Color(red, green, blue);
	colorfield.setBackground(c);

	// Red Wert anzeigen
	bluetext.setText("" + blue);

	// HSB Werte ausrechnen
	float[] hsb = Color.RGBtoHSB(red, green, blue, null);
	int hue = (int) (100.0 * hsb[0]);
	int saturation = (int) (100.0 * hsb[1]);
	int brightness = (int) (100.0 * hsb[2]);

	// Slider für HSB ändern
	hueslider.setValue(hue);
	saturationslider.setValue(saturation);
	brightnessslider.setValue(brightness);

	// Teste für HSB ändern
	huetext.setText("" + hue);
	saturationtext.setText("" + saturation);
	brightnesstext.setText("" + brightness);

	// Events wieder behandeln
	handleEvent = true;
	}

   private void changedBrightness()
	{
	if (!handleEvent)
	   {
	   return;
	   }

	// Events nicht mehr behandeln wegen setValue()
	handleEvent = false;

	// Werte auslesen
	int hue = hueslider.getValue();
	int saturation = saturationslider.getValue();
	int brightness = brightnessslider.getValue();

	// Farbe ändern
	Color c = new Color(Color.HSBtoRGB((float) hue / 100, (float) saturation / 100, (float) brightness / 100));
	colorfield.setBackground(c);

	// RGB Werte holen
	int red = c.getRed();
	int green = c.getGreen();
	int blue = c.getBlue();

	// Red Wert anzeigen
	brightnesstext.setText("" + brightness);

	// Slider für RGB ändern
	redslider.setValue(red);
	greenslider.setValue(green);
	blueslider.setValue(blue);

	// Teste für RGB ändern
	redtext.setText("" + red);
	greentext.setText("" + green);
	bluetext.setText("" + blue);

	// Events wieder behandeln
	handleEvent = true;
	}

   private void changedGreen()
	{
	if (!handleEvent)
	   {
	   return;
	   }

	// Events nicht mehr behandeln wegen setValue()
	handleEvent = false;

	// Werte auslesen
	int red = redslider.getValue();
	int green = greenslider.getValue();
	int blue = blueslider.getValue();

	// Farbe ändern
	Color c = new Color(red, green, blue);
	colorfield.setBackground(c);

	// Red Wert anzeigen
	greentext.setText("" + green);

	// HSB Werte ausrechnen
	float[] hsb = Color.RGBtoHSB(red, green, blue, null);
	int hue = (int) (100.0 * hsb[0]);
	int saturation = (int) (100.0 * hsb[1]);
	int brightness = (int) (100.0 * hsb[2]);

	// Slider für HSB ändern
	hueslider.setValue(hue);
	saturationslider.setValue(saturation);
	brightnessslider.setValue(brightness);

	// Teste für HSB ändern
	huetext.setText("" + hue);
	saturationtext.setText("" + saturation);
	brightnesstext.setText("" + brightness);

	// Events wieder behandeln
	handleEvent = true;
	}

   private void changedHue()
	{
	if (!handleEvent)
	   {
	   return;
	   }

	// Events nicht mehr behandeln wegen setValue()
	handleEvent = false;

	// Werte auslesen
	int hue = hueslider.getValue();
	int saturation = saturationslider.getValue();
	int brightness = brightnessslider.getValue();

	// Farbe ändern
	Color c = new Color(Color.HSBtoRGB((float) hue / 100, (float) saturation / 100, (float) brightness / 100));
	colorfield.setBackground(c);

	// RGB Werte holen
	int red = c.getRed();
	int green = c.getGreen();
	int blue = c.getBlue();

	// Red Wert anzeigen
	huetext.setText("" + hue);

	// Slider für RGB ändern
	redslider.setValue(red);
	greenslider.setValue(green);
	blueslider.setValue(blue);

	// Teste für RGB ändern
	redtext.setText("" + red);
	greentext.setText("" + green);
	bluetext.setText("" + blue);

	// Events wieder behandeln
	handleEvent = true;
	}

   private void changedRed()
	{
	if (!handleEvent)
	   {
	   return;
	   }

	// Events nicht mehr behandeln wegen setValue()
	handleEvent = false;

	// Werte auslesen
	int red = redslider.getValue();
	int green = greenslider.getValue();
	int blue = blueslider.getValue();

	// Farbe ändern
	Color c = new Color(red, green, blue);
	colorfield.setBackground(c);

	// Red Wert anzeigen
	redtext.setText("" + red);

	// HSB Werte ausrechnen
	float[] hsb = Color.RGBtoHSB(red, green, blue, null);
	int hue = (int) (100.0 * hsb[0]);
	int saturation = (int) (100.0 * hsb[1]);
	int brightness = (int) (100.0 * hsb[2]);

	// Slider für HSB ändern
	hueslider.setValue(hue);
	saturationslider.setValue(saturation);
	brightnessslider.setValue(brightness);

	// Teste für HSB ändern
	huetext.setText("" + hue);
	saturationtext.setText("" + saturation);
	brightnesstext.setText("" + brightness);

	// Events wieder behandeln
	handleEvent = true;
	}

   private void changedSaturation()
	{
	if (!handleEvent)
	   {
	   return;
	   }

	// Events nicht mehr behandeln wegen setValue()
	handleEvent = false;

	// Werte auslesen
	int hue = hueslider.getValue();
	int saturation = saturationslider.getValue();
	int brightness = brightnessslider.getValue();

	// Farbe ändern
	Color c = new Color(Color.HSBtoRGB((float) hue / 100, (float) saturation / 100, (float) brightness / 100));
	colorfield.setBackground(c);

	// RGB Werte holen
	int red = c.getRed();
	int green = c.getGreen();
	int blue = c.getBlue();

	// Red Wert anzeigen
	saturationtext.setText("" + saturation);

	// Slider für RGB ändern
	redslider.setValue(red);
	greenslider.setValue(green);
	blueslider.setValue(blue);

	// Teste für RGB ändern
	redtext.setText("" + red);
	greentext.setText("" + green);
	bluetext.setText("" + blue);

	// Events wieder behandeln
	handleEvent = true;
	}

   public Color getColor()
	{
	return color;
	}

   protected JPanel getUserPanel()
	{
	// create components
	final JTabbedPane tabbedpane = new JTabbedPane();

	// get default constraints
	final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// create RGB panel
	final JPanel rgbPanel = new JPanel(new GridBagLayout());
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(new JLabel(JReqTools.getResourceString("Red")), c);
	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 10;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(redslider, c);
	c.gridx = 2;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(redtext, c);
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(new JLabel(JReqTools.getResourceString("Green")), c);
	c.gridx = 1;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 10;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(greenslider, c);
	c.gridx = 2;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(greentext, c);
	c.gridx = 0;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(new JLabel(JReqTools.getResourceString("Blue")), c);
	c.gridx = 1;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 10;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(blueslider, c);
	c.gridx = 2;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	rgbPanel.add(bluetext, c);

	// create RGB panel
	final JPanel hsbPanel = new JPanel(new GridBagLayout());
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(new JLabel(JReqTools.getResourceString("Hue")), c);
	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 10;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(hueslider, c);
	c.gridx = 2;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(huetext, c);
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(new JLabel(JReqTools.getResourceString("Saturation")), c);
	c.gridx = 1;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 10;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(saturationslider, c);
	c.gridx = 2;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(saturationtext, c);
	c.gridx = 0;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(new JLabel(JReqTools.getResourceString("Brightness")), c);
	c.gridx = 1;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 10;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(brightnessslider, c);
	c.gridx = 2;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	hsbPanel.add(brightnesstext, c);

	// create tabbed pane
	tabbedpane.addTab(JReqTools.getResourceString("RGBModel"), rgbPanel);
	tabbedpane.addTab(JReqTools.getResourceString("HSBModel"), hsbPanel);
	tabbedpane.setSelectedIndex(tabindex);

	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 1;
	c.fill = GridBagConstraints.VERTICAL;
	panel.add(colorfield, c);
	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 1;
	c.fill = GridBagConstraints.BOTH;
	panel.add(tabbedpane, c);

	return panel;
	}

   protected void ok()
	{
	color = new Color(redslider.getValue(), greenslider.getValue(), blueslider.getValue());
	}

   protected void prepareForRequest()
	{
	// get RGB color values
	int red = color.getRed();
	int green = color.getGreen();
	int blue = color.getBlue();

	// calculate HSB color values
	float[] hsb = Color.RGBtoHSB(red, green, blue, null);
	int hue = (int) (100.0 * hsb[0]);
	int saturation = (int) (100.0 * hsb[1]);
	int brightness = (int) (100.0 * hsb[2]);

	// set component values
	redtext.setText("" + red);
	greentext.setText("" + green);
	bluetext.setText("" + blue);
	huetext.setText("" + hue);
	saturationtext.setText("" + saturation);
	brightnesstext.setText("" + brightness);
	redslider.setValue(red);
	greenslider.setValue(green);
	blueslider.setValue(blue);
	hueslider.setValue(hue);
	saturationslider.setValue(saturation);
	brightnessslider.setValue(brightness);
	}

   public void setColor(Color newcolor)
	{
	color = newcolor;
	}
   }
