/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import java.awt.*;
import javax.swing.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.io.*;
import de.hillenbrand.util.*;

/**
 * A Requester that loads a file, and display is for editing.
 */
public class FileEditorRequester extends OkCancelRequester
   {
   // components
   protected final HEditorPane text = new HEditorPane();
   protected final JLabel label = new JLabel("", JLabel.LEFT);

   // Property: file
   protected File file = null;

   /**
    * Constructs a new FileEditorRequester with a default title and message, and a given file.
    * @param file
    */
   public FileEditorRequester(File file)
	{
	this("JReqTools: FileEditorRequester", null, file);
	}

   /**
    * Constructs a new FileEditorRequester with a default title, a given message, and file.
    * @param message
    * @param file
    */
   public FileEditorRequester(String message, File file)
	{
	this("JReqTools: FileEditorRequester", message, file);
	}

   /**
    * Constructs a new FileEditorRequester with a given title, message, and file.
    * @param title
    * @param message
    * @param file
    */
   public FileEditorRequester(String title, String message, File file)
	{
	super(title, message);

	// init
	this.file = file;
	this.label.setText(file != null ? file.getAbsolutePath() : "ERROR: No file associated.");

	// init components
	text.setPreferredSize(new Dimension(200, 200));
	label.setFont(UIPreferences.getAcceleratorFont());
	label.setForeground(Color.gray);
	HEditorPane.setDefaultTabSize(8);
	}

   /**
    * Returns the value of the file property.
    * @return
    */
   public File getFile()
	{
	return file;
	}

   /**
    * Returns the value of the font property.
    * @return
    */
   public Font getFont()
	{
	return text.getFont();
	}

   protected JPanel getUserPanel()
	{
	// create panel
	final JPanel panel = new JPanel(new GridBagLayout());
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	c.set(0, 0, 1, 1, 1, 1, GridBagConstraints.BOTH);
	panel.add(new JScrollPane(text), c);
	c.set(0, 1, 1, 0, 1, 0, GridBagConstraints.HORIZONTAL);
	panel.add(label, c);
	return panel;
	}

   @Override
   protected void ok()
	{
	FileUtils.writeToFileNoException(file, text.getText());
	}

   @Override
   protected void prepareForRequest()
	{
	try
	   {
	   final String s = StringUtils.readStringFromFile(file);
	   text.setText(s);
	   }
	catch (Exception x)
	   {
	   label.setText(x.getLocalizedMessage());
	   }
	}

   /**
    * Assigns the value of the file property.
    * @param f 
    */
   public void setFile(File f)
	{
	this.file = f;
	this.label.setText(file != null ? file.getAbsolutePath() : "ERROR: No file associated.");
	}

   /**
    * Assigns the value of the font property.
    * @param arg
    */
   public void setFont(Font arg)
	{
	text.setFont(arg);
	}
   }
