/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
* This is a requester for image icons.
* 
* @author	Markus Hillenbrand
*/

public class IconRequester extends OkCancelRequester {
	private static final boolean DEBUG = false;

	// variables
	private Icon [] 				icons				= null;
	private JToggleButton []		buttons			= null;
	private Icon					icon				= null;
	private boolean 				addAsScrollPane 	= false;
	
	public IconRequester(String message, Icon [] icons, Icon selected)  {
		this("JReqTools: IconRequester", message, icons, selected);
	}

	public IconRequester(String title, String message, Icon [] icons, Icon selected)  {
		super(title, message);
		
		this.icons = icons;
		this.icon = selected;
	}

	/**
	* Returns the value of the icon property.
	*/

	public Icon getIcon()  {
		return icon;
	}

	protected Dimension getPreferredRequesterSize()  {
		return new Dimension(405, 400);
	}

	protected JPanel getUserPanel()  {
		// create components
		final ButtonGroup group = new ButtonGroup();
		final JPanel panel = new JPanel(new FlowLayout2D());
		
		// create storage array
		buttons = new JToggleButton[icons.length];
		
		// create toggle buttons and add them
		for (int i=0; i<icons.length; ++i) 
			{
			buttons[i] = new JToggleButton(icons[i], icons[i] == icon);
			buttons[i].setMargin(new Insets(1,1,1,1));
			buttons[i].setFocusPainted(false);
			buttons[i].setPreferredSize(new Dimension(50, 50));
			buttons[i].setMaximumSize(new Dimension(50, 50));
			buttons[i].setMinimumSize(new Dimension(50, 50));
			buttons[i].addMouseListener(new MouseAdapter() { public void mouseClicked(MouseEvent e) { if (e.getClickCount() > 1) { ok(); setVisible(false); }}});
			panel.add(buttons[i]);
			group.add(buttons[i]);
			}
		
		final JPanel result = new JPanel(new GridBagLayout());
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=1; c.fill = GridBagConstraints.BOTH; 
		final JScrollPane sp = new JScrollPane(panel);
		//sp.setBorder(BorderFactory.createEmptyBorder());
		result.add(sp, c);
		
		return result;
	}

	protected void ok()  {
		icon = null;
		
		for (int i=0; i<buttons.length; ++i)
			{
			if (buttons[i].isSelected()) icon = icons[i];
			}
	}

	protected void prepareForRequest()  {
		for (int i=0; i<icons.length; ++i)
			{
			if (icon == icons[i]) buttons[i].setSelected(true);
			}
	}

	/**
	* Assigns the value of the icon property.
	*/

	public void setIcon(Icon arg)  {
		icon = arg;
	}

}
