/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.xml.namespace.*;
import org.apache.axis.*;

/**
 * A convenience class that provides "shortcuts" to frequently used requesters and dialogs.
 */
public final class JReqTools
   {
   private static final ResourceBundle resources = ResourceBundle.getBundle("de.hillenbrand.jreqtools.resources.jreqtools");
   static final ApplicationPreferences prefs = new ApplicationPreferences("de.hillenbrand.jreqtools");

   private final static class ExceptionDetailsDialog extends HDialog
	{
	private final JButton ok = new JButton(JReqTools.getResourceString("Ok"), UIManager.getIcon(UIPreferences.BUTTON_ICON_OK));
	private static boolean axis = false;

	public ExceptionDetailsDialog(JDialog parent, String title, Throwable t, Icon icon)
	   {
	   super(parent, title);
	   init(title, t, icon);
	   }

	public ExceptionDetailsDialog(JFrame parent, String title, Throwable t, Icon icon)
	   {
	   super(parent, title);
	   init(title, t, icon);
	   }

	public ExceptionDetailsDialog(JInternalFrame parent, String title, Throwable t, Icon icon)
	   {
	   super(parent, title);
	   init(title, t, icon);
	   }

	static
	   {
	   try
		{
		Class.forName("org.apache.AxisFault");
		axis = true;
		}
	   catch (Exception x)
		{
		}
	   }

	private void init(final String title, final Throwable t, final Icon icon)
	   {
	   setDefaultButton(ok);

	   if (axis && t instanceof AxisFault)
		{
		init_AxisFault("AxisFault", (AxisFault) t, icon);
		}
	   else
		{
		init_Throwable(title, t, icon);
		}
	   }

	private void init_AxisFault(String titletext, AxisFault af, Icon icon)
	   {
	   // listeners
	   ok.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   setVisible(false);
		   }
		});

	   // get the constraints
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   // create message panel
	   final JPanel mpanel = new JPanel(new GridBagLayout());

	   final JLabel title = new JLabel(titletext, JLabel.LEFT);
	   title.setFont(UIPreferences.getMessageFont());

	   final JPanel afPanel = new JPanel(new GridBagLayout());
	   afPanel.setBorder(BorderFactory.createLoweredBevelBorder());
	   int y = 0;

	   final String faultString = af.getFaultString();
	   if (faultString != null)
		{
		c.gridx = 0;
		c.gridy = y;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel("Fault String:", JLabel.LEFT), c);
		c.gridx = 1;
		c.gridy = y++;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel(faultString, JLabel.LEFT), c);
		}

	   final String faultActor = af.getFaultActor();
	   if (faultActor != null)
		{
		c.gridx = 0;
		c.gridy = y;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel("Fault Actor:", JLabel.LEFT), c);
		c.gridx = 1;
		c.gridy = y++;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel(faultActor, JLabel.LEFT), c);
		}

	   final String faultNode = af.getFaultNode();
	   if (faultNode != null)
		{
		c.gridx = 0;
		c.gridy = y;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel("Fault Node:", JLabel.LEFT), c);
		c.gridx = 1;
		c.gridy = y++;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel(faultNode, JLabel.LEFT), c);
		}

	   final QName faultCode = af.getFaultCode();
	   if (faultCode != null)
		{
		c.gridx = 0;
		c.gridy = y;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel("Fault Code:", JLabel.LEFT), c);
		c.gridx = 1;
		c.gridy = y++;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel(faultCode.toString(), JLabel.LEFT), c);
		}

	   final QName[] faultSubcodes = af.getFaultSubCodes();
	   if (faultSubcodes != null)
		{
		c.gridx = 0;
		c.gridy = y;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel("Fault Subcodes:", JLabel.LEFT), c);
		for (int i = 0; i < faultSubcodes.length; ++i)
		   {
		   c.gridx = 1;
		   c.gridy = y++;
		   c.gridwidth = 1;
		   c.gridheight = 1;
		   c.weightx = 1;
		   c.weighty = 0;
		   c.fill = GridBagConstraints.HORIZONTAL;
		   afPanel.add(new JLabel(faultSubcodes[i].toString(), JLabel.LEFT), c);
		   }
		}

	   org.w3c.dom.Element[] details = af.getFaultDetails();
	   if (details != null)
		{
		c.gridx = 0;
		c.gridy = y;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		afPanel.add(new JLabel("Fault Details:", JLabel.LEFT), c);
		String s = "";
		for (int i = 0; i < details.length; ++i)
		   {
		   s += details[i].toString();
		   }
		if (s.startsWith("<stackTrace>") && s.endsWith("</stackTrace>"))
		   {
		   s = s.substring(12, s.length() - 13);
		   }
		final JTextArea ta = new JTextArea(s);
		ta.setEditable(false);
		ta.setBackground(getBackground());
		ta.setBorder(BorderFactory.createEmptyBorder());
		c.gridx = 1;
		c.gridy = y++;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 1;
		c.fill = GridBagConstraints.BOTH;
		afPanel.add(new JScrollPane(ta), c);
		}

	   final JLabel label = new JLabel(icon);
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.VERTICAL;
	   mpanel.add(label, c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   mpanel.add(title, c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   mpanel.add(afPanel, c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());

	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(mpanel, c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HSeparatorPanel(ok), c);

	   // finish
	   this.setContentPane(panel);
	   }

	private void init_Throwable(String titletext, Throwable t, Icon icon)
	   {
	   // listeners
	   ok.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   setVisible(false);
		   }
		});

	   // get the constraints
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   // create message panel
	   final JPanel mpanel = new JPanel(new GridBagLayout());

	   final JLabel title = new JLabel(titletext, JLabel.LEFT);
	   title.setFont(UIPreferences.getMessageFont());

	   final JTextArea ta = new JTextArea(createStringFromException(t));
	   ta.setEditable(false);
	   ta.setBackground(getBackground());
	   ta.setBorder(BorderFactory.createEmptyBorder());

	   final JLabel label = new JLabel(icon);
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.VERTICAL;
	   mpanel.add(label, c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   mpanel.add(title, c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   mpanel.add(new JScrollPane(ta), c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());

	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(mpanel, c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HSeparatorPanel(ok), c);

	   // finish
	   this.setContentPane(panel);
	   }

	public void setVisible(boolean bool)
	   {
	   if (bool)
		{
		ok.requestFocus();
		}
	   super.setVisible(bool);
	   }
	}

   private final static class ExceptionDialog extends HDialog
	{
	private final JButton ok = new JButton(JReqTools.getResourceString("Ok"), UIManager.getIcon(UIPreferences.BUTTON_ICON_OK));
	private final JButton details = new JButton(JReqTools.getResourceString("Details"), UIManager.getIcon(UIPreferences.BUTTON_ICON_APPLY));
	private static boolean axis = false;

	public ExceptionDialog(JDialog parent, String title, Throwable t, Icon icon)
	   {
	   super(parent, title);
	   init(null, title, t, icon);
	   }

	public ExceptionDialog(JFrame parent, String title, Throwable t, Icon icon)
	   {
	   super(parent, title);
	   init(parent, title, t, icon);
	   }

	public ExceptionDialog(JInternalFrame parent, String title, Throwable t, Icon icon)
	   {
	   super(parent, title);
	   init(SwingUtils.getJFrame(parent), title, t, icon);
	   }

	static
	   {
	   try
		{
		Class.forName("org.apache.axis.AxisFault");
		axis = true;
		}
	   catch (Exception x)
		{
		}
	   }

	private void init(final JFrame parent, final String title, final Throwable t, final Icon icon)
	   {
	   // init
	   setDefaultButton(ok);

	   // listeners
	   ok.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   setVisible(false);
		   }
		});
	   details.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final ExceptionDetailsDialog d = new ExceptionDetailsDialog(parent, title, t, icon);
		   d.setVisible(true);
		   }
		});

	   // get the constraints
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   // create message panel
	   final JPanel mpanel = new JPanel(new GridBagLayout());

	   final JLabel titleLabel = new JLabel(t.getClass().getName() + ":", JLabel.LEFT);
	   titleLabel.setFont(UIPreferences.getMessageFont().deriveFont(Font.ITALIC));

	   String text = t.getMessage();
	   if (text == null)
		{
		text = t.getLocalizedMessage();
		}
	   if (text == null)
		{
		text = "";
		}

	   String source = "";
	   final StackTraceElement[] stack = t.getStackTrace();
	   if (stack != null && stack.length > 0)
		{
		source += "\nClass: " + stack[0].getClassName();
		source += "\nMethod: " + stack[0].getMethodName();
		source += "\nFile: " + stack[0].getFileName();
		source += "\nLine: " + stack[0].getLineNumber();
		}

	   if (axis && t instanceof AxisFault)
		{
		if (((AxisFault) t).getFaultString() != null)
		   {
		   text = ((AxisFault) t).getFaultString();
		   }
		source += "\n";
		if (((AxisFault) t).getFaultActor() != null)
		   {
		   source += "\nAxis fault actor: " + ((AxisFault) t).getFaultActor();
		   }
		if (((AxisFault) t).getFaultCode() != null)
		   {
		   source += "\nAxis fault code: " + ((AxisFault) t).getFaultCode();
		   }
		if (((AxisFault) t).getFaultNode() != null)
		   {
		   source += "\nAxis fault node: " + ((AxisFault) t).getFaultNode();
		   }
		}

	   final HMultiLineLabel textLabel = new HMultiLineLabel(text);
	   textLabel.setFont(UIPreferences.getMessageFont());

	   final HMultiLineLabel sourceLabel = new HMultiLineLabel(source);
	   sourceLabel.setFont(UIPreferences.getAcceleratorFont());

	   final JLabel label = new JLabel(icon);
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.VERTICAL;
	   mpanel.add(label, c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   mpanel.add(titleLabel, c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   mpanel.add(textLabel, c);
	   c.gridx = 1;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   mpanel.add(sourceLabel, c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());

	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(mpanel, c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HSeparatorPanel(details, ok), c);

	   // finish
	   this.setContentPane(panel);
	   }

	public void setVisible(boolean bool)
	   {
	   if (bool)
		{
		ok.requestFocus();
		}
	   super.setVisible(bool);
	   }
	}

   private final static class MessageDialog extends HDialog
	{
	private JButton ok = new JButton(JReqTools.getResourceString("Ok"), UIManager.getIcon(UIPreferences.BUTTON_ICON_OK));

	public MessageDialog(JDialog parent, String title, String msg, Icon icon)
	   {
	   super(parent, title);
	   init(msg, icon);
	   }

	public MessageDialog(JFrame parent, String title, String msg, Icon icon)
	   {
	   super(parent, title);
	   init(msg, icon);
	   }

	public MessageDialog(JInternalFrame parent, String title, String msg, Icon icon)
	   {
	   super(parent, title);
	   init(msg, icon);
	   }

	private void init(String msg, Icon icon)
	   {
	   // init
	   setDefaultButton(ok);

	   // listeners
	   ok.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   setVisible(false);
		   }
		});

	   // content pane
	   final HGridBagConstraints c = UIPreferences.getGridBagConstraints();

	   final JLabel label = new JLabel(icon);
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);

	   final JComponent mlabel = msg.startsWith("<html>")
		     ? new JEditorPane("text/html", msg)
		     : new HMultiLineLabel(msg, HMultiLineLabel.LEFT);
	   mlabel.setFont(UIPreferences.getMessageFont());

	   if (mlabel instanceof JEditorPane)
		{
		((JEditorPane)mlabel).setEditable(false);
		mlabel.setBackground(label.getBackground());
		}

	   final JPanel mpanel = new JPanel(new GridBagLayout());
	   c.set(0, 0, 1, 1, 0, 0, GridBagConstraints.VERTICAL);
	   mpanel.add(label, c);
	   c.set(1, 0, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   mpanel.add(mlabel, c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());
	   c.set(0, 0, 1, 1, 1, 1, GridBagConstraints.BOTH);
	   panel.add(new NoBorderScrollPane(mpanel), c);
	   c.set(0, 1, 1, 1, 1, 0, GridBagConstraints.HORIZONTAL);
	   panel.add(new HSeparatorPanel(ok), c);

	   // finish
	   setContentPane(panel);
	   }

	public void setVisible(boolean bool)
	   {
	   if (bool)
		{
		ok.requestFocus();
		}
	   super.setVisible(bool);
	   }
	}

   private final static class NoBorderScrollPane extends JScrollPane
	{
	public NoBorderScrollPane(JComponent c)
	   {
	   super(c);
	   setBorder(BorderFactory.createEmptyBorder());
	   }
	}

   private static class QuestionDialog extends HDialog
	{
	private int answer = -1;
	private JButton[] buttons = null;
	private int defaultAnswer = 0;

	public QuestionDialog(JDialog parent, String title, String question, String[] answers, Icon[] buttonIcons, int defaultAnswer, Icon icon)
	   {
	   super(parent, title);
	   init(question, answers, buttonIcons, defaultAnswer, icon);
	   }

	public QuestionDialog(JFrame parent, String title, String question, String[] answers, Icon[] buttonIcons, int defaultAnswer, Icon icon)
	   {
	   super(parent, title);
	   init(question, answers, buttonIcons, defaultAnswer, icon);
	   }

	public QuestionDialog(JInternalFrame parent, String title, String question, String[] answers, Icon[] buttonIcons, int defaultAnswer, Icon icon)
	   {
	   super(parent, title);
	   init(question, answers, buttonIcons, defaultAnswer, icon);
	   }

	public int getAnswer()
	   {
	   return answer;
	   }

	private void init(String question, String[] answers, Icon[] icons, int defaultAnswer, Icon icon)
	   {
	   this.defaultAnswer = defaultAnswer;

	   // create buttons
	   buttons = new JButton[answers.length];
	   for (int i = 0; i < answers.length; ++i)
		{
		buttons[i] = new JButton(answers[i]);
		if (i == defaultAnswer)
		   {
		   setDefaultButton(buttons[i]);
		   }
		if (icons != null && icons[i] != null)
		   {
		   buttons[i].setIcon(icons[i]);
		   }
		final int fi = i;
		buttons[i].addActionListener(new ActionListener()
		   {
		   public void actionPerformed(ActionEvent e)
			{
			answer = fi;
			setVisible(false);
			}
		   });
		}

	   // get the constraints
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   // create message panel
	   final JPanel mpanel = new JPanel(new GridBagLayout());
	   final JLabel label = new JLabel(icon);
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);
	   final HMultiLineLabel mlabel = new HMultiLineLabel(question, HMultiLineLabel.LEFT);
	   mlabel.setFont(UIPreferences.getMessageFont());
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.VERTICAL;
	   mpanel.add(label, c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 10;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   mpanel.add(mlabel, c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());

	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(new NoBorderScrollPane(mpanel), c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HSeparatorPanel(buttons), c);

	   // finish
	   this.setContentPane(panel);
	   }

	public void setVisible(boolean bool)
	   {
	   if (bool)
		{
		buttons[defaultAnswer].requestFocus();
		}
	   super.setVisible(bool);
	   }
	}

   private static class YesNoDialog extends HDialog
	{
	private JButton yes = new JButton(JReqTools.getResourceString("Yes"), UIManager.getIcon(UIPreferences.BUTTON_ICON_OK));
	private JButton no = new JButton(JReqTools.getResourceString("No"), UIManager.getIcon(UIPreferences.BUTTON_ICON_CANCEL));
	private boolean answer = false;

	public YesNoDialog(JDialog parent, String title, String q, String y, String n, Icon icon)
	   {
	   super(parent, title);
	   init(q, y, n, icon);
	   }

	public YesNoDialog(JFrame parent, String title, String q, String y, String n, Icon icon)
	   {
	   super(parent, title);
	   init(q, y, n, icon);
	   }

	public YesNoDialog(JInternalFrame parent, String title, String q, String y, String n, Icon icon)
	   {
	   super(parent, title);
	   init(q, y, n, icon);
	   }

	public boolean getAnswer()
	   {
	   return answer;
	   }

	private void init(String q, String y, String n, Icon icon)
	   {
	   // init components
	   yes.setText(y);
	   no.setText(n);
	   setDefaultButton(yes);

	   // listeners
	   yes.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   answer = true;
		   setVisible(false);
		   }
		});
	   no.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   answer = false;
		   setVisible(false);
		   }
		});

	   // get the constraints
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   // create message panel
	   final JPanel mpanel = new JPanel(new GridBagLayout());
	   final JLabel label = new JLabel(icon);
	   label.setHorizontalAlignment(SwingConstants.LEFT);
	   label.setVerticalAlignment(SwingConstants.TOP);
	   final HMultiLineLabel mlabel = new HMultiLineLabel(q, HMultiLineLabel.LEFT);
	   mlabel.setFont(UIPreferences.getMessageFont());
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.VERTICAL;
	   mpanel.add(label, c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 10;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   mpanel.add(mlabel, c);

	   // create content panel
	   final JPanel panel = new JPanel(new GridBagLayout());
	   panel.setBorder(UIPreferences.getFrameBorder());

	   // add components
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   panel.add(new NoBorderScrollPane(mpanel), c);
	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HSeparatorPanel(yes, no), c);

	   // finish
	   this.setContentPane(panel);
	   }

	public void setVisible(boolean bool)
	   {
	   if (bool)
		{
		no.requestFocus();
		}
	   super.setVisible(bool);
	   }
	}

   private JReqTools()
	{
	}

   private static String createStringFromException(Throwable t)
	{
	if (t == null)
	   {
	   return "";
	   }

	try
	   {
	   final StringWriter sw = new StringWriter();
	   final PrintWriter pw = new PrintWriter(sw);
	   t.printStackTrace(pw);

	   final StringBuffer buf = sw.getBuffer();
	   if (buf == null)
		{
		return "";
		}

	   for (int i = 0; i < buf.length(); ++i)
		{
		if (buf.charAt(i) != '\t')
		   {
		   continue;
		   }
		buf.setCharAt(i, ' ');
		buf.insert(i, "   ");
		i += 3;
		}

	   return buf.toString();
	   }
	catch (Throwable th)
	   {
	   th.printStackTrace(System.err);
	   return "";
	   }
	}

   public static Action getAboutAction(final JComponent parent, final String abouttext)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	return iframe != null
		  ? getAboutAction(iframe, abouttext)
		  : getAboutAction(SwingUtils.getJFrame(parent), abouttext);
	}

   public static Action getAboutAction(final JDialog parent, final String abouttext)
	{
	return new AbstractAction()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		JReqTools.showAboutMessage(parent, abouttext);
		}
	   };
	}

   public static Action getAboutAction(final JFrame parent, final String abouttext)
	{
	return new AbstractAction()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		JReqTools.showAboutMessage(parent, abouttext);
		}
	   };
	}

   public static Action getAboutAction(final JInternalFrame parent, final String abouttext)
	{
	return new AbstractAction()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		JReqTools.showAboutMessage(parent, abouttext);
		}
	   };
	}

   public static String getResourceString(String id)
	{
	try
	   {
	   return resources.getString(id);
	   }
	catch (MissingResourceException x)
	   {
	   System.err.println(x);
	   return "MISSING RESOURCE: " + id;
	   }
	}

   public static String getResourceString(String id, Object... args)
	{
	String res = getResourceString(id);

	try
	   {
	   res = MessageFormat.format(res, args);
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	return res;
	}

   static final void packAndTestSize(JDialog d, int minWidth, int minHeight)
	{
	d.setMinimumSize(new Dimension(minWidth, minHeight));
	d.pack();

	// too big ??
	final Dimension screen = d.getToolkit().getScreenSize();
	if (d.getWidth() >= screen.width)
	   {
	   d.setSize(screen.width, d.getHeight());
	   }
	if (d.getHeight() >= screen.height)
	   {
	   d.setSize(d.getWidth(), screen.height);
	   }

	// set position
	d.setLocation((screen.width - d.getWidth()) / 2, (screen.height - d.getHeight()) / 2);
	if (d.getParent() != null)
	   {
	   SwingUtils.centerOn(d.getParent(), d);
	   }
	}

   public static byte requestByte(JComponent parent, String title, String message, byte defaultValue)
	{
	final ByteRequester r = new ByteRequester(title, message, defaultValue);
	r.request(parent);
	return r.getByte();
	}

   public static byte requestByte(JDialog parent, String title, String message, byte defaultValue)
	{
	final ByteRequester r = new ByteRequester(title, message, defaultValue);
	r.request(parent);
	return r.getByte();
	}

   public static byte requestByte(JFrame parent, String title, String message, byte defaultValue)
	{
	final ByteRequester r = new ByteRequester(title, message, defaultValue);
	r.request(parent);
	return r.getByte();
	}

   public static double requestDouble(JComponent parent, String title, String message, double defaultValue)
	{
	final DoubleRequester r = new DoubleRequester(title, message, defaultValue);
	r.request(parent);
	return r.getDouble();
	}

   public static double requestDouble(JDialog parent, String title, String message, double defaultValue)
	{
	final DoubleRequester r = new DoubleRequester(title, message, defaultValue);
	r.request(parent);
	return r.getDouble();
	}

   public static double requestDouble(JFrame parent, String title, String message, double defaultValue)
	{
	final DoubleRequester r = new DoubleRequester(title, message, defaultValue);
	r.request(parent);
	return r.getDouble();
	}

   public static float requestFloat(JComponent parent, String title, String message, float defaultValue)
	{
	final FloatRequester r = new FloatRequester(title, message, defaultValue);
	r.request(parent);
	return r.getFloat();
	}

   public static float requestFloat(JDialog parent, String title, String message, float defaultValue)
	{
	final FloatRequester r = new FloatRequester(title, message, defaultValue);
	r.request(parent);
	return r.getFloat();
	}

   public static float requestFloat(JFrame parent, String title, String message, float defaultValue)
	{
	final FloatRequester r = new FloatRequester(title, message, defaultValue);
	r.request(parent);
	return r.getFloat();
	}

   public static Icon requestIcon(JComponent parent, String title, String message, Icon[] icons, Icon defaultIcon)
	{
	final IconRequester r = new IconRequester(title, message, icons, defaultIcon);
	r.request(parent);
	return r.getIcon();
	}

   public static Icon requestIcon(JDialog parent, String title, String message, Icon[] icons, Icon defaultIcon)
	{
	final IconRequester r = new IconRequester(title, message, icons, defaultIcon);
	r.request(parent);
	return r.getIcon();
	}

   public static Icon requestIcon(JFrame parent, String title, String message, Icon[] icons, Icon defaultIcon)
	{
	final IconRequester r = new IconRequester(title, message, icons, defaultIcon);
	r.request(parent);
	return r.getIcon();
	}

   public static int requestInteger(JComponent parent, String title, String message, int defaultValue)
	{
	final IntegerRequester r = new IntegerRequester(title, message, defaultValue);
	r.request(parent);
	return r.getInteger();
	}

   public static int requestInteger(JDialog parent, String title, String message, int defaultValue)
	{
	final IntegerRequester r = new IntegerRequester(title, message, defaultValue);
	r.request(parent);
	return r.getInteger();
	}

   public static int requestInteger(JFrame parent, String title, String message, int defaultValue)
	{
	final IntegerRequester r = new IntegerRequester(title, message, defaultValue);
	r.request(parent);
	return r.getInteger();
	}

   public static long requestLong(JComponent parent, String title, String message, long defaultValue)
	{
	final LongRequester r = new LongRequester(title, message, defaultValue);
	r.request(parent);
	return r.getLong();
	}

   public static long requestLong(JDialog parent, String title, String message, long defaultValue)
	{
	final LongRequester r = new LongRequester(title, message, defaultValue);
	r.request(parent);
	return r.getLong();
	}

   public static long requestLong(JFrame parent, String title, String message, long defaultValue)
	{
	final LongRequester r = new LongRequester(title, message, defaultValue);
	r.request(parent);
	return r.getLong();
	}

   public static Object requestObject(JComponent parent, String title, String message, Object[] selectableValues)
	{
	return requestObject(parent, title, message, selectableValues, null);
	}

   public static Object requestObject(JDialog parent, String title, String message, Object[] selectableValues)
	{
	return requestObject(parent, title, message, selectableValues, null);
	}

   public static Object requestObject(JFrame parent, String title, String message, Object[] selectableValues)
	{
	return requestObject(parent, title, message, selectableValues, null);
	}

   public static Object requestObject(JComponent parent, String title, String message, Object[] selectableValues, Object selectedValue)
	{
	final SelectRequester r = new SelectRequester(title, message, selectableValues, selectedValue);
	if (r.request(parent) != SelectRequester.OK)
	   {
	   return null;
	   }
	return r.getSelectedValue();
	}

   public static Object requestObject(JDialog parent, String title, String message, Object[] selectableValues, Object selectedValue)
	{
	final SelectRequester r = new SelectRequester(title, message, selectableValues, selectedValue);
	if (r.request(parent) != SelectRequester.OK)
	   {
	   return null;
	   }
	return r.getSelectedValue();
	}

   public static Object requestObject(JFrame parent, String title, String message, Object[] selectableValues, Object selectedValue)
	{
	final SelectRequester r = new SelectRequester(title, message, selectableValues, selectedValue);
	if (r.request(parent) != SelectRequester.OK)
	   {
	   return null;
	   }
	return r.getSelectedValue();
	}

   public static char[] requestPassword(JComponent parent, String title, String message)
	{
	final PasswordRequester r = new PasswordRequester(title, message);
	if (r.request(parent) != PasswordRequester.OK)
	   {
	   return null;
	   }
	return r.getPassword();
	}

   public static char[] requestPassword(JDialog parent, String title, String message)
	{
	final PasswordRequester r = new PasswordRequester(title, message);
	if (r.request(parent) != PasswordRequester.OK)
	   {
	   return null;
	   }
	return r.getPassword();
	}

   public static char[] requestPassword(JFrame parent, String title, String message)
	{
	final PasswordRequester r = new PasswordRequester(title, message);
	if (r.request(parent) != PasswordRequester.OK)
	   {
	   return null;
	   }
	return r.getPassword();
	}

   public static short requestShort(JComponent parent, String title, String message, short defaultValue)
	{
	final ShortRequester r = new ShortRequester(title, message, defaultValue);
	r.request(parent);
	return r.getShort();
	}

   public static short requestShort(JDialog parent, String title, String message, short defaultValue)
	{
	final ShortRequester r = new ShortRequester(title, message, defaultValue);
	r.request(parent);
	return r.getShort();
	}

   public static short requestShort(JFrame parent, String title, String message, short defaultValue)
	{
	final ShortRequester r = new ShortRequester(title, message, defaultValue);
	r.request(parent);
	return r.getShort();
	}

   public static String requestString(JComponent parent, String title, String message, String defaultValue)
	{
	final StringRequester r = new StringRequester(title, message, defaultValue);
	if (r.request(parent) != StringRequester.OK)
	   {
	   return null;
	   }
	return r.getString();
	}

   public static String requestString(JDialog parent, String title, String message, String defaultValue)
	{
	final StringRequester r = new StringRequester(title, message, defaultValue);
	if (r.request(parent) != StringRequester.OK)
	   {
	   return null;
	   }
	return r.getString();
	}

   public static String requestString(JFrame parent, String title, String message, String defaultValue)
	{
	final StringRequester r = new StringRequester(title, message, defaultValue);
	if (r.request(parent) != StringRequester.OK)
	   {
	   return null;
	   }
	return r.getString();
	}

   public static String requestText(JComponent parent, String title, String message, String defaultValue)
	{
	final TextRequester r = new TextRequester(title, message, defaultValue);
	if (r.request(parent) != TextRequester.OK)
	   {
	   return null;
	   }
	return r.getText();
	}

   public static String requestText(JDialog parent, String title, String message, String defaultValue)
	{
	final TextRequester r = new TextRequester(title, message, defaultValue);
	if (r.request(parent) != TextRequester.OK)
	   {
	   return null;
	   }
	return r.getText();
	}

   public static String requestText(JFrame parent, String title, String message, String defaultValue)
	{
	final TextRequester r = new TextRequester(title, message, defaultValue);
	if (r.request(parent) != TextRequester.OK)
	   {
	   return null;
	   }
	return r.getText();
	}

   public static void showAboutMessage(JComponent parent, String message)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showAboutMessage(iframe, message);
	   }
	else
	   {
	   showAboutMessage(SwingUtils.getJFrame(parent), message);
	   }
	}

   public static void showAboutMessage(JDialog parent, String message)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("About"), message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ABOUT));
	d.setVisible(true);
	}

   public static void showAboutMessage(JFrame parent, String message)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("About"), message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ABOUT));
	d.setVisible(true);
	}

   public static void showAboutMessage(JInternalFrame parent, String message)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("About"), message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ABOUT));
	d.setVisible(true);
	}

   public static void showAboutMessage(JComponent parent, String title, String message)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showAboutMessage(iframe, title, message);
	   }
	else
	   {
	   showAboutMessage(SwingUtils.getJFrame(parent), title, message);
	   }
	}

   public static void showAboutMessage(JDialog parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ABOUT));
	d.setVisible(true);
	}

   public static void showAboutMessage(JFrame parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ABOUT));
	d.setVisible(true);
	}

   public static void showAboutMessage(JInternalFrame parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ABOUT));
	d.setVisible(true);
	}

   public static void showError(JComponent parent, String message)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showError(iframe, message);
	   }
	else
	   {
	   showError(SwingUtils.getJFrame(parent), message);
	   }
	}

   public static void showError(JDialog parent, String message)
	{
	showError(parent, getResourceString("Error"), message);
	}

   public static void showError(JFrame parent, String message)
	{
	showError(parent, getResourceString("Error"), message);
	}

   public static void showError(JInternalFrame parent, String message)
	{
	showError(parent, getResourceString("Error"), message);
	}

   public static void showError(JComponent parent, String title, String message)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showError(iframe, title, message);
	   }
	else
	   {
	   showError(SwingUtils.getJFrame(parent), title, message);
	   }
	}

   public static void showError(JDialog parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ERROR));
	d.setVisible(true);
	}

   public static void showError(JFrame parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ERROR));
	d.setVisible(true);
	}

   public static void showError(JInternalFrame parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ERROR));
	d.setVisible(true);
	}

   public static void showException(JComponent parent, Throwable throwable)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showException(iframe, throwable);
	   }
	else
	   {
	   showException(SwingUtils.getJFrame(parent), throwable);
	   }
	}

   public static void showException(JDialog parent, Throwable throwable)
	{
	showException(parent, getResourceString("Exception"), throwable);
	}

   public static void showException(JFrame parent, Throwable throwable)
	{
	showException(parent, getResourceString("Exception"), throwable);
	}

   public static void showException(JInternalFrame parent, Throwable throwable)
	{
	showException(parent, getResourceString("Exception"), throwable);
	}

   public static void showException(JComponent parent, String title, Throwable throwable)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showException(iframe, title, throwable);
	   }
	else
	   {
	   showException(SwingUtils.getJFrame(parent), title, throwable);
	   }
	}

   public static void showException(JDialog parent, String title, Throwable throwable)
	{
	final ExceptionDialog d = new ExceptionDialog(parent, title, throwable, UIManager.getIcon(UIPreferences.DIALOG_ICON_EXCEPTION));
	d.setVisible(true);
	}

   public static void showException(JFrame parent, String title, Throwable throwable)
	{
	final ExceptionDialog d = new ExceptionDialog(parent, title, throwable, UIManager.getIcon(UIPreferences.DIALOG_ICON_EXCEPTION));
	d.setVisible(true);
	}

   public static void showException(JInternalFrame parent, String title, Throwable throwable)
	{
	final ExceptionDialog d = new ExceptionDialog(parent, title, throwable, UIManager.getIcon(UIPreferences.DIALOG_ICON_EXCEPTION));
	d.setVisible(true);
	}

   public static void showMessage(JComponent parent, String message)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showMessage(iframe, message);
	   }
	else
	   {
	   showMessage(SwingUtils.getJFrame(parent), message);
	   }
	}

   public static void showMessage(JDialog parent, String message)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("Message"), message, UIManager.getIcon(UIPreferences.DIALOG_ICON_MESSAGE));
	d.setVisible(true);
	}

   public static void showMessage(JFrame parent, String message)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("Message"), message, UIManager.getIcon(UIPreferences.DIALOG_ICON_MESSAGE));
	d.setVisible(true);
	}

   public static void showMessage(JInternalFrame parent, String message)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("Message"), message, UIManager.getIcon(UIPreferences.DIALOG_ICON_MESSAGE));
	d.setVisible(true);
	}

   public static void showMessage(JComponent parent, String title, String message)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showMessage(iframe, title, message);
	   }
	else
	   {
	   showMessage(SwingUtils.getJFrame(parent), title, message);
	   }
	}

   public static void showMessage(JDialog parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_MESSAGE));
	d.setVisible(true);
	}

   public static void showMessage(JFrame parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_MESSAGE));
	d.setVisible(true);
	}

   public static void showMessage(JInternalFrame parent, String title, String message)
	{
	final MessageDialog d = new MessageDialog(parent, title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_MESSAGE));
	d.setVisible(true);
	}

   /**
    * Pops up a dialog that tells the user that the request funtionality has not yet been implemented.
    */
   public static void showNotYetImplemented(JComponent parent)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   showNotYetImplemented(iframe);
	   }
	else
	   {
	   showNotYetImplemented(SwingUtils.getJFrame(parent));
	   }
	}

   /**
    * Pops up a dialog that tells the user that the request funtionality has not yet been implemented.
    */
   public static void showNotYetImplemented(JDialog parent)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("Sorry"), getResourceString("NotYetImplemented"), UIManager.getIcon(UIPreferences.DIALOG_ICON_NOTYETIMPLEMENTED));
	d.setVisible(true);
	}

   /**
    * Pops up a dialog that tells the user that the request funtionality has not yet been implemented.
    */
   public static void showNotYetImplemented(JFrame parent)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("Sorry"), getResourceString("NotYetImplemented"), UIManager.getIcon(UIPreferences.DIALOG_ICON_NOTYETIMPLEMENTED));
	d.setVisible(true);
	}

   /**
    * Pops up a dialog that tells the user that the request funtionality has not yet been implemented.
    */
   public static void showNotYetImplemented(JInternalFrame parent)
	{
	final MessageDialog d = new MessageDialog(parent, getResourceString("Sorry"), getResourceString("NotYetImplemented"), UIManager.getIcon(UIPreferences.DIALOG_ICON_NOTYETIMPLEMENTED));
	d.setVisible(true);
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent component
    * @param	question the question to answer
    */
   public static boolean showQuestion(JComponent parent, String question)
	{
	return showQuestion(parent, question, getResourceString("Yes"), getResourceString("No"));
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent dialog
    * @param	question the question to answer
    */
   public static boolean showQuestion(JDialog parent, String question)
	{
	return showQuestion(parent, question, getResourceString("Yes"), getResourceString("No"));
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent frame
    * @param	question the question to answer
    */
   public static boolean showQuestion(JFrame parent, String question)
	{
	return showQuestion(parent, question, getResourceString("Yes"), getResourceString("No"));
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent component
    * @param	title the title for the dialog
    * @param	question the question to answer
    */
   public static boolean showQuestion(JComponent parent, String title, String question)
	{
	return showQuestion(parent, title, question, getResourceString("Yes"), getResourceString("No"));
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent dialog
    * @param	title the title for the dialog
    * @param	question the question to answer
    */
   public static boolean showQuestion(JDialog parent, String title, String question)
	{
	return showQuestion(parent, title, question, getResourceString("Yes"), getResourceString("No"));
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent frame
    * @param	title the title for the dialog
    * @param	question the question to answer
    */
   public static boolean showQuestion(JFrame parent, String title, String question)
	{
	return showQuestion(parent, title, question, getResourceString("Yes"), getResourceString("No"));
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent component
    * @param	question the question to answer
    * @param	yesButton the text for the yes-button
    * @param	noButton the text for the no-button
    */
   public static boolean showQuestion(JComponent parent, String question, String yesButton, String noButton)
	{
	return showQuestion(parent, getResourceString("Question"), question, yesButton, noButton);
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent dialog
    * @param	question the question to answer
    * @param	yesButton the text for the yes-button
    * @param	noButton the text for the no-button
    */
   public static boolean showQuestion(JDialog parent, String question, String yesButton, String noButton)
	{
	return showQuestion(parent, getResourceString("Question"), question, yesButton, noButton);
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent frame
    * @param	question the question to answer
    * @param	yesButton the text for the yes-button
    * @param	noButton the text for the no-button
    */
   public static boolean showQuestion(JFrame parent, String question, String yesButton, String noButton)
	{
	return showQuestion(parent, getResourceString("Question"), question, yesButton, noButton);
	}

   /**
    * Pops up a dialog that tells the user that the request funtionality has not yet been implemented.
    */
   public static boolean showQuestion(JComponent parent, String title, String question, String yes, String no)
	{
	final JInternalFrame iframe = SwingUtils.getJInternalFrame(parent);

	if (iframe != null)
	   {
	   return showQuestion(iframe, title, question, yes, no);
	   }
	else
	   {
	   return showQuestion(SwingUtils.getJFrame(parent), title, question, yes, no);
	   }
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent dialog
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	yesButton the text for the yes-button
    * @param	noButton the text for the no-button
    */
   public static boolean showQuestion(JDialog parent, String title, String question, String yesButton, String noButton)
	{
	final YesNoDialog d = new YesNoDialog(parent, title, question, yesButton, noButton, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent frame
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	yesButton the text for the yes-button
    * @param	noButton the text for the no-button
    */
   public static boolean showQuestion(JFrame parent, String title, String question, String yesButton, String noButton)
	{
	final YesNoDialog d = new YesNoDialog(parent, title, question, yesButton, noButton, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent component
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	yesButton the text for the yes-button
    * @param	noButton the text for the no-button
    */
   public static boolean showQuestion(JInternalFrame parent, String title, String question, String yesButton, String noButton)
	{
	final YesNoDialog d = new YesNoDialog(parent, title, question, yesButton, noButton, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent dialog
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	answers the possible answers
    * @param	defaultAnswer the index of the pre-selected answer
    */
   public static int showQuestion(JDialog parent, String title, String question, String[] answers, int defaultAnswer)
	{
	final QuestionDialog d = new QuestionDialog(parent, title, question, answers, null, defaultAnswer, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent frame
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	answers the possible answers
    * @param	defaultAnswer the index of the pre-selected answer
    */
   public static int showQuestion(JFrame parent, String title, String question, String[] answers, int defaultAnswer)
	{
	final QuestionDialog d = new QuestionDialog(parent, title, question, answers, null, defaultAnswer, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent component
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	answers the possible answers
    * @param	defaultAnswer the index of the pre-selected answer
    */
   public static int showQuestion(JInternalFrame parent, String title, String question, String[] answers, int defaultAnswer)
	{
	final QuestionDialog d = new QuestionDialog(parent, title, question, answers, null, defaultAnswer, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent dialog
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	answers the possible answers
    * @param	buttonIcons the icons that will be shown at each button
    * @param	defaultAnswer the index of the pre-selected answer
    */
   public static int showQuestion(JDialog parent, String title, String question, String[] answers, Icon[] buttonIcons, int defaultAnswer)
	{
	final QuestionDialog d = new QuestionDialog(parent, title, question, answers, buttonIcons, defaultAnswer, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent frame
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	answers the possible answers
    * @param	buttonIcons the icons that will be shown at each button
    * @param	defaultAnswer the index of the pre-selected answer
    */
   public static int showQuestion(JFrame parent, String title, String question, String[] answers, Icon[] buttonIcons, int defaultAnswer)
	{
	final QuestionDialog d = new QuestionDialog(parent, title, question, answers, buttonIcons, defaultAnswer, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}

   /**
    * Shows a dialog with a question the user should answer.
    *
    * @param	parent the parent component
    * @param	title the title of the dialog
    * @param	question the question to answer
    * @param	answers the possible answers
    * @param	buttonIcons the icons that will be shown at each button
    * @param	defaultAnswer the index of the pre-selected answer
    */
   public static int showQuestion(JInternalFrame parent, String title, String question, String[] answers, Icon[] buttonIcons, int defaultAnswer)
	{
	final QuestionDialog d = new QuestionDialog(parent, title, question, answers, buttonIcons, defaultAnswer, UIManager.getIcon(UIPreferences.DIALOG_ICON_QUESTION));
	d.setVisible(true);
	return d.getAnswer();
	}
   }
