/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
* This is a requester for login purposes. A login name and a password is requested.
* 
* @author	Markus Hillenbrand
*/

public class LoginRequester extends OkCancelRequester {
	private static final boolean DEBUG = false;

	private String		login				= null;
	private char [] 		password 		= null;
	private String		loginLabel			= null;
	private String		passwordLabel		= null;
	
	// components
	private HTextField		loginfield 			= new HTextField();
	private JPasswordField 	passwordfield 		= new JPasswordField();
	
	public LoginRequester(String message)  {
		this("JReqTools: LoginRequester", message, JReqTools.getResourceString("Login"), JReqTools.getResourceString("Password"));
	}

	public LoginRequester(String title, String message)  {
		this (title, message, JReqTools.getResourceString("Login"), JReqTools.getResourceString("Password"));
	}

	public LoginRequester(String message, String loginLabel, String passwordLabel)  {
		this("JReqTools: LoginRequester", message, loginLabel, passwordLabel);
	}

	public LoginRequester(String title, String message, String loginLabel, String passwordLabel)  {
		super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_REQUEST_LOGIN));
		
		// init variables
		this.loginLabel = loginLabel;
		this.passwordLabel = passwordLabel;
		
		// listeners
		loginfield.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { passwordfield.requestFocus(); } });
	}

	public String getLogin()  {
		return login;
	}

	public JComponent getOkTriggerComponent()  {
		return passwordfield;
	}

	public char [] getPassword()  {
		return password;
	}

	protected JPanel getUserPanel()  {
		// get default constraints
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// create panel
		final JPanel panel = new JPanel(new GridBagLayout());
		
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(new JLabel(loginLabel, JLabel.LEFT), c);
		c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(new JLabel(passwordLabel, JLabel.LEFT), c);
		
		c.gridx = 1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(loginfield, c);
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(passwordfield, c);
		
		return panel;
	}

	protected void ok()  {
		login = loginfield.getText(); 
		password = passwordfield.getPassword();
	}

	protected void prepareForRequest()  {
		loginfield.setText(login);
		passwordfield.setText("");
	}

	public void setLogin(String text)  {
		login = text;
	}

}
