/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;

public abstract class OkCancelFrame extends HFrame {
	private static final boolean DEBUG = false;

	// components
	private JPanel contentPane = null;
	final protected JButton ok = new JButton(JReqTools.getResourceString("Ok"), UIManager.getIcon(UIPreferences.BUTTON_ICON_OK));
	final protected JButton cancel = new JButton(JReqTools.getResourceString("Cancel"), UIManager.getIcon(UIPreferences.BUTTON_ICON_CANCEL));
	
	// Property: icon
	private Icon icon = null;
	// Property: message
	private String message = "";
	
	public OkCancelFrame(String title, String message)  {
		this(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ENTERDATA));
	}

	public OkCancelFrame(String title, String message, Icon icon)  {
		super(title);
		
		this.message = message;
		this.icon = icon;
	}

	protected void cancel()  {
	}

	private JPanel createContentPane()  {
		// get user panel
		final JPanel userPanel = getUserPanel();
		
		// create components
		final HMultiLineLabel mlabel = new HMultiLineLabel(message, HMultiLineLabel.LEFT);
		mlabel.setFont(UIPreferences.getMessageFont());
		
		// listeners
		ok.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { ok(); } });
		cancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { cancel(); } });
		this.addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { cancel(); } });
		
		// get default constraints
		GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// create content panel
		final JPanel panel = new JPanel(new GridBagLayout());
		panel.setBorder(UIPreferences.getFrameBorder());
		
		// create icon label
		final JLabel label = new JLabel(icon);
		label.setHorizontalAlignment(SwingConstants.LEFT);
		label.setVerticalAlignment(SwingConstants.TOP);
		
		// add components
		if (message != null)
			{
			c.gridx = 1; c.gridy=0; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
			panel.add(new HSeparator(), c);
			c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.NONE; 
			panel.add(label, c);
			c.gridx = 1; c.gridy=1; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
			panel.add(mlabel, c);
			c.gridx = 1; c.gridy=2; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
			panel.add(new HSeparator(), c);
			}
		
		c.insets = new Insets(0,0,0,0);
		c.gridx = 1; c.gridy=3; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=10; c.fill = GridBagConstraints.BOTH; 
		panel.add(userPanel, c);
		
		// restore default constraints
		c = UIPreferences.getGridBagConstraints();
		
		c.gridx = 0; c.gridy=4; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(new HSeparatorPanel(ok, cancel), c);
		
		getRootPane().setDefaultButton(ok);
		
		return panel;
	}

	/**
	* Returns the value of the message property.
	*/

	public String getMessage()  {
		return message;
	}

	protected abstract JPanel getUserPanel() ;

	protected void ok()  {
	}

	/**
	* Sets the text of the "Cancel" button.
	*/

	public void setCancelButtonText(String text)  {
		cancel.setText(text);
	}

	/**
	* Assigns the value of the message property.
	*/

	public void setMessage(String arg)  {
		message = arg;
	}

	/**
	* Sets the text of the "Ok" button.
	*/

	public void setOkButtonText(String text)  {
		ok.setText(text);
	}

	public void setVisible(boolean b)  {
		// create content pane 
		if (b && contentPane == null) 
			{
			contentPane = createContentPane();
			this.setContentPane(contentPane);
			Dimension size = getSize();
			if (size == null || size.width == 0 || size.height == 0) this.pack();
			}
		
		// call super class
		super.setVisible(b);
		
		// bring to front
		toFront();
	}

}
