/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

public abstract class OkCancelRequester extends AbstractRequester
   {
   // variables
   protected int answer = CANCEL;
   protected String comment = null;
   // components
   final protected JButton ok = new JButton(JReqTools.getResourceString("Ok"), UIManager.getIcon(UIPreferences.BUTTON_ICON_OK));
   final protected JButton cancel = new JButton(JReqTools.getResourceString("Cancel"), UIManager.getIcon(UIPreferences.BUTTON_ICON_CANCEL));
   // listeners
   private final KeyAdapter escapeListener = new KeyAdapter()
	{
	public void keyTyped(KeyEvent e)
	   {
	   if (e.getKeyChar() == KeyEvent.VK_ESCAPE)
		{
		cancelPressed();
		}
	   }
	};
   private final ActionListener actionListener = new ActionListener()
	{
	public void actionPerformed(ActionEvent e)
	   {
	   okPressed();
	   }
	};
   private final MouseListener mouseListener = new MouseAdapter()
	{
	public void mouseClicked(MouseEvent e)
	   {
	   if (e.getClickCount() < 2)
		{
		return;
		}
	   if (e.getSource() instanceof JTree)
		{
		// only a double click on leafs triggers !!
		final TreePath path = ((JTree) e.getSource()).getSelectionPath();
		if (path == null)
		   {
		   return;
		   }
		Object[] objects = path.getPath();
		if (!((TreeNode) objects[objects.length - 1]).isLeaf())
		   {
		   return;
		   }
		}
	   okPressed();
	   }
	};
   private final KeyListener keyListener = new KeyAdapter()
	{
	public void keyTyped(KeyEvent e)
	   {
	   if (e.getKeyChar() == KeyEvent.VK_ENTER)
		{
		okPressed();
		}
	   else if (e.getKeyChar() == KeyEvent.VK_ESCAPE)
		{
		cancelPressed();
		}
	   }
	};
   // Property: icon
   protected Icon icon;

   public OkCancelRequester(String title, String message)
	{
	this(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ENTERDATA));
	}

   public OkCancelRequester(String title, String message, Icon icon)
	{
	super(title, message);

	this.icon = icon;
	}
   protected boolean drawFirstSeparator = true;

   public boolean isDrawFirstSeparator()
	{
	return drawFirstSeparator;
	}

   public void setDrawFirstSeparator(boolean drawFirstSeparator)
	{
	this.drawFirstSeparator = drawFirstSeparator;
	}
   protected boolean drawSecondSeparator = true;

   public boolean isDrawSecondSeparator()
	{
	return drawSecondSeparator;
	}

   public void setDrawSecondSeparator(boolean drawSecondSeparator)
	{
	this.drawSecondSeparator = drawSecondSeparator;
	}

   public void setDrawSeparators(boolean drawSeparators)
	{
	drawFirstSeparator = drawSeparators;
	drawSecondSeparator = drawSeparators;
	}

   protected void cancel()
	{
	}

   private void cancelPressed()
	{
	answer = CANCEL;
	cancel();
	setVisible(false);
	}

   protected JPanel createContentPane()
	{
	// set variables
	final JPanel userPanel = getUserPanel();
	final JComponent okTrigger = getOkTriggerComponent();

	// create components
	final HMultiLineLabel mlabel = new HMultiLineLabel(message, HMultiLineLabel.LEFT);
	mlabel.setFont(UIPreferences.getMessageFont());

	// listeners
	ok.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		okPressed();
		}
	   });
	cancel.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		cancelPressed();
		}
	   });

	// add ESC key listener
	final Vector<Component> containers = new Vector<Component>();
	containers.add(userPanel);
	while (containers.size() > 0)
	   {
	   final Container c = (Container) containers.elementAt(0);
	   containers.remove(0);
	   for (int i = 0; i < c.getComponentCount(); ++i)
		{
		Component comp = c.getComponent(i);
		if (comp instanceof JPanel)
		   {
		   containers.add(comp);
		   }
		else if (comp instanceof JComponent)
		   {
		   comp.removeKeyListener(escapeListener);
		   comp.addKeyListener(escapeListener);
		   }
		}
	   }

	// add the last component's listener
	if (okTrigger != null && okTrigger instanceof JTextField)
	   {
	   ((JTextField) okTrigger).removeActionListener(actionListener);
	   ((JTextField) okTrigger).addActionListener(actionListener);
	   }
	else if (okTrigger != null && okTrigger instanceof JComboBox)
	   {
	   final JComboBox cb = (JComboBox) okTrigger;
	   if (cb.isEditable())
		{
		cb.getEditor().getEditorComponent().removeKeyListener(keyListener);
		cb.getEditor().getEditorComponent().addKeyListener(keyListener);
		}
	   else
		{
		cb.removeActionListener(actionListener);
		cb.addActionListener(actionListener);
		}
	   }
	else if (okTrigger != null && (okTrigger instanceof JList || okTrigger instanceof JTree))
	   {
	   okTrigger.removeMouseListener(mouseListener);
	   okTrigger.addMouseListener(mouseListener);
	   okTrigger.removeKeyListener(keyListener);
	   okTrigger.addKeyListener(keyListener);
	   }
	else if (okTrigger != null)
	   {
	   System.err.println(getClass().getName() + ": okTrigger " + okTrigger.getClass().getName() + " not supported");
	   }

	// get default constraints
	GridBagConstraints c = UIPreferences.getGridBagConstraints();

	// create content panel
	final JPanel panel = new JPanel(new GridBagLayout());
	panel.setBorder(UIPreferences.getFrameBorder());

	// create icon label
	final JLabel label = new JLabel(icon);
	label.setHorizontalAlignment(SwingConstants.LEFT);
	label.setVerticalAlignment(SwingConstants.TOP);

	// add components
	int ycount = 0;
	if (message != null && message.length() > 0)
	   {
	   if (drawFirstSeparator)
		{
		c.gridx = 1;
		c.gridy = ycount++;
		c.gridwidth = 4;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(new HSeparator(), c);
		}
	   c.gridx = 0;
	   c.gridy = ycount;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.NONE;
	   panel.add(label, c);
	   c.gridx = 1;
	   c.gridy = ycount++;
	   c.gridwidth = 3;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(mlabel, c);
	   }

	// add comment if neccessary
	if (comment != null)
	   {
	   c.gridx = 1;
	   c.gridy = ycount++;
	   c.gridwidth = 3;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HMultiLineLabel(comment == null ? "" : comment, HMultiLineLabel.LEFT), c);
	   }

	// and complete with a sepator
	if (ycount > 0 && drawSecondSeparator)
	   {
	   c.gridx = 1;
	   c.gridy = ycount++;
	   c.gridwidth = 4;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   panel.add(new HSeparator(), c);
	   }

	c.insets = new Insets(0, 0, 0, 0);
	c.gridx = 1;
	c.gridy = ycount++;
	c.gridwidth = 3;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 10;
	c.fill = GridBagConstraints.BOTH;
	panel.add(userPanel, c);
	c = UIPreferences.getGridBagConstraints();

	c.gridx = 0;
	c.gridy = ycount++;
	c.gridwidth = 4;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	panel.add(new HSeparatorPanel(ok, cancel), c);

	return panel;
	}

   /**
    * Returns the value of the comment property.
    * @return
    */
   public String getComment()
	{
	return comment;
	}

   protected JButton getDefaultButton()
	{
	return ok;
	}

   /**
    * Returns the value of the icon property.
    * @return
    */
   public Icon getIcon()
	{
	return icon;
	}

   protected JComponent getOkTriggerComponent()
	{
	return null;
	}

   /**
    * Return the preferred size of the dialog's frame to be used when the requester opens for the first time.
    * If it returns null, Dialog.pack() will be used ti determine the preferred size.
    * @return
    */
   protected Dimension getPreferredRequesterSize()
	{
	return null;
	}

   protected int getReturnValue()
	{
	return answer;
	}

   protected abstract JPanel getUserPanel();

   protected void ok()
	{
	}

   private void okPressed()
	{
	// get error message
	final String error = validateData();

	if (error == null)
	   {
	   answer = OK;
	   ok();
	   setVisible(false);
	   }
	else
	   {
	   SwingUtils.beep();
	   showError(error);
	   if (getOkTriggerComponent() != null)
		{
		getOkTriggerComponent().requestFocus();
		}
	   }
	}

   protected void prepareForRequest()
	{
	}

   /**
    * Sets the text of the "Cancel" button.
    * @param text
    */
   public void setCancelButtonText(String text)
	{
	cancel.setText(text);
	}

   /**
    * Assigns the value of the comment property.
    * @param arg
    */
   public void setComment(String arg)
	{
	comment = arg;
	}

   /**
    * Assigns the value of the icon property.
    * @param arg
    */
   public void setIcon(Icon arg)
	{
	icon = arg;
	}

   /**
    * Sets the text of the "Ok" button.
    * @param text
    */
   public void setOkButtonText(String text)
	{
	ok.setText(text);
	}

   public void simulateCancelPressed()
	{
	cancelPressed();
	}

   public void simulateOkPressed()
	{
	okPressed();
	}

   protected String validateData()
	{
	return null;
	}
   }
