/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
* This is a requester for passwords.
* 
* @author	Markus Hillenbrand
*/

public class PasswordRequester extends OkCancelRequester {
	private static final boolean DEBUG = false;

	private char [] password = null;
	
	// components
	private JPasswordField passwordfield = new JPasswordField();
	
	public PasswordRequester(String message)  {
		this("JReqTools: PasswordRequester", message);
	}

	public PasswordRequester(String title, String message)  {
		super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_REQUEST_PASSWORD));
	}

	protected JComponent getOkTriggerComponent()  {
		return passwordfield;
	}

	public char [] getPassword()  {
		return password;
	}

	protected JPanel getUserPanel()  {
		// get default constraints
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// create panel
		final JPanel panel = new JPanel(new GridBagLayout());
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(passwordfield, c);
		
		return panel;
	}

	protected void ok()  {
		password = passwordfield.getPassword();
	}

	protected void prepareForRequest()  {
		passwordfield.setText("");
	}

}
