/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import javax.swing.*;
import de.hillenbrand.swing.*;

public abstract class ProgressTask
   {
   private HDialog dialog;
   private Exception exception;

   public Exception getException()
	{
	return exception;
	}

   public void setException(Exception exception)
	{
	this.exception = exception;
	}

   public void cancel()
	{
	}

   public abstract int getCurrent();

   public abstract String getCurrentText();

   /**
    * Returns the dialog where the progress of the task is shown. If the task has not yet been started, it returns null.
    * You should use this method to show additional dialogs upon the one being shown for this task.
    * @return
    */
   public HDialog getDialog()
	{
	return dialog;
	}

   public abstract int getMaximum();

   public abstract int getMinimum();

   public abstract void run();

   void setDialog(HDialog dialog)
	{
	this.dialog = dialog;
	}

   /**
    * Shows a message to the user. This should be used instead of JReqTools.showMessage(), because only this method knows the real dialog which is being shown.
    * @param message
    */
   public void showError(String message)
	{
	dialog.showMessage(message);
	}

   /**
    * Shows an exception to the user. This should be used instead of JReqTools.showException(), because only this method knows the real dialog which is being shown.
    * @param exc
    */
   public void showException(Exception exc)
	{
	dialog.showException(exc);
	}

   /**
    * Shows a question to the user. This should be used instead of JReqTools.showQuestion(), because only this method knows the real dialog which is being shown.
    * @param message
    * @return
    */
   public boolean showQuestion(String message)
	{
	return dialog.showQuestion(message);
	}

   /**
    * Shows a question to the user. This should be used instead of JReqTools.showQuestion(), because only this method knows the real dialog which is being shown.
    * @param message
    * @param answers
    * @param defaultAnswer
    * @return
    */
   public int showQuestion(String message, String[] answers, int defaultAnswer)
	{
	return dialog.showQuestion(JReqTools.getResourceString("Question"), message, answers, defaultAnswer);
	}

   /**
    * Shows a question to the user. This should be used instead of JReqTools.showQuestion(), because only this method knows the real dialog which is being shown.
    * @param message
    * @param answers
    * @param icons
    * @param defaultAnswer
    * @return
    */
   public int showQuestion(String message, String[] answers, Icon[] icons, int defaultAnswer)
	{
	return dialog.showQuestion(JReqTools.getResourceString("Question"), message, answers, icons, defaultAnswer);
	}
   }
