/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;

/**
 * This is a requester for complete texts.
 *
 * @author	Markus Hillenbrand
 */
public class TextRequester extends OkCancelRequester
   {
   private String text = "";
   // components
   private JTextArea textfield = new JTextArea(text);
   private JScrollPane scrollpane = new JScrollPane(textfield, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

   /**
    * Constructs a new TextRequester.
    *
    * @param	message - The message to be shown in the requester.
    * @param	defaultstring - The initial value of the text field.
    */
   public TextRequester(String message, String defaultstring)
	{
	this("JReqTools: TextRequester", message, defaultstring);
	}

   /**
    * Constructs a new TextRequester.
    *
    * @param	title - Title of the requester.
    * @param	message - The message to be shown in the requester.
    * @param	defaultstring - The initial value of the text field.
    */
   public TextRequester(String title, String message, String defaultstring)
	{
	super(title, message, UIManager.getIcon(UIPreferences.DIALOG_ICON_ENTERDATA));

	textfield.setLineWrap(true);
	textfield.setWrapStyleWord(true);
	setText(defaultstring);
	scrollpane.setPreferredSize(new java.awt.Dimension(200, 200));
	}

   public int getTabSize()
	{
	return textfield.getTabSize();
	}

   /**
    * Gets the string which has been entered by the user.
    * @return
    */
   public String getText()
	{
	return text;
	}

   protected JPanel getUserPanel()
	{
	final HGridBagConstraints c = UIPreferences.getGridBagConstraints();
	final JPanel panel = new JPanel(new GridBagLayout());
	c.set(0, 0, 1, 1, 1, 1, GridBagConstraints.BOTH);
	panel.add(scrollpane, c);
	return panel;
	}

   public boolean getWrapStyleWord()
	{
	return textfield.getWrapStyleWord();
	}

   protected void ok()
	{
	text = textfield.getText();
	}

   protected void prepareForRequest()
	{
	textfield.setText(text);
	}

   /**
    * Sets the line-wrapping policy of the text area. If set to true the lines will be wrapped if they are too long to fit within the allocated width.
    * If set to false, the lines will always be unwrapped. By default this property is true.
    * @param b
    */
   public void setLineWrap(boolean b)
	{
	textfield.setLineWrap(b);
	}

   public void setTabSize(int i)
	{
	textfield.setTabSize(i);
	}

   /**
    * Sets the string in the requester.
    * @param newText
    */
   public final void setText(String newText)
	{
	text = newText;
	}

   /**
    * Sets the style of wrapping used if the text area is wrapping lines. If set to true the lines will be wrapped at word boundaries (whitespace) if they are too long to fit within the allocated width.
    * If set to false, the lines will be wrapped at character boundaries. By default this property is true.
    * @param b
    */
   public void setWrapStyleWord(boolean b)
	{
	textfield.setWrapStyleWord(b);
	}

   public void setFont(Font f)
	{
	textfield.setFont(f);
	}
   }
