/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.jreqtools;

import de.hillenbrand.swing.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class TipOfTheDay extends JFrame {
	private static final boolean DEBUG = false;

	private String []	tips		= null;
	private int			lastTip	= 0;
	
	// components
	private JCheckBox	show 	= new JCheckBox(JReqTools.getResourceString("ShowNextTime"), true);
	private JTextArea	tipArea	= new JTextArea();
	private JLabel 		counter	= new JLabel("999/999");
	
	public TipOfTheDay(String title, String [] tips, int firstTipToShow, boolean showNextTime)  {
		super(title);
		
		// set variables
		this.tips = tips;
		this.lastTip = firstTipToShow;
		
		// create components
		final JButton next = new JButton(HIconFactory.getIcon("arrowRight"));
		final JButton prev = new JButton(HIconFactory.getIcon("arrowLeft"));
		final JButton close = new JButton(JReqTools.getResourceString("CloseTip"), UIManager.getIcon(UIPreferences.BUTTON_ICON_CANCEL));
		final JScrollPane scrollpane = new JScrollPane(tipArea);
		final JLabel titleLabel = new JLabel(JReqTools.getResourceString("TipOfTheDayTitle"), JLabel.LEFT);
		
		// init components
		titleLabel.setFont(UIPreferences.getMessageFont());
		tipArea.setEditable(false);
		scrollpane.setPreferredSize(new Dimension(200,100));
		show.setSelected(showNextTime);
		next.setMargin(new Insets(1,1,1,1));
		prev.setMargin(new Insets(1,1,1,1));
		close.setMaximumSize(close.getPreferredSize());
		// listeners
		close.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { setVisible(false); } });
		next.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) {
			lastTip = (lastTip + 1) % TipOfTheDay.this.tips.length;
			setTip(lastTip);
		}});
		prev.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) {
			lastTip = lastTip-1;
			if (lastTip == -1) lastTip = TipOfTheDay.this.tips.length-1;
			setTip(lastTip);
		}});
		
		// get the constraints
		GridBagConstraints c = UIPreferences.getGridBagConstraints();
		c.insets.left = 0;
		c.insets.right = 0;
		
		// create tip panel
		final JPanel tpanel = new JPanel(new GridBagLayout());
		c.gridx = 0; c.gridy=0; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=10; c.fill = GridBagConstraints.BOTH; 
		tpanel.add(scrollpane, c);
		c.gridx = 0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.NONE; 
		tpanel.add(prev, c);
		c.gridx = 1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		tpanel.add(counter, c);
		c.gridx = 2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.NONE; 
		tpanel.add(next, c);
		c.gridx = 3; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		tpanel.add(new JPanel(), c);
		
		// get the default constraints
		c = UIPreferences.getGridBagConstraints();
		
		// create panel
		final JPanel panel = new JPanel(new GridBagLayout());
		panel.setBorder(UIPreferences.getFrameBorder());
		
		// create icon label
		final JLabel label = new JLabel(UIManager.getIcon(UIPreferences.DIALOG_ICON_TIP));
		label.setHorizontalAlignment(SwingConstants.LEFT);
		label.setVerticalAlignment(SwingConstants.TOP);
		
		// add components
		c.gridx = 0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.NONE; 
		panel.add(label, c);
		c.gridx = 1; c.gridy=0; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(titleLabel, c);
		c.gridx = 1; c.gridy=1; c.gridwidth=3; c.gridheight=1; c.weightx=1; c.weighty=10; c.fill = GridBagConstraints.BOTH; 
		panel.add(tpanel, c);
		c.gridx = 0; c.gridy=2; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL; 
		panel.add(new HSeparatorPanel(show, close), c);
		
		// finish
		setContentPane(panel);
		setTip(firstTipToShow);
		pack();
		if (getWidth() < 400) setSize(400, getHeight());
		
		// center on screen
		SwingUtils.centerOnPrimaryDisplay(this);
	}

	public boolean getShowNextTime()  {
		return show.isSelected();
	}

	public int getTip()  {
		return lastTip;
	}

	public void setShowNextTime(boolean b)  {
		show.setSelected(b);
	}

	public void setTip(int i)  {
		lastTip = i;
		if (i >= tips.length) i=0;
		tipArea.setText(tips[i]);
		counter.setText("" + (i+1) + "/" + tips.length);
	}

	public void setVisible(boolean b)  {
		super.setVisible(b);
		if (b) toFront();
	}

}
