/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.util.ArrayUtils;
import java.lang.reflect.Method;
import java.util.ArrayList;
import javax.swing.*;

/**
 * Extends the standard javax.swing.JComboBox with an undo/redo buffer and a cut & paste menu, if the combo box is editable.
 * @param <T>
 */
public class HComboBox<T extends Object> extends JComboBox
   {
   private final T objectType;

   /*
    * Creates a JComboBox that contains the elements in the specified array.
    * By default the first item in the array (and therefore the data model) becomes selected.
    * @param items
    */
   public HComboBox(T... items)
	{
	super(items);
	if (items.length == 0)
	   {
	   throw new IllegalArgumentException("A HComboBox must have at least one element.");
	   }
	objectType = items[0];
	}

   public HComboBox(ArrayList<T> items)
	{
	super(items.toArray());
	if (items.size() == 0)
	   {
	   throw new IllegalArgumentException("A HComboBox must have at least one element.");
	   }
	objectType = items.get(0);
	}

   @SuppressWarnings("unchecked")
   @Override
   public T getSelectedItem()
	{
	Object result = super.getSelectedItem();

	if (objectType instanceof String)
	   {
	   return (T) result;
	   }

	if (result != null && result instanceof String)
	   {
	   try
		{
		final Class cls = objectType.getClass();
		final Method method = cls.getMethod("valueOf", ArrayUtils.toArray(String.class));
		result = method.invoke(objectType, result);
		}
	   catch (Exception ex)
		{
		}
	   }

	return (T) result;
	}

   public void addItems(T[] items)
	{
	for (T item : items)
	   {
	   addItem(item);
	   }
	}

   public void addItems(ArrayList<T> items)
	{
	for (T item : items)
	   {
	   addItem(item);
	   }
	}

   @Override
   public void setSelectedIndex(int anIndex)
	{
	if (anIndex < 0)
	   {
	   return;
	   }
	super.setSelectedIndex(anIndex);
	}
   }
