/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import javax.swing.*;
import de.hillenbrand.jreqtools.*;

public class HDesktop
   {
   private HDesktop()
	{
	}

   private static Process execWin(String cmd) throws IOException
	{
	return Runtime.getRuntime().exec(cmd);
	}

   private static Process execUnix(String... commands) throws IOException
	{
	for (String command : commands)
	   {
	   try
		{
		return Runtime.getRuntime().exec(command);
		}
	   catch (java.io.IOException x)
		{
		}
	   }
	throw new IOException("Can't find a program to handle your request.");
	}

   /**
    * Edit a file.
    * @param parent
    * @param f
    * @return
    */
   public static Process edit(JComponent parent, File f)
	{
	return edit(SwingUtils.getJFrame(parent), f);
	}

   /**
    * Edit a file.
    * @param parent
    * @param f
    * @return
    */
   public static Process edit(JFrame parent, File f)
	{
	try
	   {
	   if (Desktop.isDesktopSupported() && Desktop.getDesktop().isSupported(Desktop.Action.EDIT))
		{
		Desktop.getDesktop().edit(f);
		}
	   else if (SwingUtils.isWindowsSystem())
		{
		return execWin("rundll32 url.dll,FileProtocolHandler " + f);
		}
	   else
		{
		return execUnix("gedit " + f, "kate " + f);
		}
	   }
	catch (Exception x)
	   {
	   JReqTools.showException(parent, x);
	   }
	return null;
	}

   /**
    * Open an URL in a browser.
    * @param parent
    * @param url
    * @return
    */
   public static Process browse(JComponent parent, String url)
	{
	return browse(SwingUtils.getJFrame(parent), url);
	}

   /**
    * Open an URL in a browser.
    * @param parent
    * @param url
    * @return
    */
   public static Process browse(JFrame parent, String url)
	{
	try
	   {
	   if (Desktop.isDesktopSupported() && Desktop.getDesktop().isSupported(Desktop.Action.BROWSE))
		{
		Desktop.getDesktop().browse(new java.net.URI(url));
		return null;
		}
	   else if (SwingUtils.isWindowsSystem())
		{
		return execWin("rundll32 url.dll,FileProtocolHandler " + url);
		}
	   else
		{
		return execUnix("firefox -remoteURL" + url, "mozilla -remoteURL" + url, "kfmclient exec " + url);
		}
	   }
	catch (Exception x)
	   {
	   JReqTools.showException(parent, x);
	   }
	return null;
	}
   }
