/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import de.hillenbrand.jreqtools.*;

public class HDialog
   {
   // contants
   private final int MINWIDTH = 400;
   private final int MINHEIGHT = 200;
   // the possible parents
   private JFrame jframeParent = null;
   private JDialog jdialogParent = null;
   private JInternalFrame iframeParent = null;
   // the two possible dialog forms
   private JDialog jdialog = null;
   private HInternalDialog hdialog = null;
   // additional parameters
   private JButton defaultButton = null;

   public HDialog(JDialog parent, String title)
	{
	this.jdialogParent = parent;
	this.jdialog = createRealDialog(parent, title);
	}

   public HDialog(JFrame parent, String title)
	{
	this.jframeParent = parent;
	this.jdialog = createRealDialog(parent, title);
	}

   public HDialog(JInternalFrame parent, String title)
	{
	this.iframeParent = parent;
	this.hdialog = createRealDialog(parent, title);
	}

   private HInternalDialog createRealDialog(final JInternalFrame parent, final String title)
	{
	final HInternalDialog result = new HInternalDialog(parent, title);

	result.setClosable(false);
	result.setMaximizable(true);
	result.setIconifiable(false);
	result.setResizable(true);

	result.addSetMaximumListener(new HInternalDialog.SetMaximumListener()
	   {
	   public void setMaximum(boolean b)
		{
		if (!b)
		   {
		   return;
		   }

		final Dimension size = hdialog.getSize();
		final Point location = hdialog.getLocation();
		final JDesktopPane d = SwingUtils.getJDesktopPane(parent);
		if (d != null)
		   {
		   location.x += d.getLocationOnScreen().x;
		   location.y += d.getLocationOnScreen().y;
		   }

		hdialog.setVisible(false);

		jdialog = createRealDialog(SwingUtils.getJFrame(parent), title);
		jdialog.setContentPane(hdialog.getContentPane());

		hdialog.dispose();
		hdialog = null;

		jdialog.setLocation(location);
		jdialog.setSize(size);
		jdialog.setVisible(true);
		}
	   });

	return result;
	}

   private JDialog createRealDialog(JDialog parent, String title)
	{
	final JDialog result = new JDialog(parent, title, true);

	result.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	result.setResizable(true);

	return result;
	}

   private JDialog createRealDialog(JFrame parent, String title)
	{
	final JDialog result = new JDialog(parent, title, true);

	result.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	result.setResizable(true);

	return result;
	}

   public void dispose()
	{
	if (jdialog != null)
	   {
	   jdialog.dispose();
	   }
	else if (hdialog != null)
	   {
	   hdialog.dispose();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public Color getBackground()
	{
	if (jdialog != null)
	   {
	   return jdialog.getBackground();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getBackground();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   private Insets getBorderInsets(Component c)
	{
	if (c instanceof JComponent)
	   {
	   final Border b = ((JComponent) c).getBorder();
	   if (b != null)
		{
		return b.getBorderInsets(c);
		}
	   else
		{
		return new Insets(0, 0, 0, 0);
		}
	   }
	else
	   {
	   return new Insets(0, 0, 0, 0);
	   }
	}

   public Container getContentPane()
	{
	if (jdialog != null)
	   {
	   return jdialog.getContentPane();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getContentPane();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public int getHeight()
	{
	if (jdialog != null)
	   {
	   return jdialog.getHeight();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getHeight();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public LayoutManager getLayout()
	{
	if (jdialog != null)
	   {
	   return jdialog.getLayout();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getLayout();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public Point getLocation()
	{
	if (jdialog != null)
	   {
	   return jdialog.getLocation();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getLocation();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public Point getLocationOnScreen()
	{
	if (jdialog != null)
	   {
	   return jdialog.getLocationOnScreen();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getLocationOnScreen();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public Container getParent()
	{
	if (jframeParent != null)
	   {
	   return jframeParent;
	   }
	else if (jdialogParent != null)
	   {
	   return jdialogParent;
	   }
	else if (iframeParent != null)
	   {
	   return iframeParent;
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   private Rectangle getParentBounds()
	{
	if (jframeParent != null)
	   {
	   return jframeParent.getBounds();
	   }
	else if (jdialogParent != null)
	   {
	   return jdialogParent.getBounds();
	   }
	else if (iframeParent != null)
	   {
	   return iframeParent.getBounds();
	   }
	else
	   {
	   final GraphicsDevice primary = UIPreferences.getPrimaryGraphicsDevice();
	   return primary.getDefaultConfiguration().getBounds();
	   }
	}

   public Dimension getSize()
	{
	if (jdialog != null)
	   {
	   return jdialog.getSize();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getSize();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public String getTitle()
	{
	if (jdialog != null)
	   {
	   return jdialog.getTitle();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getTitle();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public Toolkit getToolkit()
	{
	if (jdialog != null)
	   {
	   return jdialog.getToolkit();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getToolkit();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public int getWidth()
	{
	if (jdialog != null)
	   {
	   return jdialog.getWidth();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getWidth();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public int getX()
	{
	if (jdialog != null)
	   {
	   return jdialog.getX();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getX();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public int getY()
	{
	if (jdialog != null)
	   {
	   return jdialog.getY();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.getY();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public boolean isVisible()
	{
	if (jdialog != null)
	   {
	   return jdialog.isVisible();
	   }
	else if (hdialog != null)
	   {
	   return hdialog.isVisible();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void pack()
	{
	if (jdialog != null)
	   {
	   jdialog.pack();
	   }
	else if (hdialog != null)
	   {
	   hdialog.pack();
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setContentPane(Container contentPane)
	{
	if (jdialog != null)
	   {
	   jdialog.setContentPane(contentPane);
	   jdialog.setMinimumSize(new Dimension(MINWIDTH, MINHEIGHT));
	   jdialog.getRootPane().setDefaultButton(defaultButton);
	   jdialog.pack();
	   final Dimension dialogSize = jdialog.getPreferredSize();
	   if (dialogSize.width < MINWIDTH)
		{
		dialogSize.width = MINWIDTH;
		}
	   if (dialogSize.height < MINHEIGHT)
		{
		dialogSize.height = MINHEIGHT;
		}
	   final Rectangle parentBounds = getParentBounds();
	   final Insets insets = getBorderInsets(contentPane);
	   final Rectangle bounds = new Rectangle(
		     parentBounds.x - insets.left + (parentBounds.width - dialogSize.width) / 2, parentBounds.y - insets.top + (parentBounds.height - dialogSize.height) / 2,
		     dialogSize.width + insets.left + insets.right, dialogSize.height + insets.top + insets.bottom);
	   jdialog.setBounds(bounds);
	   }
	else if (hdialog != null)
	   {
	   // set content
	   hdialog.setContentPane(contentPane);
	   hdialog.setMinimumSize(new Dimension(MINWIDTH, MINHEIGHT));
	   hdialog.getRootPane().setDefaultButton(defaultButton);
	   hdialog.pack();
	   // check dimensions
	   final Dimension dialogSize = hdialog.getPreferredSize();
	   if (dialogSize.width < MINWIDTH)
		{
		dialogSize.width = MINWIDTH;
		}
	   if (dialogSize.height < MINHEIGHT)
		{
		dialogSize.height = MINHEIGHT;
		}
	   // calculate bounds
	   final Rectangle parentBounds = getParentBounds();
	   final Insets insets = getBorderInsets(contentPane);
	   final Rectangle bounds = new Rectangle(
		     parentBounds.x - insets.left + (parentBounds.width - dialogSize.width) / 2, parentBounds.y - insets.top + (parentBounds.height - dialogSize.height) / 2,
		     dialogSize.width + insets.left + insets.right, dialogSize.height + insets.top + insets.bottom);														//System.out.println(bounds);
	   if (bounds.x < 0)
		{
		bounds.x = 0;
		}
	   if (bounds.y < 0)
		{
		bounds.y = 0;
		}
	   // check if everything is visible
	   final Rectangle desktopBounds = SwingUtils.getJDesktopPane(iframeParent).getBounds(); 															//System.out.println(desktopBounds);
	   final int invisibleX = bounds.x + bounds.width - desktopBounds.width; 																			//System.out.println(invisibleX);
	   final int invisibleY = bounds.y + bounds.height - desktopBounds.height; 																			//System.out.println(invisibleY);
	   if (invisibleX > 0)
		{
		bounds.x = Math.max(0, bounds.x - invisibleX); 																				//System.out.println(bounds);
		}
	   if (invisibleY > 0)
		{
		bounds.y = Math.max(0, bounds.y - invisibleY); 																				//System.out.println(bounds);
		}			// set bounds
	   hdialog.setBounds(bounds);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setDefaultButton(JButton b)
	{
	defaultButton = b;

	if (jdialog != null)
	   {
	   jdialog.getRootPane().setDefaultButton(b);
	   }
	else if (hdialog != null)
	   {
	   hdialog.getRootPane().setDefaultButton(b);
	   }
	}

   public void setLayout(LayoutManager manager)
	{
	if (jdialog != null)
	   {
	   jdialog.setLayout(manager);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setLayout(manager);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setLocation(Point p)
	{
	if (jdialog != null)
	   {
	   jdialog.setLocation(p);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setLocation(p);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setLocation(int x, int y)
	{
	if (jdialog != null)
	   {
	   jdialog.setLocation(x, y);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setLocation(x, y);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setMinimumSize(Dimension d)
	{
	if (jdialog != null)
	   {
	   jdialog.setMinimumSize(d);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setMinimumSize(d);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setSize(Dimension d)
	{
	if (jdialog != null)
	   {
	   jdialog.setSize(d);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setSize(d);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setSize(int width, int height)
	{
	if (jdialog != null)
	   {
	   jdialog.setSize(width, height);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setSize(width, height);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void setVisible(boolean b)
	{
	if (jdialog != null)
	   {
	   jdialog.setVisible(b);
	   }
	else if (hdialog != null)
	   {
	   hdialog.setVisible(b);
	   }
	else
	   {
	   throw new IllegalStateException("Dialog not found.");
	   }
	}

   public void showError(String errorText)
	{
	if (jdialog != null)
	   {
	   JReqTools.showError(jdialog, errorText);
	   }
	else if (hdialog != null)
	   {
	   JReqTools.showError(hdialog, errorText);
	   }
	else
	   {
	   JReqTools.showError((JFrame) null, errorText);
	   }
	}

   public void showException(Throwable t)
	{
	if (jdialog != null)
	   {
	   JReqTools.showException(jdialog, t);
	   }
	else if (hdialog != null)
	   {
	   JReqTools.showException(hdialog, t);
	   }
	else
	   {
	   JReqTools.showException((JFrame) null, t);
	   }
	}

   public void showMessage(String text)
	{
	if (jdialog != null)
	   {
	   JReqTools.showMessage(jdialog, text);
	   }
	else if (hdialog != null)
	   {
	   JReqTools.showMessage(hdialog, text);
	   }
	else
	   {
	   JReqTools.showMessage((JFrame) null, text);
	   }
	}

   public boolean showQuestion(String text)
	{
	if (jdialog != null)
	   {
	   return JReqTools.showQuestion(jdialog, text);
	   }
	else if (hdialog != null)
	   {
	   return JReqTools.showQuestion(hdialog, text);
	   }
	else
	   {
	   return JReqTools.showQuestion((JFrame) null, text);
	   }
	}

   public int showQuestion(String title, String message, String[] answers, int defaultAnswer)
	{
	if (jdialog != null)
	   {
	   return JReqTools.showQuestion(jdialog, title, message, answers, defaultAnswer);
	   }
	else if (hdialog != null)
	   {
	   return JReqTools.showQuestion(hdialog, title, message, answers, defaultAnswer);
	   }
	else
	   {
	   return JReqTools.showQuestion((JFrame) null, title, message, answers, defaultAnswer);
	   }
	}

   public int showQuestion(String title, String message, String[] answers, Icon[] icons, int defaultAnswer)
	{
	if (jdialog != null)
	   {
	   return JReqTools.showQuestion(jdialog, title, message, answers, icons, defaultAnswer);
	   }
	else if (hdialog != null)
	   {
	   return JReqTools.showQuestion(hdialog, title, message, answers, icons, defaultAnswer);
	   }
	else
	   {
	   return JReqTools.showQuestion((JFrame) null, title, message, answers, icons, defaultAnswer);
	   }
	}
   }
