/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.io.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class HEditableList extends JPanel
   {
   // Constants
   private final String NEW_ITEM = Helper.GRS("HEditableList.NewItem");
   // Components
   private final JList list = new JList();
   private final HTextField textfield = new HTextField();
   private final JButton add = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_ADD));
   private final JButton sub = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_REMOVE));
   private final JButton up = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_MOVEUP));
   private final JButton down = new JButton(UIManager.getIcon(UIPreferences.BUTTON_ICON_MOVEDOWN));
   private final JScrollPane scrollPane = new JScrollPane(list, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
   private JPopupMenu menu = null;
   private JComponent nextComponent = null;
   // Data
   private final Vector<String> items = new Vector<String>(10, 10);
   private final Vector<ListSelectionListener> listeners = new Vector<ListSelectionListener>();
   private final Vector<MouseListener> mouseListeners = new Vector<MouseListener>();
   private final Vector<ChangeListener> changeListeners = new Vector<ChangeListener>();
   private int index = -1;
   private boolean added = false;

   public class ListMouseListener implements MouseListener
	{
	public void mouseClicked(java.awt.event.MouseEvent arg0)
	   {
	   // Das ausgewählte in das Textfeld
	   setSelectedIndex(list.locationToIndex(arg0.getPoint()));

	   // Und ein Event senden an die Listeners
	   arg0 = new MouseEvent(HEditableList.this,
		     arg0.getID(),
		     arg0.getWhen(),
		     arg0.getModifiers(),
		     arg0.getX(),
		     arg0.getY(),
		     arg0.getClickCount(),
		     arg0.isPopupTrigger());

	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseClicked(arg0);
		}
	   }

	public void mouseEntered(java.awt.event.MouseEvent arg0)
	   {
	   arg0 = new MouseEvent(HEditableList.this,
		     arg0.getID(),
		     arg0.getWhen(),
		     arg0.getModifiers(),
		     arg0.getX(),
		     arg0.getY(),
		     arg0.getClickCount(),
		     arg0.isPopupTrigger());

	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseEntered(arg0);
		}
	   }

	public void mouseExited(java.awt.event.MouseEvent arg0)
	   {
	   arg0 = new MouseEvent(HEditableList.this,
		     arg0.getID(),
		     arg0.getWhen(),
		     arg0.getModifiers(),
		     arg0.getX(),
		     arg0.getY(),
		     arg0.getClickCount(),
		     arg0.isPopupTrigger());

	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseExited(arg0);
		}
	   }

	public void mousePressed(java.awt.event.MouseEvent arg0)
	   {
	   arg0 = new MouseEvent(HEditableList.this,
		     arg0.getID(),
		     arg0.getWhen(),
		     arg0.getModifiers(),
		     arg0.getX(),
		     arg0.getY(),
		     arg0.getClickCount(),
		     arg0.isPopupTrigger());

	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mousePressed(arg0);
		}
	   }

	public void mouseReleased(java.awt.event.MouseEvent arg0)
	   {
	   arg0 = new MouseEvent(HEditableList.this,
		     arg0.getID(),
		     arg0.getWhen(),
		     arg0.getModifiers(),
		     arg0.getX(),
		     arg0.getY(),
		     arg0.getClickCount(),
		     arg0.isPopupTrigger());

	   for (Enumeration e = mouseListeners.elements(); e.hasMoreElements();)
		{
		((MouseListener) e.nextElement()).mouseReleased(arg0);
		}
	   }
	}

   public HEditableList()
	{
	this(null);
	}

   public HEditableList(String[] data)
	{
	super(new GridBagLayout());

	// Properties
	up.setMargin(new Insets(1, 1, 1, 1));
	down.setMargin(new Insets(1, 1, 1, 1));
	add.setMargin(new Insets(1, 1, 1, 1));
	sub.setMargin(new Insets(1, 1, 1, 1));

	// Tooltips
	add.setToolTipText(Helper.GRS("HEditableList.ToolTip.Add"));
	sub.setToolTipText(Helper.GRS("HEditableList.ToolTip.Remove"));
	up.setToolTipText(Helper.GRS("HEditableList.ToolTip.Up"));
	down.setToolTipText(Helper.GRS("HEditableList.ToolTip.Down"));

	// Listeners
	add.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		addItem(NEW_ITEM);
		}
	   });
	sub.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		removeSelectedItem();
		}
	   });
	up.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		moveSelectedItemUp();
		}
	   });
	down.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		moveSelectedItemDown();
		}
	   });
	list.addMouseListener(new HEditableList.ListMouseListener());
	textfield.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		if (list.getSelectedIndex() == -1)
		   {
		   addItem(textfield.getText());
		   }
		else
		   {
		   changeSelectedItem(textfield.getText());
		   }
		}
	   });

	// Create Panel
	final GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0, 0, 0, 0);

	// JList
	c.fill = GridBagConstraints.BOTH;
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 5;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 1;
	this.add(new JScrollPane(list, ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);

	// HTextField
	c.fill = GridBagConstraints.BOTH;
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	this.add(textfield, c);

	// JButtons
	c.fill = GridBagConstraints.NONE;
	c.gridx = 1;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(add, c);
	c.gridx = 2;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(sub, c);
	c.gridx = 3;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(up, c);
	c.gridx = 4;
	c.gridy = 1;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	this.add(down, c);

	// Menus
	menu = SwingUtils.createPopupMenu(list, "Items");
	JMenuItem item = menu.add(new JMenuItem(Helper.GRS("HEditableList.Sort"), UIManager.getIcon(UIPreferences.MENU_ICON_SORT)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		sortItems();
		}
	   });
	item = menu.add(new JMenuItem(Helper.GRS("HEditableList.Clear"), UIManager.getIcon(UIPreferences.MENU_ICON_CLEAR)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		removeAllItems();
		}
	   });

	// Initialisieren mit Daten
	if (data != null)
	   {
	   set(data);
	   }
	}

   public synchronized JMenuItem add(JMenuItem item)
	{
	if (!added)
	   {
	   menu.addSeparator();
	   added = true;
	   }

	menu.add(item);
	return item;
	}

   public void addChangeListener(ChangeListener l)
	{
	changeListeners.addElement(l);
	}

   public synchronized void addItem(String text)
	{
	items.addElement(text);
	index = text.equals(NEW_ITEM) ? items.size() - 1 : -1;
	updateJList();
	sendChangeEvent();
	}

   public synchronized void addItems(String[] texts)
	{
	for (int i = 0; i < texts.length; ++i)
	   {
	   items.addElement(texts[i]);
	   }
	index = -1;
	updateJList();
	sendChangeEvent();
	}

   public void addListSelectionListener(ListSelectionListener l)
	{
	listeners.addElement(l);
	}

   public void addMouseListener(MouseListener l)
	{
	mouseListeners.addElement(l);
	}

   public synchronized void changeSelectedItem(String text)
	{
	int index = list.getSelectedIndex();
	if (index == -1)
	   {
	   return;
	   }
	items.setElementAt(text, index);
	updateJList();
	sendChangeEvent();
	}

   public synchronized boolean contains(String text)
	{
	return items.contains(text);
	}

   public void createPopupMenuFromFile(String file, String iconname)
	{
	menu.addSeparator();
	SwingUtils.addMenuItemsFromFile(file, "", "", iconname, menu,
		  new ActionListener()
		     {
		     public void actionPerformed(ActionEvent e)
			  {
			  HEditableList.this.addItem(e.getActionCommand());
			  }
		     });
	}

   public void createPopupMenuFromStringArray(String[] data, String iconname)
	{
	menu.addSeparator();
	SwingUtils.addMenuItemsFromStringArray(data, "", "", iconname, menu,
		  new ActionListener()
		     {
		     public void actionPerformed(ActionEvent e)
			  {
			  HEditableList.this.addItem(e.getActionCommand());
			  }
		     });
	}

   public synchronized String[] get()
	{
	int i, len = items.size();

	String[] result = new String[len];
	for (i = 0; i < len; i++)
	   {
	   result[i] = items.elementAt(i);
	   }

	return result;
	}

   /**
    * Returns the JPoupMenu associated with this component. Be careful using it!
    */
   public JPopupMenu getPopupMenu()
	{
	return menu;
	}

   public synchronized int getSelectedIndex()
	{
	return list.getSelectedIndex();
	}

   public synchronized String getSelectedItem()
	{
	int index = getSelectedIndex();
	if (index == -1)
	   {
	   return null;
	   }
	return items.elementAt(index);
	}

   public synchronized HTextField getTextField()
	{
	return textfield;
	}

   public synchronized int length()
	{
	return items.size();
	}

   public synchronized void moveSelectedItemDown()
	{
	int index = list.getSelectedIndex();
	if (index == -1 || index == items.size() - 1)
	   {
	   SwingUtils.beep();
	   return;
	   }

	final String s = items.elementAt(index);
	items.removeElementAt(index);
	items.insertElementAt(s, ++index);

	this.index = index;

	updateJList();
	sendChangeEvent();
	}

   public synchronized void moveSelectedItemUp()
	{
	int index = list.getSelectedIndex();
	if (index == -1 || index == 0)
	   {
	   SwingUtils.beep();
	   return;
	   }

	final String s = items.elementAt(index);
	items.removeElementAt(index);
	items.insertElementAt(s, --index);

	this.index = index;

	updateJList();
	sendChangeEvent();
	}

   public void readFrom(String file) throws java.io.IOException
	{
	removeAllItems();

	final FileAccess fa = new FileAccess(file);
	do
	   {
	   String line = fa.readLine();
	   if (line == null)
		{
		break;
		}
	   addItem(line);
	   }
	while (true);

	fa.close();
	}

   public synchronized void removeAllItems()
	{
	items.setSize(0);
	index = -1;

	updateJList();
	sendChangeEvent();
	}

   public synchronized void removeSelectedItem()
	{
	int index = list.getSelectedIndex();
	if (index == -1)
	   {
	   SwingUtils.beep();
	   return;
	   }

	String s = items.elementAt(index);
	items.removeElementAt(index);
	this.index = Math.min(index, items.size() - 1);

	updateJList();
	sendChangeEvent();
	}

   public void saveToFile(String directory, String filename) throws java.io.IOException
	{
	File dir = new File(directory);
	dir.mkdirs();

	FileAccess fa = new FileAccess(directory + File.separator + filename, FileAccess.WRITE);
	String[] data = get();
	for (int i = 0; i < data.length; ++i)
	   {
	   fa.writeLine(data[i]);
	   }
	fa.close();
	}

   private void sendChangeEvent()
	{
	for (Enumeration e = changeListeners.elements(); e.hasMoreElements();)
	   {
	   ((ChangeListener) e.nextElement()).stateChanged(new ChangeEvent(this));
	   }
	}

   public synchronized void set(String[] texts)
	{
	if (texts == null)
	   {
	   texts = new String[0];
	   }

	items.setSize(0);

	for (int i = 0; i < texts.length; i++)
	   {
	   items.addElement(texts[i]);
	   }
	index = texts.length == 0 ? -1 : 0;
	updateJList();
	sendChangeEvent();
	}

   public void setChangeEvent()
	{
	}

   public void setLabel(String text)
	{
	add(new JLabel(text), 0);
	}

   public synchronized void setSelectedIndex(int value)
	{
	index = value;
	updateJList();
	}

   public synchronized void setSelectedItem(String text)
	{
	index = items.indexOf(text);
	updateJList();
	}

   public void setToolTipText(String text)
	{
	list.setToolTipText(text);
	}

   private synchronized void sortItems()
	{
	final String[] items = get();
	Arrays.sort(items);
	set(items);
	}

   private void updateJList()
	{
	list.setListData(items);
	list.repaint();

	if (index != -1)
	   {
	   list.setSelectedIndex(index);

	   String text = items.elementAt(index);
	   if (text.equals(NEW_ITEM))
		{
		text = "";
		}

	   textfield.setText(text);
	   }
	else
	   {
	   textfield.setText("");
	   }

	scrollPane.validate();
	textfield.requestFocus();
	}

   public void valueChanged(javax.swing.event.ListSelectionEvent ev)
	{
	ev = new ListSelectionEvent(HEditableList.this, ev.getFirstIndex(), ev.getLastIndex(), ev.getValueIsAdjusting());
	for (Enumeration e = listeners.elements(); e.hasMoreElements();)
	   {
	   ((ListSelectionListener) e.nextElement()).valueChanged(ev);
	   }
	}
   }
