/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import de.hillenbrand.swing.text.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;

public class HEditorPane extends JEditorPane implements Comparable
   {
   private static final boolean DEBUG = false;
   // the editor's additional actions
   private Action[] actions = new Action[]
	{
	new AutoIndentAction1(),
	new AutoIndentAction2(),
	new UndoAction(),
	new RedoAction(),
	new SortDecreasingAction(),
	new SortIncreasingAction(),
	};
   // the user's actions
   private Action[] userActions = new Action[0];
   // context menu
   private HPopupMenu popupmenu = null;
   private MouseListener mouselistener = null;
   // prefs values
   private static int defaultTabSize = -1;
   private static Font defaultFont = null;
   // register all HEditorPanes
   private static final TreeSet<HEditorPane> editors = new TreeSet<HEditorPane>();

   public class AutoIndentAction1 extends AbstractAction
	{
	public AutoIndentAction1()
	   {
	   super("insert-break (auto-indent mode 1)");
	   putValue(Action.SHORT_DESCRIPTION, Helper.GRS("HEditorPane.AutoIndentAction1.Description"));
	   }

	public void actionPerformed(java.awt.event.ActionEvent arg0)
	   {
	   try
		{
		final JTextComponent source = (JTextComponent) arg0.getSource();
		final Document d = source.getDocument();
		final int pos = source.getCaretPosition();

		final StringBuffer line = new StringBuffer(50);
		int offset = pos - 1;
		while (offset != -1)
		   {
		   String s = d.getText(offset, 1);
		   if (s.equals("\n"))
			{
			break;
			}
		   line.insert(0, s);
		   offset--;
		   }

		String lastLine = line.toString();

		String toAdd = "";
		for (offset = 0; offset < lastLine.length() && Character.isWhitespace(lastLine.charAt(offset)); ++offset)
		   {
		   toAdd += lastLine.charAt(offset);
		   }

		String trimmed = lastLine.trim();

		if (trimmed.endsWith("{"))
		   {
		   d.insertString(pos, "\n" + toAdd + "\t", null);
		   }
		else if (trimmed.endsWith("}") && toAdd.length() > 0)
		   {
		   toAdd = toAdd.substring(0, toAdd.length() - 1);
		   d.insertString(pos, "\n" + toAdd, null);
		   }
		else
		   {
		   d.insertString(pos, "\n" + toAdd, null);
		   }
		}
	   catch (Exception x)
		{
		System.err.println(x);
		}
	   }
	}

   public class AutoIndentAction2 extends AbstractAction
	{
	public AutoIndentAction2()
	   {
	   super("insert-break (auto-indent mode 2)");
	   putValue(Action.SHORT_DESCRIPTION, Helper.GRS("HEditorPane.AutoIndentAction2.Description"));
	   }

	public void actionPerformed(java.awt.event.ActionEvent arg0)
	   {
	   try
		{
		final JTextComponent source = (JTextComponent) arg0.getSource();
		final Document d = source.getDocument();
		final int pos = source.getCaretPosition();

		final StringBuffer line = new StringBuffer(50);
		int offset = pos - 1;
		while (offset != -1)
		   {
		   String s = d.getText(offset, 1);
		   if (s.equals("\n"))
			{
			break;
			}
		   line.insert(0, s);
		   offset--;
		   }

		String lastLine = line.toString();

		String toAdd = "";
		for (offset = 0; offset < lastLine.length() && Character.isWhitespace(lastLine.charAt(offset)); ++offset)
		   {
		   toAdd += lastLine.charAt(offset);
		   }

		String trimmed = lastLine.trim();
		if (trimmed.endsWith("}") && toAdd.length() > 0)
		   {
		   toAdd = toAdd.substring(0, toAdd.length() - 1);
		   d.insertString(pos, "\n" + toAdd, null);
		   }
		else
		   {
		   d.insertString(pos, "\n" + toAdd, null);
		   }
		}
	   catch (Exception x)
		{
		System.err.println(x);
		}
	   }
	}

   class RedoAction extends TextAction
	{
	public RedoAction()
	   {
	   super("redo");
	   putValue(Action.SHORT_DESCRIPTION, Helper.GRS("HEditorPane.RedoAction.Description"));
	   }

	public void actionPerformed(ActionEvent a)
	   {
	   redo();
	   }
	}

   class SortDecreasingAction extends SortIncreasingAction
	{
	public SortDecreasingAction()
	   {
	   putValue(Action.NAME, "sort-decreasing");
	   putValue(Action.SHORT_DESCRIPTION, Helper.GRS("HEditorPane.SortDecreasingAction.Description"));
	   }

	protected String sort(String text)
	   {
	   final StringTokenizer tok = new StringTokenizer(text, "\n");
	   final String[] data = new String[tok.countTokens()];
	   for (int i = 0; i < data.length; ++i)
		{
		data[i] = tok.nextToken();
		}

	   Arrays.sort(data);

	   final StringBuffer res = new StringBuffer(text.length());
	   for (int i = data.length; i > 0; --i)
		{
		res.append(data[i - 1]).append("\n");
		}
	   if (!text.endsWith("\n"))
		{
		res.setLength(res.length() - 1);
		}

	   return res.toString();
	   }
	}

   class SortIncreasingAction extends TextAction
	{
	public SortIncreasingAction()
	   {
	   super("sort-increasing");
	   putValue(Action.SHORT_DESCRIPTION, Helper.GRS("HEditorPane.SortIncreasingAction.Description"));
	   }

	public void actionPerformed(ActionEvent e)
	   {
	   final JTextComponent t = getFocusedComponent();
	   if (t == null)
		{
		SwingUtils.beep();
		return;
		}

	   final int start = t.getSelectionStart();
	   if (start < 0)
		{
		SwingUtils.beep();
		return;
		}

	   final int end = t.getSelectionEnd();
	   if (end < 0 || end <= start)
		{
		SwingUtils.beep();
		return;
		}

	   final int caretPos = t.getCaretPosition();
	   final String text = t.getText();
	   final String t1 = text.substring(0, start);
	   final String t2 = sort(text.substring(start, end));
	   final String t3 = text.substring(end);

	   try
		{
		t.setText(t1 + t2 + t3);
		t.setCaretPosition(caretPos);
		t.select(start, end);
		}
	   catch (Exception x)
		{
		}
	   }

	protected String sort(String text)
	   {
	   final StringTokenizer tok = new StringTokenizer(text, "\n");
	   final String[] data = new String[tok.countTokens()];
	   for (int i = 0; i < data.length; ++i)
		{
		data[i] = tok.nextToken();
		}

	   Arrays.sort(data);

	   final StringBuffer res = new StringBuffer(text.length());
	   for (int i = 0; i < data.length; ++i)
		{
		res.append(data[i]).append("\n");
		}
	   if (!text.endsWith("\n"))
		{
		res.setLength(res.length() - 1);
		}

	   return res.toString();
	   }
	}

   class UndoAction extends TextAction
	{
	public UndoAction()
	   {
	   super("undo");
	   putValue(Action.SHORT_DESCRIPTION, Helper.GRS("HEditorPane.UndoAction.Description"));
	   }

	public void actionPerformed(ActionEvent a)
	   {
	   undo();
	   }
	}

   public HEditorPane()
	{
	// set new document type
	setDocument(new UndoableDocument());

	// undo
	this.addKeyListener(new KeyAdapter()
	   {
	   public void keyPressed(KeyEvent e)
		{
		if (e.getKeyCode() == KeyEvent.VK_Z && e.getModifiersEx() == KeyEvent.CTRL_DOWN_MASK)
		   {
		   undo();
		   }
		else if (e.getKeyCode() == KeyEvent.VK_Z && e.getModifiersEx() == (KeyEvent.CTRL_DOWN_MASK | KeyEvent.SHIFT_DOWN_MASK))
		   {
		   redo();
		   }
		}
	   });

	// menu
	popupmenu = new HPopupMenu(Helper.GRS("HTextField.PopupMenu.Title"));

	// mouse listener
	mouselistener = SwingUtils.createMouseListenerForPopupMenu(this, popupmenu);
	this.addMouseListener(mouselistener);

	// items
	JMenuItem item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Undo"), UIManager.getIcon(UIPreferences.MENU_ICON_UNDO)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		undo();
		}
	   });
	item.setAccelerator(KeyStroke.getKeyStroke("control Z"));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Redo"), UIManager.getIcon(UIPreferences.MENU_ICON_REDO)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		redo();
		}
	   });
	item.setAccelerator(KeyStroke.getKeyStroke("control shift Z"));

	popupmenu.addSeparator();
	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Cut"), UIManager.getIcon(UIPreferences.MENU_ICON_CUT)));
	item.addActionListener(getAction(DefaultEditorKit.cutAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.cutAction));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Copy"), UIManager.getIcon(UIPreferences.MENU_ICON_COPY)));
	item.addActionListener(getAction(DefaultEditorKit.copyAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.copyAction));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Paste"), UIManager.getIcon(UIPreferences.MENU_ICON_PASTE)));
	item.addActionListener(getAction(DefaultEditorKit.pasteAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.pasteAction));

	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Delete"), UIManager.getIcon(UIPreferences.MENU_ICON_DELETE)));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		setText("");
		}
	   });

	popupmenu.addSeparator();
	item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.SelectAll"), UIManager.getIcon(UIPreferences.MENU_ICON_SELECTALL)));
	item.addActionListener(getAction(DefaultEditorKit.selectAllAction));
	item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.selectAllAction));

	popupmenu.addSeparator();
	item = popupmenu.add(new JMenuItem(Helper.GRS("HEditorPane.SortIncreasingAction.Description"), UIManager.getIcon(UIPreferences.MENU_ICON_SORT)));
	item.addActionListener(new SortIncreasingAction());
	item = popupmenu.add(new JMenuItem(Helper.GRS("HEditorPane.SortDecreasingAction.Description"), UIManager.getIcon(UIPreferences.MENU_ICON_SORT)));
	item.addActionListener(new SortDecreasingAction());

	// set the prefs
	setDefaultPrefs();

	// register
	editors.add(this);
	}

   public synchronized void addAction(Action action)
	{
	if (action == null)
	   {
	   return;
	   }

	// check if already present
	final Action a = getAction((String) action.getValue(Action.NAME));
	if (a != null)
	   {
	   System.err.println(getClass().getName() + ".addAction(): action '" + action.getValue(Action.NAME) + "' already present in " + this);
	   return;
	   }

	//if (DEBUG) Logger.log("Adding action " + action.getValue(Action.NAME));

	// insert into array
	final Action[] newActions = new Action[userActions.length + 1];
	System.arraycopy(userActions, 0, newActions, 0, userActions.length);
	newActions[userActions.length] = action;

	// set
	userActions = newActions;
	}

   public void addActions(Action[] a)
	{
	if (a == null)
	   {
	   return;
	   }
	if (DEBUG)
	   {
	   Logger.log(this, "addActions", "adding " + a.length + " actions");
	   }
	for (int i = 0; i < a.length; ++i)
	   {
	   addAction(a[i]);
	   }
	}

   public int compareTo(Object o)
	{
	if (this == o)
	   {
	   return 0;
	   }
	return hashCode() - o.hashCode();
	}

   public HToolBar createDefaultToolBar()
	{
	final HToolBar toolbar = new HToolBar();

	toolbar.add(getAction("cut-to-clipboard"), HIconFactory.getToolbarIcon("editcut"));
	toolbar.add(getAction("copy-to-clipboard"), HIconFactory.getToolbarIcon("editcopy"));
	toolbar.add(getAction("paste-from-clipboard"), HIconFactory.getToolbarIcon("editpaste"));
	toolbar.addSeparator();
	toolbar.add(getAction("undo"), HIconFactory.getToolbarIcon("undo"));
	toolbar.add(getAction("redo"), HIconFactory.getToolbarIcon("redo"));
	toolbar.addSeparator();
	toolbar.add(getAction("sort-increasing"), HIconFactory.getToolbarIcon("sort-increasing"));
	toolbar.add(getAction("sort-decreasing"), HIconFactory.getToolbarIcon("sort-decreasing"));
	toolbar.addSeparator();

	return toolbar;
	}

   public final Action getAction(String name)
	{
	final Action[] a = getActions();
	for (int i = 0; i < a.length; ++i)
	   {
	   if (a[i].getValue(Action.NAME).equals(name))
		{
		return a[i];
		}
	   }

	return null;
	}

   public Action[] getActions()
	{
	final Action[] a1 = super.getActions();
	final Action[] a2 = actions != null ? actions : new Action[0];
	final Action[] a3 = userActions != null ? userActions : new Action[0];

	return TextAction.augmentList(TextAction.augmentList(a1, a2), a3);
	}

   public static synchronized Font getDefaultFont()
	{
	return defaultFont;
	}

   public static synchronized int getDefaultTabSize()
	{
	return defaultTabSize;
	}

   public JPopupMenu getJPopupMenu()
	{
	return popupmenu;
	}

   public HPopupMenu getHPopupMenu()
	{
	return popupmenu;
	}

   public final KeyStroke getKeyStrokeForAction(String action)
	{
	// get input map
	final InputMap im = (InputMap) UIManager.get("EditorPane.focusInputMap");
	if (im == null)
	   {
	   return null;
	   }

	// create result
	KeyStroke result1 = null;
	KeyStroke result2 = null;

	// find result
	final KeyStroke[] keys = im.allKeys();
	for (int i = 0; i < keys.length; ++i)
	   {
	   final String s = (String) im.get(keys[i]);
	   if (result1 == null && s != null && s.equals(action))
		{
		result1 = keys[i];
		}
	   else if (result2 == null && s != null && s.equals(action))
		{
		result2 = keys[i];
		}
	   }

	// check results
	if (result1 != null && result2 != null && result1.getModifiers() == 0)
	   {
	   return result2;
	   }
	return result1;
	}

   public void insertText(String text)
	{
	replaceSelection(text);
	}

   public void redo()
	{
	try
	   {
	   final UndoableDocument document = (UndoableDocument) this.getDocument();
	   if (document.canRedo())
		{
		document.redo();
		}
	   else
		{
		SwingUtils.beep();
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }
	}

   public static synchronized void setDefaultFont(Font font)
	{
	if (DEBUG)
	   {
	   System.out.println("HEditorPane.setDefaultFont(): " + font);
	   }

	// set the default value
	defaultFont = font;

	// notify all editors
	for (Iterator i = editors.iterator(); i.hasNext();)
	   {
	   ((HEditorPane) i.next()).setDefaultPrefs();
	   }
	}

   private void setDefaultPrefs()
	{
	final Document doc = getDocument();

	// set tab size
	if (doc != null && defaultTabSize > 0)
	   {
	   if (DEBUG)
		{
		System.out.println(this.getClass().getName() + ".setDefaultPrefs(): setting tab size to " + defaultTabSize);
		}
	   doc.putProperty(PlainDocument.tabSizeAttribute, new Integer(defaultTabSize));
	   }

	if (defaultFont != null)
	   {
	   if (DEBUG)
		{
		System.out.println(this.getClass().getName() + ".setDefaultPrefs(): setting font to " + defaultFont.getName() + "/" + defaultFont.getSize());
		}
	   setFont(defaultFont);
	   }

	// repaint
	repaint();
	}

   public void setTabSize(int tabSize)
	{
	final Document doc = getDocument();
	if (doc != null)
	   {
	   doc.putProperty(PlainDocument.tabSizeAttribute, new Integer(tabSize));
	   }
	}

   public static synchronized void setDefaultTabSize(int tabSize)
	{
	if (DEBUG)
	   {
	   System.out.println("HEditorPane.setDefaultTabSize(): " + tabSize);
	   }

	// set the default value
	defaultTabSize = tabSize;

	// notify all editors
	for (Iterator i = editors.iterator(); i.hasNext();)
	   {
	   ((HEditorPane) i.next()).setDefaultPrefs();
	   }
	}

   public final void setDocument(Document doc)
	{
	// set tab width
	doc.putProperty(PlainDocument.tabSizeAttribute, new Integer(defaultTabSize));

	// set document
	super.setDocument(doc);
	}

   public void setEditable(boolean b)
	{
	if (b)
	   {
	   if (mouselistener != null)
		{
		addMouseListener(mouselistener);
		}
	   }
	else
	   {
	   if (mouselistener != null)
		{
		removeMouseListener(mouselistener);
		}
	   }

	super.setEditable(b);
	}

   public void setEnabled(boolean b)
	{
	if (b)
	   {
	   if (mouselistener != null)
		{
		addMouseListener(mouselistener);
		}
	   }
	else
	   {
	   if (mouselistener != null)
		{
		removeMouseListener(mouselistener);
		}
	   }

	super.setEnabled(b);
	}

   public void undo()
	{
	try
	   {
	   final UndoableDocument document = (UndoableDocument) this.getDocument();
	   if (document.canUndo())
		{
		document.undo();
		}
	   else
		{
		SwingUtils.beep();
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }
	}

   public void clearUndoRedo()
	{
	final UndoableDocument document = (UndoableDocument) this.getDocument();
	document.discardAllEdits();
	}
   }
