/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

/**
 * Implements a FileChooser that can have its own bounds.
 * The javax.swing.JFileChooser always opens at an unknown position with a pack()-ed size.
 * This is a workaround until Sun implements the feature in JFileChooser.
 */
public class HFileChooser extends JFileChooser
   {
   // Property: bounds
   private static Rectangle defaultBounds = null;
   // Property: bounds
   private Rectangle bounds = null;
   // Icons
   private static final Icon DIR_ROOT = HIconFactory.getIcon("16x16_folder_locked");
   private static final Icon DIR_FOLDER = HIconFactory.getIcon("16x16_fileopen");
   private static final Icon DIR_FLOPPY = HIconFactory.getIcon("device_16x16_3floppy_unmount");
   private static final Icon DIR_CD = HIconFactory.getIcon("device_16x16_cdrom_unmount");
   private static final Icon DIR_DVD = HIconFactory.getIcon("device_16x16_dvd_unmount");
   private static final Icon DIR_CDRECORDER = HIconFactory.getIcon("device_16x16_cdwriter_unmount");
   private static final Icon DIR_DVDRECORDER = HIconFactory.getIcon("device_16x16_cdwriter_unmount");
   private static final Icon DIR_ZIP = HIconFactory.getIcon("device_16x16_zip_unmount");
   private static final Icon DIR_NET = HIconFactory.getIcon("device_16x16_nfs_unmount");
   private final Object[] suffixes = new Object[]
	{
	".afm", HIconFactory.getIcon("mime_16x16_font_type1"),
	".au", HIconFactory.getIcon("mime_16x16_sound"),
	".avi", HIconFactory.getIcon("mime_16x16_video"),
	".bash", HIconFactory.getIcon("mime_16x16_shellscript"),
	".bin", HIconFactory.getIcon("mime_16x16_binary"),
	".bmp", HIconFactory.getIcon("mime_16x16_image"),
	".c", HIconFactory.getIcon("mime_16x16_source_c"),
	".cpp", HIconFactory.getIcon("mime_16x16_source_cpp"),
	".csh", HIconFactory.getIcon("mime_16x16_shellscript"),
	".deb", HIconFactory.getIcon("mime_16x16_deb"),
	".doc", HIconFactory.getIcon("mime_16x16_document"),
	".dvi", HIconFactory.getIcon("mime_16x16_dvi"),
	".enc", HIconFactory.getIcon("mime_16x16_encrypted"),
	".f", HIconFactory.getIcon("mime_16x16_source_f"),
	".flw", HIconFactory.getIcon("mime_16x16_kivio_flw"),
	".gf", HIconFactory.getIcon("mime_16x16_gf"),
	".gif", HIconFactory.getIcon("mime_16x16_image"),
	".h", HIconFactory.getIcon("mime_16x16_source_h"),
	".htm", HIconFactory.getIcon("mime_16x16_html"),
	".html", HIconFactory.getIcon("mime_16x16_html"),
	".iff", HIconFactory.getIcon("mime_16x16_image"),
	".iso", HIconFactory.getIcon("mime_16x16_cdimage"),
	".java", HIconFactory.getIcon("mime_16x16_source_java"),
	".jpeg", HIconFactory.getIcon("mime_16x16_image"),
	".jpg", HIconFactory.getIcon("mime_16x16_image"),
	".kfo", HIconFactory.getIcon("mime_16x16_kformula_kfo"),
	".kpr", HIconFactory.getIcon("mime_16x16_kpresenter_kpr"),
	".kre", HIconFactory.getIcon("mime_16x16_krita_kra"),
	".ksp", HIconFactory.getIcon("mime_16x16_kspread_ksp"),
	".kud", HIconFactory.getIcon("mime_16x16_kugar_kud"),
	".kwd", HIconFactory.getIcon("mime_16x16_kword_kwd"),
	".l", HIconFactory.getIcon("mime_16x16_source_l"),
	".log", HIconFactory.getIcon("mime_16x16_log"),
	".make", HIconFactory.getIcon("mime_16x16_make"),
	".man", HIconFactory.getIcon("mime_16x16_man"),
	".mf", HIconFactory.getIcon("mime_16x16_metafont"),
	".mid", HIconFactory.getIcon("mime_16x16_midi"),
	".mime", HIconFactory.getIcon("mime_16x16_mime"),
	".moc", HIconFactory.getIcon("mime_16x16_source_moc"),
	".mpeg", HIconFactory.getIcon("mime_16x16_video"),
	".mpg", HIconFactory.getIcon("mime_16x16_video"),
	".msg", HIconFactory.getIcon("mime_16x16_message"),
	".news", HIconFactory.getIcon("mime_16x16_news"),
	".o", HIconFactory.getIcon("mime_16x16_source_o"),
	".ogg", HIconFactory.getIcon("mime_16x16_sound"),
	".pcf", HIconFactory.getIcon("mime_16x16_font"),
	".pcf.gz", HIconFactory.getIcon("mime_16x16_font"),
	".pdf", HIconFactory.getIcon("mime_16x16_pdf"),
	".pfa", HIconFactory.getIcon("mime_16x16_font_type1"),
	".pfb", HIconFactory.getIcon("mime_16x16_font_type1"),
	".php", HIconFactory.getIcon("mime_16x16_source_php"),
	".pk", HIconFactory.getIcon("mime_16x16_pk"),
	".pl", HIconFactory.getIcon("mime_16x16_source_pl"),
	".png", HIconFactory.getIcon("mime_16x16_image"),
	".ps", HIconFactory.getIcon("mime_16x16_postscript"),
	".ps", HIconFactory.getIcon("mime_16x16_postscript"),
	".py", HIconFactory.getIcon("mime_16x16_source_py"),
	".qt", HIconFactory.getIcon("mime_16x16_quicktime"),
	".qt", HIconFactory.getIcon("mime_16x16_video"),
	".readme", HIconFactory.getIcon("mime_16x16_readme"),
	".rpm", HIconFactory.getIcon("mime_16x16_rpm"),
	".s", HIconFactory.getIcon("mime_16x16_source_s"),
	".sh", HIconFactory.getIcon("mime_16x16_shellscript"),
	".svg", HIconFactory.getIcon("mime_16x16_vectorgfx"),
	".tar", HIconFactory.getIcon("mime_16x16_tar"),
	".tar.gz", HIconFactory.getIcon("mime_16x16_tgz"),
	".tex", HIconFactory.getIcon("mime_16x16_tex"),
	".tgz", HIconFactory.getIcon("mime_16x16_tgz"),
	".ttf", HIconFactory.getIcon("mime_16x16_font_truetype"),
	".txt", HIconFactory.getIcon("mime_16x16_txt"),
	".vcard", HIconFactory.getIcon("mime_16x16_vcard"),
	".vkal", HIconFactory.getIcon("mime_16x16_vcalendar"),
	".wav", HIconFactory.getIcon("mime_16x16_sound"),
	".wmf", HIconFactory.getIcon("mime_16x16_vectorgfx"),
	".xls", HIconFactory.getIcon("mime_16x16_spreadsheet"),
	".y", HIconFactory.getIcon("mime_16x16_source_y"),
	".zip", HIconFactory.getIcon("mime_16x16_zip"),
	};
   private static final Icon FILE_UNKNOWN = HIconFactory.getIcon("mime_16x16_ascii");

   public HFileChooser()
	{
	super();
	}

   protected JDialog createDialog(Component parent) throws HeadlessException
	{
	// get the dialog
	final JDialog d = super.createDialog(parent);

	// set the dialog bounds
	if (bounds != null)
	   {
	   d.setBounds(bounds);
	   }
	else if (defaultBounds != null)
	   {
	   d.setBounds(defaultBounds);
	   }

	// add a listener to get the final bounds after the dialog has been closed
	// this may be too late :-(
	d.addWindowListener(new WindowAdapter()
	   {
	   public void windowClosed(WindowEvent e)
		{
		bounds = d.getBounds();
		if (defaultBounds == null)
		   {
		   defaultBounds = bounds;
		   }
		}
	   });

	// return the dialog
	return d;
	}

   /**
    * Returns the value of the dialogBounds property.
    * @return
    */
   public Rectangle getDialogBounds()
	{
	return bounds;
	}

   public Icon getIcon(File f)
	{
	final String name = f.getAbsolutePath().toLowerCase();

	if (f.isDirectory())
	   {
	   if (name.equals("/"))
		{
		return DIR_ROOT;
		}
	   if (name.equals("/net"))
		{
		return DIR_NET;
		}
	   if (name.equals("/media/floppy"))
		{
		return DIR_FLOPPY;
		}
	   if (name.equals("/media/cdrom"))
		{
		return DIR_CD;
		}
	   if (name.equals("/media/dvd"))
		{
		return DIR_DVD;
		}
	   if (name.equals("/media/dvdrom"))
		{
		return DIR_DVD;
		}
	   if (name.equals("/media/zip"))
		{
		return DIR_ZIP;
		}
	   if (name.equals("/media/cdrecorder"))
		{
		return DIR_CDRECORDER;
		}
	   if (name.equals("/media/dvdrecorder"))
		{
		return DIR_DVDRECORDER;
		}
	   return DIR_FOLDER;
	   }
	else
	   {
	   for (int i = 0; i < suffixes.length; i += 2)
		{
		if (name.endsWith((String) suffixes[i]))
		   {
		   return (Icon) suffixes[i + 1];
		   }
		}
	   return FILE_UNKNOWN;
	   }
	}

   /**
    * Assigns the value of the dialogBounds property.
    * @param arg
    */
   public void setDialogBounds(Rectangle arg)
	{
	bounds = new Rectangle(arg.x, arg.y, arg.width, arg.height);
	}
   }
