/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;

/**
 * These Grid Bag Constraints behave like the super class. The only addition is the set() methods which make modifications easier.
 * @author hillenbr
 */
public class HGridBagConstraints extends GridBagConstraints
   {
   /**
    * The zero insets are (0, 0, 0, 0).
    */
   public static final Insets ZERO_INSETS = new Insets(0, 0, 0, 0);
   /**
    * The default insets are (4, 4, 4, 4).
    */
   public static final Insets DEFAULT_INSETS = new Insets(4, 4, 4, 4);
   /**
    * The default horizontal insets are (0, 4, 0, 4).
    */
   public static final Insets DEFAULT_HORIZONTAL_INSETS = new Insets(0, 4, 0, 4);
   /**
    * The default vertical insets are (4, 0, 4, 0).
    */
   public static final Insets DEFAULT_VERTICAL_INSETS = new Insets(4, 0, 4, 0);

   /**
    * Sets constraints.
    * @param gridx
    * @param gridy
    * @param gridwidth
    * @param gridheight
    * @param weightx
    * @param weighty
    * @param fill
    * @return
    */
   public HGridBagConstraints set(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, int fill)
	{
	this.gridx = gridx;
	this.gridy = gridy;
	this.gridwidth = gridwidth;
	this.gridheight = gridheight;
	this.weightx = weightx;
	this.weighty = weighty;
	this.fill = fill;
	this.insets = (Insets) DEFAULT_INSETS.clone();

	return this;
	}

   /**
    * Sets constraints.
    * @param gridx
    * @param gridy
    * @param gridwidth
    * @param gridheight
    * @param weightx
    * @param weighty
    * @param fill
    * @param insets
    * @return
    */
   public HGridBagConstraints set(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, int fill, Insets insets)
	{
	this.gridx = gridx;
	this.gridy = gridy;
	this.gridwidth = gridwidth;
	this.gridheight = gridheight;
	this.weightx = weightx;
	this.weighty = weighty;
	this.fill = fill;
	this.insets = (Insets) insets.clone();

	return this;
	}

   /**
    * Sets constraints.
    * @param gridx
    * @param gridy
    * @param gridwidth
    * @param gridheight
    * @param weightx
    * @param weighty
    * @param fill
    * @param anchor
    * @return
    */
   public HGridBagConstraints set(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, int fill, int anchor)
	{
	this.gridx = gridx;
	this.gridy = gridy;
	this.gridwidth = gridwidth;
	this.gridheight = gridheight;
	this.weightx = weightx;
	this.weighty = weighty;
	this.fill = fill;
	this.anchor = anchor;
	this.insets = (Insets) DEFAULT_INSETS.clone();

	return this;
	}

   /**
    * Sets constraints.
    * @param gridx
    * @param gridy
    * @param gridwidth
    * @param gridheight
    * @param weightx
    * @param weighty
    * @param fill
    * @param anchor
    * @param insets
    * @return
    */
   public HGridBagConstraints set(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty, int fill, int anchor, Insets insets)
	{
	this.gridx = gridx;
	this.gridy = gridy;
	this.gridwidth = gridwidth;
	this.gridheight = gridheight;
	this.weightx = weightx;
	this.weighty = weighty;
	this.fill = fill;
	this.anchor = anchor;
	this.insets = (Insets) insets.clone();

	return this;
	}
   }
