/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.event.*;
import java.net.URL;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;

public class HHTMLPane extends JEditorPane
   {
   protected boolean toolTipEnabled = true;
   protected URL baseURL;
   protected TooltipHyperlinkListener listener = new TooltipHyperlinkListener();
   protected ArrayList<TooltipResolver> resolvers = new ArrayList<TooltipResolver>();

   public class TooltipHyperlinkListener implements HyperlinkListener
	{
	protected boolean isHovering = false;
	protected String hoveredURL = null;

	public TooltipHyperlinkListener()
	   {
	   }

	public String getHoveredURL()
	   {
	   return hoveredURL;
	   }

	public void hyperlinkUpdate(HyperlinkEvent e)
	   {
	   if (e.getEventType() == HyperlinkEvent.EventType.ENTERED)
		{
		isHovering = true;
		final java.net.URL url = e.getURL();
		hoveredURL = (url != null) ? url.toExternalForm() : null;
		}
	   else if (e.getEventType() == HyperlinkEvent.EventType.EXITED)
		{
		isHovering = false;
		hoveredURL = null;
		}
	   }

	public boolean isHovering()
	   {
	   return isHovering;
	   }
	}

   public interface TooltipResolver
	{
	public String resolveTooltip(String url);
	}

   public HHTMLPane()
	{
	setEditable(false);
	setContentType("text/html");
	addHyperlinkListener(listener);

	toolTipEnabled = ToolTipManager.sharedInstance().isEnabled();
	ToolTipManager.sharedInstance().registerComponent(this);
	}

   public void addTooltipResolver(TooltipResolver r)
	{
	resolvers.add(r);
	}

   public String getToolTipText(MouseEvent event)
	{
	if (listener.isHovering() && listener.getHoveredURL() != null)
	   {
	   ToolTipManager.sharedInstance().setEnabled(true);
	   final String s = listener.getHoveredURL();
	   if (s == null)
		{
		return s;
		}
	   for (TooltipResolver r : resolvers)
		{
		final String resolved = r.resolveTooltip(s);
		if (resolved != null)
		   {
		   return resolved;
		   }
		}
	   return s;
	   }
	else
	   {
	   ToolTipManager.sharedInstance().setEnabled(false);
	   return null;
	   }
	}

   public URL getBaseURL()
	{
	return baseURL;
	}

   public void setBaseURL(URL baseURL)
	{
	this.baseURL = baseURL;
	}

   @Override
   public void setText(String t)
	{
	super.setText(t);
	final Document d = getDocument();
	if (d instanceof HTMLDocument)
	   {
	   ((HTMLDocument) d).setBase(baseURL);
	   }
	}
   }
