/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;
import java.lang.reflect.*;
import java.security.*;
import java.util.*;
import javax.swing.*;

public class HInternalDialog extends JInternalFrame
   {
   private static final boolean DEBUG = false;
   private final JInternalFrame owner;
   private boolean modalMode = false;
   private final ArrayList<SetMaximumListener> listeners = new ArrayList<SetMaximumListener>();

   public static class ModalPrivilegedAction implements PrivilegedAction
	{
	private Class clazz;
	private String methodName;

	public ModalPrivilegedAction(Class clazz, String methodName)
	   {
	   this.clazz = clazz;
	   this.methodName = methodName;
	   }

	@SuppressWarnings("unchecked")
	public Method run()
	   {
	   try
		{
		final Method method = clazz.getDeclaredMethod(methodName, (Class[]) null);
		method.setAccessible(true);
		return method;
		}
	   catch (NoSuchMethodException x)
		{
		System.err.println(x);
		return null;
		}
	   }
	}

   public static interface SetMaximumListener
	{
	public void setMaximum(boolean b);
	}

   public HInternalDialog(JInternalFrame owner, String title)
	{
	super(title);

	// init
	this.owner = owner;
	}

   public void addSetMaximumListener(SetMaximumListener l)
	{
	listeners.add(l);
	}

   public void setMaximum(boolean b) throws java.beans.PropertyVetoException
	{
	if (listeners.size() > 0)
	   {
	   for (SetMaximumListener l : listeners)
		{
		l.setMaximum(b);
		}
	   }
	else
	   {
	   super.setMaximum(b);
	   }
	}

   @SuppressWarnings("unchecked")
   public void setVisible(boolean b)
	{
	if (b)
	   {
	   final JDesktopPane desktop = SwingUtils.getJDesktopPane(owner);
	   if (desktop != null)
		{
		desktop.add(this, JLayeredPane.MODAL_LAYER);
		}
	   super.setVisible(true);
	   }

	try
	   {
	   if (!modalMode && b)
		{
		modalMode = true;
		final Object obj = AccessController.doPrivileged(new ModalPrivilegedAction(Container.class, "startLWModal"));
		if (obj != null)
		   {
		   ((Method) obj).invoke(this, (Object[]) null);
		   }
		}
	   else if (modalMode && !b)
		{
		modalMode = false;
		final Object obj = AccessController.doPrivileged(new ModalPrivilegedAction(Container.class, "stopLWModal"));
		if (obj != null)
		   {
		   ((Method) obj).invoke(this, (Object[]) null);
		   }
		}
	   else if (DEBUG)
		{
		System.err.println("Error: modalMode = " + modalMode + ", b = " + b);
		}
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }

	if (!b)
	   {
	   super.setVisible(false);
	   final JDesktopPane desktop = SwingUtils.getJDesktopPane(owner);
	   if (desktop != null)
		{
		desktop.remove(this);
		}
	   }
	}
   }
