/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.*;
import javax.swing.*;
import de.hillenbrand.util.*;

public class HMultiLineLabel extends JPanel
   {
   public static final int LEFT = JLabel.LEFT;
   public static final int RIGHT = JLabel.RIGHT;
   public static final int CENTER = JLabel.CENTER;

   public HMultiLineLabel(String text)
	{
	this(text, LEFT);
	}

   public HMultiLineLabel(String text, int alignment)
	{
	super(new GridBagLayout());

	// the current line
	int line = 0;

	// the grid bag constraints
	final GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0, 0, 0, 0);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;

	// create the text fields
	if (text == null)
	   {
	   text = "";
	   }
	text = StringUtils.replace(text, "\\n", "\n");
	int counter = 0;
	final StringTokenizer tok = new StringTokenizer(text, "\n", true);
	while (tok.hasMoreTokens())
	   {
	   // get the next token
	   String token = tok.nextToken();
	   // check for extra new lines
	   if (token.equals("\n"))
		{
		counter++;
		if (counter == 1)
		   {
		   continue;
		   }
		}
	   else
		{
		counter = 0;
		}
	   // create the label
	   final JLabel label = new JLabel(token, alignment);

	   // set the constraints
	   c.gridy = line;
	   // add the text field to the panel
	   add(label, c);
	   // increase line number
	   line++;
	   }
	}

   public Dimension getMinimumSize()
	{
	return getPreferredSize();
	}

   public Dimension getPreferredSize()
	{
	final Dimension result = new Dimension(0, 0);

	final Component[] components = getComponents();
	for (int i = 0; i < components.length; ++i)
	   {
	   final Dimension dim = components[i].getPreferredSize();
	   result.width = Math.max(result.width, dim.width);
	   }

	result.height = components.length * getFont().getSize() + 50;
	result.width += 50;

	return result;
	}

   public void setFont(Font font)
	{
	super.setFont(font);

	final Component[] components = getComponents();
	for (int i = 0; i < components.length; ++i)
	   {
	   components[i].setFont(font);
	   }
	}
   }
