/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing;

import de.hillenbrand.swing.text.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
* HPasswordField is a lightweight component that allows the editing of a single line of text, similar to JPasswordField. 
* But it additionally has a context menu.
* 
* @see javax.swing.JTextField
*/

public class HPasswordField extends JPasswordField {
	private static final boolean DEBUG = false;

	private JPopupMenu popupmenu = null;
	private MouseListener mouselistener = null;
	
	public HPasswordField()  {
		super();
		init();
	}

	public HPasswordField(String text)  {
		super(text);
		init();
	}

	public HPasswordField(String text, int columns)  {
		super(text, columns);
		init();
	}

	public Action getAction(String name)  {
		final Action [] actions = getActions();
		for (int i=0; i<actions.length; ++i)
			{
			if (actions[i].getValue(Action.NAME).equals(name)) return actions[i];
			}
		
		return null;
	}

	public JPopupMenu getJPopupMenu()  {
		return popupmenu;
	}

	public KeyStroke getKeyStrokeForAction(String action)  {
		// get input map
		final InputMap im = (InputMap) UIManager.get("PasswordField.focusInputMap");
		if (im == null) return null;
		
		// create result
		KeyStroke result1 = null;
		KeyStroke result2 = null;
		
		// find result
		final KeyStroke [] keys = im.allKeys();
		for (int i=0; i<keys.length; ++i)
			{
			final String s = (String) im.get(keys[i]);
			if (result1 == null && s != null && s.equals(action)) result1 = keys[i];
			else if (result2 == null && s != null && s.equals(action)) result2 = keys[i];
			}
		
		// results prüfen
		if (result1 != null && result2 != null && result1.getModifiers() == 0) return result2;
		return result1;
	}

	private void init()  {
		// menu
		popupmenu = new JPopupMenu(Helper.GRS("HTextField.PopupMenu.Title"));
		
		// mouse listener
		mouselistener = SwingUtils.createMouseListenerForPopupMenu(this, popupmenu);
		this.addMouseListener(mouselistener);
		
		// items
		JMenuItem item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Paste"), UIManager.getIcon(UIPreferences.MENU_ICON_PASTE)));
		item.addActionListener(getAction(DefaultEditorKit.pasteAction));
		item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.pasteAction));
		
		item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.Delete"), UIManager.getIcon(UIPreferences.MENU_ICON_DELETE)));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { setText(""); }});
		
		popupmenu.addSeparator();
		item = popupmenu.add(new JMenuItem(Helper.GRS("HTextField.PopupMenu.SelectAll"), UIManager.getIcon(UIPreferences.MENU_ICON_SELECTALL)));
		item.addActionListener(getAction(DefaultEditorKit.selectAllAction));
		item.setAccelerator(getKeyStrokeForAction(DefaultEditorKit.selectAllAction));
	}

}
