/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing;

import de.hillenbrand.system.*;
import java.awt.*;
import javax.swing.*;

public class HSeparator extends JComponent {
	private static final boolean DEBUG = false;

	public static final int HORIZONTAL 	= 1;
	public static final int VERTICAL 		= 2;
	
	// Property: text
	private String text=null;
	
	// Property: orientation
	protected int orientation=HORIZONTAL;
	
	public HSeparator()  {
		this(null);
	}

	public HSeparator(String text)  {
		// set font 
		setFont(UIPreferences.getAcceleratorFont());
		
		// set text
		setText(text);
	}

	public HSeparator(int orientation)  {
		this(null);
		setOrientation(orientation);
	}

	/**
	* Returns the value of the orientation property.
	*/

	public int getOrientation()  {
		return orientation;
	}

	/**
	* Returns the value of the text property.
	*/

	public String getText()  {
		return text;
	}

	public void paint(Graphics g)  {
		if (orientation == HORIZONTAL) paintHorizontal(g); else paintVertical(g);
	}

	public void paintHorizontal(Graphics g)  {
		// get size
		final Dimension dim = getSize();
		
		// get y coordinate
		final int y = dim.height / 2 - 1;
		
		// draw first line
		g.setColor(Color.black);
		g.drawLine(0, y,dim.width, y);
		
		// draw second line
		g.setColor(Color.white);
		g.drawLine(0, y+1, dim.width, y+1);
		
		// if no text is given, we are finished 
		if (text == null) return;
		
		// calculate width of text
		final FontMetrics fm = getFontMetrics(this.getFont());
		final int width = fm.stringWidth(text);
		
		// clear area where the text will be drawn
		g.setColor(getBackground());
		g.fillRect((dim.width - width - 10) / 2, 0, width + 10, dim.height);
		
		// set font and write the text
		g.setFont(getFont());
		g.setColor(Color.black);
		g.drawString(text, (dim.width - width - 10) / 2 + 5, fm.getAscent());
	}

	public void paintVertical(Graphics g)  {
		// get size
		final Dimension dim = getSize();
		
		// get x coordinate
		final int x = dim.width / 2 - 1;
		
		// draw first line
		g.setColor(Color.black);
		g.drawLine(x, 0, x, dim.height);
		
		// draw second line
		g.setColor(Color.white);
		g.drawLine(x+1, 0, x+1, dim.height);
	}

	/**
	* Assigns the value of the orientation property.
	*/

	public void setOrientation(int arg)  {
		orientation = arg;
	}

	/**
	* Assigns the value of the text property.
	*/

	public void setText(String arg)  {
		// set text
		text = arg;
		
		// calculate new sizes
		final FontMetrics fm = getFontMetrics(getFont());
		setMaximumSize	(new Dimension(Integer.MAX_VALUE	, text == null ? 4 : fm.getHeight() + 2));
		setMinimumSize	(new Dimension(10						, text == null ? 4 : fm.getHeight() + 2));
		setPreferredSize	(new Dimension(50						, text == null ? 4 : fm.getHeight() + 2));
		
		// and repaint
		repaint();
	}

}
