/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import javax.swing.*;
import java.awt.*;
import de.hillenbrand.system.*;
import java.util.*;

public class HSeparatorPanel extends JPanel
   {
   protected final int LEFT = 1;
   protected final int RIGHT = 2;
   private boolean addSeparator = true;
   private ArrayList<JComponent> components = new ArrayList<JComponent>();
   private String message = null;

   public HSeparatorPanel(JComponent... components)
	{
	this(components, true, null);
	}

   public HSeparatorPanel(String msg, JComponent... components)
	{
	this(components, true, msg);
	}

   protected HSeparatorPanel(JComponent[] components, boolean addSeparator, String msg)
	{
	super(new GridBagLayout());

	this.addSeparator = addSeparator;
	this.message = msg;
	this.components.addAll(Arrays.asList(components));

	layoutComponents();
	}

   private void layoutComponents()
	{
	removeAll();

	// variables
	final GridBagConstraints constraints = UIPreferences.getGridBagConstraints();
	int x = 0;
	int y = 0;

	// find out if there is a component that stretches horizonatally
	boolean addJPanel = true;
	for (JComponent c : components)
	   {
	   if (getDefaultFill(c) == GridBagConstraints.HORIZONTAL)
		{
		addJPanel = false;
		}
	   }

	// add separator
	if (addSeparator)
	   {
	   int width = components.size();
	   if (addJPanel)
		{
		if ((getJPanelPosition() & LEFT) != 0)
		   {
		   width = width + 1;
		   }
		if ((getJPanelPosition() & RIGHT) != 0)
		   {
		   width = width + 1;
		   }
		}
	   constraints.insets = new Insets(8, 0, 8, 0);
	   constraints.gridx = 0;
	   constraints.gridy = y++;
	   constraints.gridwidth = width;
	   constraints.gridheight = 1;
	   constraints.weightx = 1;
	   constraints.weighty = 0;
	   constraints.fill = GridBagConstraints.HORIZONTAL;
	   this.add(message == null ? new HSeparator() : new HSeparator(message), constraints);
	   }

	// define insets
	constraints.insets = new Insets(1, 1, 1, 1);

	// add left stretching panel
	if (addJPanel && (getJPanelPosition() & LEFT) != 0)
	   {
	   constraints.gridx = x++;
	   constraints.gridy = y;
	   constraints.gridwidth = 1;
	   constraints.gridheight = 1;
	   constraints.weightx = 1;
	   constraints.weighty = 0;
	   constraints.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JPanel(), constraints);
	   }

	// add the components
	for (int i = 0; i < components.size(); ++i)
	   {
	   if (i == 0)
		{
		constraints.insets = new Insets(1, 0, 1, 1);
		}
	   else if (i == components.size() - 1)
		{
		constraints.insets = new Insets(1, 1, 1, 0);
		}
	   else
		{
		constraints.insets = new Insets(1, 1, 1, 1);
		}

	   constraints.gridx = x++;
	   constraints.gridy = y;
	   constraints.gridwidth = 1;
	   constraints.gridheight = 1;
	   constraints.weightx = getDefaultWeightX(components.get(i));
	   constraints.weighty = 0;
	   constraints.fill = getDefaultFill(components.get(i));
	   this.add(components.get(i), constraints);
	   }

	// add right stretching panel
	if (addJPanel && (getJPanelPosition() & RIGHT) != 0)
	   {
	   constraints.gridx = x++;
	   constraints.gridy = y;
	   constraints.gridwidth = 1;
	   constraints.gridheight = 1;
	   constraints.weightx = 1;
	   constraints.weighty = 0;
	   constraints.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JPanel(), constraints);
	   }
	}

   protected int getJPanelPosition()
	{
	return LEFT;
	}

   protected int getDefaultFill(JComponent c)
	{
	if (c instanceof JButton)
	   {
	   return GridBagConstraints.NONE;
	   }
	if (c instanceof JCheckBox)
	   {
	   return GridBagConstraints.NONE;
	   }
	if (c instanceof JComboBox)
	   {
	   return GridBagConstraints.NONE;
	   }
	if (c instanceof JLabel)
	   {
	   return GridBagConstraints.NONE;
	   }

	return GridBagConstraints.HORIZONTAL;
	}

   protected int getDefaultWeightX(JComponent c)
	{
	if (c instanceof JButton)
	   {
	   return 0;
	   }
	if (c instanceof JCheckBox)
	   {
	   return 0;
	   }
	if (c instanceof JComboBox)
	   {
	   return 0;
	   }
	if (c instanceof JLabel)
	   {
	   return 0;
	   }

	return 1;
	}

   public void addJComponent(int pos, JComponent comp)
	{
	components.add(pos, comp);
	layoutComponents();
	}
   }
