/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.text.*;
import javax.swing.*;

/**
 * A GUI component for editing a time value. It supports different time formats.
 */
public class HTimeField extends JFormattedTextField
   {
   public enum TimeFormat
	{
	HHMM(new SimpleDateFormat("HH:mm"), "14:00"),
	HHMMSS(new SimpleDateFormat("HH:mm:ss"), "14:00:00"),
	HHMMSSmmm(new SimpleDateFormat("HH:mm:ss:SSS"), "14:00:00:000");
	private SimpleDateFormat format;
	private String value;

	TimeFormat(SimpleDateFormat f, String defaultValue)
	   {
	   this.format = f;
	   this.value = defaultValue;
	   }

	public SimpleDateFormat getFormat()
	   {
	   return format;
	   }

	public String getValue()
	   {
	   return value;
	   }
	}

   class TimeInputVerifier extends InputVerifier
	{
	public boolean shouldYieldFocus(JComponent c)
	   {
	   return verify(c);
	   }

	public boolean verify(JComponent c)
	   {
	   if (c instanceof JFormattedTextField)
		{
		final JFormattedTextField ftf = (JFormattedTextField) c;
		final AbstractFormatter formatter = ftf.getFormatter();
		if (formatter != null)
		   {
		   final String text = ftf.getText();
		   try
			{
			formatter.stringToValue(text);
			return true;
			}
		   catch (ParseException pe)
			{
			SwingUtils.beep();
			return false;
			}
		   }
		}

	   return true;
	   }
	}

   public HTimeField(TimeFormat f)
	{
	this(f, f.getValue());
	}

   public HTimeField(TimeFormat f, String value)
	{
	super(f.getFormat());

	setFocusLostBehavior(COMMIT_OR_REVERT);
	setInputVerifier(new TimeInputVerifier());
	setText(value);
	}
   }
