/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.Insets;
import javax.swing.*;
import de.hillenbrand.system.*;

public class HToolBar extends JToolBar
   {
   public HToolBar()
	{
	}

   public JButton add(Action action, Icon icon)
	{
	return this.add(action, icon, -1);
	}

   public void add(JComponent component, Action action)
	{
	super.add(component, action);

	if (component instanceof JButton)
	   {
	   ((JButton) component).setMargin(new Insets(1, 1, 1, 1));
	   }
	}

   public JButton add(Action action, Icon icon, int position)
	{
	// create button
	final JButton button = new JButton();
	button.addActionListener(action);
	button.setRequestFocusEnabled(false);

	// set icon or text
	if (icon != null && icon.getIconWidth() > 0)
	   {
	   button.setIcon(icon);
	   }
	else
	   {
	   button.setText((String) action.getValue(Action.NAME));
	   }

	// set margin
	button.setMargin(new Insets(1, 1, 1, 1));

	// set tool tip text
	String tip = (String) action.getValue(Action.SHORT_DESCRIPTION);
	if (tip != null)
	   {
	   button.setToolTipText(tip);
	   }

	// add
	if (position < 0)
	   {
	   super.add(button);
	   }
	else
	   {
	   super.add(button, position);
	   }

	return button;
	}
   }
