/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;

public class HTree extends JTree
   {
   private DefaultMutableTreeNode rootNode;
   private DefaultTreeModel htreeModel;

   public HTree(DefaultTreeModel model)
	{
	super(model);

	// init component
	setEditable(true);
	getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	setShowsRootHandles(true);

	// get some variables
	rootNode = (DefaultMutableTreeNode) model.getRoot();
	htreeModel = model;
	}

   public HTree(String rootNodeName)
	{
	this(new DefaultMutableTreeNode(rootNodeName));
	}

   public HTree(TreeNode rootNode)
	{
	this(new DefaultTreeModel(rootNode));
	}

   public String[] add(String data)
	{
	int index = getChildPosition(rootNode, data);
	if (index == -1)
	   {
	   DefaultMutableTreeNode n = new DefaultMutableTreeNode(data);
	   rootNode.insert(n, rootNode.getChildCount());
	   htreeModel.reload();
	   }

	return new String[]
		     {
		     data
		     };
	}

   public String[] add(String[] path, String data)
	{
	DefaultMutableTreeNode node = rootNode;

	for (int i = 0; i < path.length; ++i)
	   {
	   int index = getChildPosition(node, path[i]);
	   if (index == -1)
		{
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(path[i]);
		node.insert(n, node.getChildCount());
		node = n;
		}
	   else
		{
		node = (DefaultMutableTreeNode) node.getChildAt(index);
		}
	   }

	node.insert(new DefaultMutableTreeNode(data), 0);

	htreeModel.reload();

	String[] result = new String[path.length + 1];
	System.arraycopy(path, 0, result, 0, path.length);
	result[path.length] = data;

	return result;
	}

   public void add(String[] path, String[] data)
	{
	DefaultMutableTreeNode node = rootNode;

	for (int i = 0; i < path.length; ++i)
	   {
	   int index = getChildPosition(node, path[i]);
	   if (index == -1)
		{
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(path[i]);
		node.insert(n, node.getChildCount());
		node = n;
		}
	   else
		{
		node = (DefaultMutableTreeNode) node.getChildAt(index);
		}
	   }

	for (int i = 0; i < data.length; ++i)
	   {
	   node.insert(new DefaultMutableTreeNode(data[i]), i);
	   }

	htreeModel.reload();
	}

   public String[] addSorted(String data)
	{
	int index = getChildPosition(rootNode, data);
	if (index == -1)
	   {
	   rootNode.insert(new DefaultMutableTreeNode(data), searchSortedPosition(rootNode, data));
	   htreeModel.reload();
	   }

	return new String[]
		     {
		     data
		     };
	}

   public String[] addSorted(String[] path, String data)
	{
	DefaultMutableTreeNode node = rootNode;

	for (int i = 0; i < path.length; ++i)
	   {
	   int index = getChildPosition(node, path[i]);
	   if (index == -1)
		{
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(path[i]);
		node.insert(n, searchSortedPosition(node, path[i]));
		node = n;
		}
	   else
		{
		node = (DefaultMutableTreeNode) node.getChildAt(index);
		}
	   }

	node.insert(new DefaultMutableTreeNode(data), searchSortedPosition(node, data));

	htreeModel.reload();

	String[] result = new String[path.length + 1];
	System.arraycopy(path, 0, result, 0, path.length);
	result[path.length] = data;

	return result;
	}

   public void addSorted(String[] path, String[] data)
	{
	DefaultMutableTreeNode node = rootNode;

	for (int i = 0; i < path.length; ++i)
	   {
	   int index = getChildPosition(node, path[i]);
	   if (index == -1)
		{
		DefaultMutableTreeNode n = new DefaultMutableTreeNode(path[i]);
		node.insert(n, searchSortedPosition(node, path[i]));
		node = n;
		}
	   else
		{
		node = (DefaultMutableTreeNode) node.getChildAt(index);
		}
	   }

	for (int i = 0; i < data.length; ++i)
	   {
	   node.insert(new DefaultMutableTreeNode(data[i]), searchSortedPosition(node, data[i]));
	   }

	htreeModel.reload();
	}

   public void clear()
	{
	rootNode.removeAllChildren();
	htreeModel.reload();
	}

   private static int getChildPosition(DefaultMutableTreeNode parent, String child)
	{
	int counter = 0;

	if (parent == null)
	   {
	   return -1;
	   }

	for (Enumeration e = parent.children(); e.hasMoreElements();)
	   {
	   DefaultMutableTreeNode node = (DefaultMutableTreeNode) e.nextElement();
	   if (((String) node.getUserObject()).equals(child))
		{
		return counter;
		}
	   ++counter;
	   }

	return -1;
	}

   public String getSelectedItem()
	{
	TreePath path = getSelectionPath();
	if (path == null)
	   {
	   return null;
	   }

	Object[] objects = path.getPath();
	if (!((DefaultMutableTreeNode) objects[objects.length - 1]).isLeaf())
	   {
	   return null;
	   }

	return ((DefaultMutableTreeNode) objects[objects.length - 1]).getUserObject().toString();
	}

   public String getSelectedItemWithFullPath(char separator)
	{
	StringBuffer result = new StringBuffer(100);

	TreePath path = getSelectionPath();
	if (path == null)
	   {
	   return null;
	   }

	Object[] objects = path.getPath();
	if (!((DefaultMutableTreeNode) objects[objects.length - 1]).isLeaf())
	   {
	   return null;
	   }

	for (int i = 1; i < objects.length; ++i)
	   {
	   result.append(((DefaultMutableTreeNode) objects[i]).getUserObject().toString());
	   if (i + 1 < objects.length)
		{
		result.append(separator);
		}
	   }

	return result.toString();
	}

   private static int searchSortedPosition(DefaultMutableTreeNode parent, String name)
	{
	int max = parent.getChildCount();
	int pos = 0;

	for (pos = 0; pos < max; ++pos)
	   {
	   DefaultMutableTreeNode child = (DefaultMutableTreeNode) parent.getChildAt(pos);
	   if (((String) child.getUserObject()).compareTo(name) > 0)
		{
		break;
		}
	   }

	return pos;
	}

   public void expandAll(boolean expand)
	{
	final TreeNode root = (TreeNode) getModel().getRoot();
	expandAll(new TreePath(root), expand);
	}

   private void expandAll(TreePath parent, boolean expand)
	{
	final TreeNode node = (TreeNode) parent.getLastPathComponent();
	if (node.getChildCount() >= 0)
	   {
	   for (Enumeration e = node.children(); e.hasMoreElements();)
		{
		final TreeNode n = (TreeNode) e.nextElement();
		final TreePath path = parent.pathByAddingChild(n);
		expandAll(path, expand);
		}
	   }

	if (expand)
	   {
	   expandPath(parent);
	   }
	else
	   {
	   collapsePath(parent);
	   }
	}

   public void selectRootNode()
	{
	final TreeNode root = (TreeNode) getModel().getRoot();
	final TreePath path = new TreePath(root);
	setSelectionPath(path);
	scrollPathToVisible(path);
	}

   public void selectNode(String name)
	{
	final TreeNode root = (TreeNode) getModel().getRoot();
	if (root.toString().equals(name))
	   {
	   final TreePath path = new TreePath(root);
	   setSelectionPath(path);
	   scrollPathToVisible(path);
	   }
	else
	   {
	   selectNode(new TreePath(root), name);
	   }
	}

   private void selectNode(TreePath parent, String name)
	{
	final TreeNode node = (TreeNode) parent.getLastPathComponent();
	if (node.getChildCount() >= 0)
	   {
	   for (Enumeration e = node.children(); e.hasMoreElements();)
		{
		final TreeNode n = (TreeNode) e.nextElement();
		if (n.toString().equals(name))
		   {
		   final TreePath path = parent.pathByAddingChild(n);
		   setSelectionPath(path);
		   scrollPathToVisible(path);
		   return;
		   }
		else
		   {
		   final TreePath path = parent.pathByAddingChild(n);
		   selectNode(path, name);
		   }
		}
	   }
	}
   }
