/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;
import javax.swing.*;

public class HVerticalTabbedPane extends JTabbedPane
   {
   final boolean vertical;

   private class VerticalTextIcon implements Icon
	{
	private String text;
	private boolean clockwise;

	public VerticalTextIcon(String text, boolean clockwise)
	   {
	   this.text = text;
	   this.clockwise = clockwise;
	   }

	public int getIconHeight()
	   {
	   return getFontMetrics(getFont()).stringWidth(text) + 16;
	   }

	public int getIconWidth()
	   {
	   return 10;
	   }

	public void paintIcon(Component c, Graphics g, int x, int y)
	   {
	   final Graphics2D g2 = (Graphics2D) g;
	   g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	   g2.rotate(Math.toRadians(clockwise ? 90 : -90), x, y);
	   g2.drawString(text, x - (clockwise ? 0 : getIconHeight()) + 8, y + (clockwise ? 0 : getIconWidth()));
	   g2.rotate(Math.toRadians(clockwise ? -90 : 90), x, y);
	   }
	}

   public HVerticalTabbedPane()
	{
	this(JTabbedPane.LEFT);
	}

   public HVerticalTabbedPane(int tabPlacement)
	{
	super(tabPlacement);
	this.vertical = (tabPlacement == LEFT || tabPlacement == RIGHT);
	}

   public void addTab(String title, Component component)
	{
	insertTab(title, null, component, null, getTabCount());
	}

   public void addTab(String title, Icon icon, Component component)
	{
	insertTab(title, icon, component, null, getTabCount());
	}

   public void addTab(String title, Icon icon, Component component, String tip)
	{
	insertTab(title, icon, component, tip, getTabCount());
	}

   public void insertTab(String title, Icon icon, Component component, String tip, int index)
	{
	if (vertical)
	   {
	   if (icon != null)
		{
		System.err.println("ERROR: icon not supported in " + getClass().getName());
		}
	   super.insertTab(null, new VerticalTextIcon(title, getTabPlacement() == RIGHT), component, tip, index);
	   }
	else
	   {
	   super.insertTab(title, icon, component, tip, index);
	   }
	}
   }
