/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing;

import java.awt.*;
import javax.swing.*;

public final class UIStartupWindow extends JWindow
   {
   // components
   private final JLabel image = new MyLabel();

   private final static class MyLabel extends JLabel
	{
	private String text;

	public MyLabel()
	   {
	   setFont(new Font("SansSerif", Font.PLAIN, 12));
	   }

	public void paint(Graphics g)
	   {
	   super.paint(g);

	   if (g instanceof Graphics2D)
		{
		((Graphics2D) g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		((Graphics2D) g).setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		}

	   final FontMetrics fm = g.getFontMetrics();
	   final int stringWidth = fm.stringWidth(text);
	   final int x = (getWidth() - stringWidth) / 2;
	   final int y = getHeight() - 4;

	   g.setColor(getForeground());
	   g.drawString(text, x, y);
	   }

	public void setText(String s)
	   {
	   this.text = s;
	   repaint();
	   }
	}

   public UIStartupWindow(Icon image)
	{
	super();

	// init components
	this.image.setIcon(image);
	this.image.setText(Helper.GRS("UIStartupWindow.Starting"));

	// set content pane
	final JPanel panel = (JPanel) getContentPane();
	panel.setLayout(new BorderLayout());
	panel.setBorder(BorderFactory.createLineBorder(Color.black, 1));
	panel.add(this.image, BorderLayout.CENTER);

	// pack
	this.pack();
	SwingUtils.centerOnPrimaryDisplay(this);

	// set visible
	this.setVisible(true);
	}

   public void setMessage(String text)
	{
	image.setText(text);
	}

   public void setMessageBackgroundColor(Color color)
	{
	image.setBackground(color);
	}

   public void setMessageForegroundColor(Color color)
	{
	image.setForeground(color);
	}
   }
