/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing.border;

import de.hillenbrand.util.*;
import java.awt.*;
import javax.swing.border.*;

/**
* 
* @author Administrator
* 
*/

public class RoundedBorder implements Border {
	private int cornerRadius;
	
	public RoundedBorder()  {
		this(10);
	}

	public RoundedBorder(int cornerRadius)  {
		this.cornerRadius = cornerRadius;
	}

	public Insets getBorderInsets(Component c)  {
		return getBorderInsets(c, new Insets(0,0,0,0));
	}

	public Insets getBorderInsets(Component c, Insets insets)  {
		insets.top = insets.bottom = cornerRadius/2; 
		insets.left = insets.right = 1;
		return insets;
	}

	public boolean isBorderOpaque()  {
		return false;
	}

	public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		final Graphics2D g2 = (Graphics2D)g.create();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		final Color color = ColorUtils.deriveColorHSB(c.getBackground(), 0, 0, -.3f);
		g2.setColor(ColorUtils.deriveColorAlpha(color, 40));        
		g2.drawRoundRect(x, y + 2, width - 1, height - 3, cornerRadius, cornerRadius);
		g2.setColor(ColorUtils.deriveColorAlpha(color, 90));        
		g2.drawRoundRect(x, y + 1, width - 1, height - 2, cornerRadius, cornerRadius); 
		g2.setColor(ColorUtils.deriveColorAlpha(color, 255));        
		g2.drawRoundRect(x, y, width - 1, height - 1, cornerRadius, cornerRadius);
		
		g2.dispose();            
	}

}
