/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing.border;

import de.hillenbrand.system.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

/**
* 
* @author Administrator
* 
*/

public class RoundedTitleBorder extends RoundedBorder {
	private final String title;
	private final Color[] titleGradientColors;
	
	public RoundedTitleBorder(String title)  {
		this(title, UIManager.getColor(UIPreferences.TITLE_GRADIENT_COLOR1_KEY), UIManager.getColor(UIPreferences.TITLE_GRADIENT_COLOR2_KEY));
	}

	public RoundedTitleBorder(String title, Color titleGradientColor1, Color titleGradientColor2)  {
		super(10);
		
		this.title = title;
		this.titleGradientColors = new Color[] { titleGradientColor1, titleGradientColor2 };
	}

	public Insets getBorderInsets(Component c, Insets insets)  {
		final Insets borderInsets = super.getBorderInsets(c, insets);        
		borderInsets.top = getTitleHeight(c);
		return borderInsets;
	}

	protected int getTitleHeight(Component c)  {
		final FontMetrics metrics = c.getFontMetrics(UIManager.getFont(UIPreferences.TITLE_FONT_KEY));
		return (int)(metrics.getHeight() * 1.80);
	}

	public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)  {
		int titleHeight = getTitleHeight(c);
		
		final BufferedImage titleImage = ColorUtils.createTranslucentImage(width, titleHeight);
		final GradientPaint gradient = new GradientPaint(0, 0, titleGradientColors[0], 0, titleHeight, titleGradientColors[1], false);
		
		Graphics2D g2 = (Graphics2D)titleImage.getGraphics();
		g2.setPaint(gradient);
		g2.fillRoundRect(x, y, width, height, 10, 10);
		g2.setColor(ColorUtils.deriveColorHSB(titleGradientColors[1], 0, 0, -.2f));
		g2.drawLine(x + 1, titleHeight - 1, width - 2, titleHeight - 1);
		g2.setColor(ColorUtils.deriveColorHSB(titleGradientColors[1], 0, -.5f, .5f));
		g2.drawLine(x + 1, titleHeight, width - 2, titleHeight);
		g2.setPaint(new GradientPaint(0, 0, new Color(0.0f, 0.0f, 0.0f, 1.0f), width, 0, new Color(0.0f, 0.0f, 0.0f, 0.0f)));        
		g2.setComposite(AlphaComposite.DstIn);
		g2.fillRect(x, y, width, titleHeight);
		g2.dispose();
		
		g.drawImage(titleImage, x, y, c);
		
		// draw rounded border
		super.paintBorder(c, g, x, y, width, height);
		
		// draw title string
		g2 = (Graphics2D)g.create();
		g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		g2.setColor(c.getForeground());
		g2.setFont(UIManager.getFont(UIPreferences.TITLE_FONT_KEY));
		
		final FontMetrics metrics = c.getFontMetrics(UIManager.getFont(UIPreferences.TITLE_FONT_KEY));
		g2.drawString(title, x + 8, y + (titleHeight - metrics.getHeight())/2 + metrics.getAscent()); 
		g2.dispose();
	}

}
