/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing.plaf.metal.themes;

import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.metal.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;

/**
* This class describes a higher-contrast Metal Theme.
* 
* @version 1.10 07/26/04
* @author Michael C. Albers
* 
*/

public class ContrastTheme extends DefaultMetalTheme {
	private final ColorUIResource primary1 = new ColorUIResource(0, 0, 0);
	private final ColorUIResource primary2 = new ColorUIResource(204, 204, 204);
	private final ColorUIResource primary3 = new ColorUIResource(255, 255, 255);
	private final ColorUIResource primaryHighlight = new ColorUIResource(102,102,102);
	private final ColorUIResource secondary2 = new ColorUIResource(204, 204, 204);
	private final ColorUIResource secondary3 = new ColorUIResource(255, 255, 255);
	private final ColorUIResource controlHighlight = new ColorUIResource(102,102,102);
	
	public void addCustomEntriesToTable(UIDefaults table)  {
		Border blackLineBorder = new BorderUIResource(new LineBorder( getBlack() ));
		
		Object textBorder = new BorderUIResource( new CompoundBorder(
		     blackLineBorder,
		            new BasicBorders.MarginBorder()));
		
		table.put( "ToolTip.border", blackLineBorder);
		table.put( "TitledBorder.border", blackLineBorder);
		
		table.put( "TextField.border", textBorder);
		table.put( "PasswordField.border", textBorder);
		table.put( "TextArea.border", textBorder);
		table.put( "TextPane.border", textBorder);
		table.put( "EditorPane.border", textBorder);
		
		
	}

	public ColorUIResource getAcceleratorForeground()  {
		return getBlack(); 
	}

	public ColorUIResource getAcceleratorSelectedForeground()  {
		return getWhite(); 
	}

	public ColorUIResource getControlHighlight()  {
		return super.getSecondary3(); 
	}

	public ColorUIResource getFocusColor()  {
		return getBlack(); 
	}

	public ColorUIResource getHighlightedTextColor()  {
		return getWhite(); 
	}

	public ColorUIResource getMenuSelectedBackground()  {
		return getBlack(); 
	}

	public ColorUIResource getMenuSelectedForeground()  {
		return getWhite(); 
	}

	public String getName()  {
		return "Contrast"; 
	}

	protected ColorUIResource getPrimary1()  {
		return primary1; 
	}

	protected ColorUIResource getPrimary2()  {
		return primary2; 
	}

	protected ColorUIResource getPrimary3()  {
		return primary3; 
	}

	public ColorUIResource getPrimaryControlHighlight()  {
		return primaryHighlight;
	}

	protected ColorUIResource getSecondary2()  {
		return secondary2; 
	}

	protected ColorUIResource getSecondary3()  {
		return secondary3; 
	}

	public ColorUIResource getTextHighlightColor()  {
		return getBlack(); 
	}

}
