/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing.plaf.threeD;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.plaf.threeD.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class HBorderRequester extends OkCancelRequester
   {
   // variables
   protected int borderType = -1;
   // components
   private final JToggleButton[] toggle = new JToggleButton[ThreeDUtils.BORDER_TYPES];

   public HBorderRequester(int bordertype)
	{
	super("3DLF: Borders", "Please choose a border:");

	// init
	this.borderType = bordertype;
	}

   protected void cancel()
	{
	}

   /**
    * Returns the value of the borderType property.
    */
   public int getBorderType()
	{
	return borderType;
	}

   protected JPanel getUserPanel()
	{
	final JPanel result = new JPanel(new GridLayout(0, ThreeDUtils.BORDER_TYPES / 3, 4, 4));

	final ButtonGroup grp = new ButtonGroup();
	for (int i = 0; i < ThreeDUtils.BORDER_TYPES; ++i)
	   {
	   toggle[i] = makeToggleButton(i);
	   result.add(toggle[i]);
	   grp.add(toggle[i]);
	   }

	return result;
	}

   private JToggleButton makeToggleButton(final int t)
	{
	final JToggleButton result = new JToggleButton();

	result.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		borderType = t;
		}
	   });

	result.setIcon(new Icon()
	   {
	   private int w = 100, h = 50;

	   public int getIconWidth()
		{
		return w;
		}

	   public int getIconHeight()
		{
		return h;
		}

	   public void paintIcon(Component c, Graphics g, int x, int y)
		{
		ThreeDUtils.drawBorder(t, g, x, y, w, h);
		}
	   });

	return result;
	}

   protected void ok()
	{
	}

   protected void prepareForRequest()
	{
	toggle[borderType].setSelected(true);
	}

   /**
    * Assigns the value of the borderType property.
    */
   public void setBorderType(int arg)
	{
	borderType = arg;
	}
   }
