/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing.plaf.threeD;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.swing.plaf.threeD.*;
import de.hillenbrand.system.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;

public class ThreeDPrefsFrame extends HFrame {
	// App Prefs
	private static final ApplicationPreferences appPrefs = new ApplicationPreferences("JavaGPE");
	
	// Komponenten
	private final JButton 		save 			= new JButton("Save", HIconFactory.getButtonIcon("ok"));
	private final JButton 		test				= new JButton("Test", HIconFactory.getButtonIcon("apply"));
	private final JButton 		cancel			= new JButton("Cancel", HIconFactory.getButtonIcon("cancel"));
	private final FileRequester	fileRequester	= new FileRequester("Please select a file:"); 
	private final JTabbedPane 	tabbedPane 	= new JTabbedPane();
	
	// System
	private final HFontRequesterButton	font1					= new HFontRequesterButton();
	private final JCheckBox				useAntiAliasing			= new JCheckBox();
	private final HColorRequesterButton	backgroundColor		= new HColorRequesterButton();
	private final HColorRequesterButton	textColor				= new HColorRequesterButton();
	private final HColorRequesterButton	highlightColor			= new HColorRequesterButton();
	private final HColorRequesterButton	focusColor				= new HColorRequesterButton();
	private final HColorRequesterButton	disableColor			= new HColorRequesterButton();
	private final HColorRequesterButton	disabledTextColor		= new HColorRequesterButton();
	
	// Borders
	private final HColorRequesterButton	lightEdgeColor					= new HColorRequesterButton();
	private final HColorRequesterButton	darkEdgeColor					= new HColorRequesterButton();
	private final HBorderRequesterButton	buttonBorderType1				= new HBorderRequesterButton();
	private final HBorderRequesterButton	buttonBorderType2				= new HBorderRequesterButton();
	private final HBorderRequesterButton	checkBoxBorderType1			= new HBorderRequesterButton();
	private final HBorderRequesterButton	checkBoxBorderType2			= new HBorderRequesterButton();
	private final HBorderRequesterButton	comboBoxBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	listBorderType					= new HBorderRequesterButton();
	private final HBorderRequesterButton	menuBorderType					= new HBorderRequesterButton();
	private final HBorderRequesterButton	menuBarBorderType				= new HBorderRequesterButton();
	private final HBorderRequesterButton	menuItemBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	popupMenuBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	progressBarBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	radioButtonBorderType1		= new HBorderRequesterButton();
	private final HBorderRequesterButton	radioButtonBorderType2		= new HBorderRequesterButton();
	private final HBorderRequesterButton	scrollBarThumbBorderType		= new HBorderRequesterButton();
	private final HBorderRequesterButton	scrollBarTrackBorderType		= new HBorderRequesterButton();
	private final HBorderRequesterButton	sliderThumbBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	sliderTrackBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	tableBorderType					= new HBorderRequesterButton();
	private final HBorderRequesterButton	tableHeaderBorderType			= new HBorderRequesterButton();
	private final HBorderRequesterButton	textComponentBorderType1	= new HBorderRequesterButton();
	private final HBorderRequesterButton	textComponentBorderType2	= new HBorderRequesterButton();
	private final HBorderRequesterButton	toggleButtonBorderType1		= new HBorderRequesterButton();
	private final HBorderRequesterButton	toggleButtonBorderType2		= new HBorderRequesterButton();
	private final HBorderRequesterButton	treeBorderType					= new HBorderRequesterButton();
	
	// Desktop
	private final HColorRequesterButton	desktopColor						= new HColorRequesterButton();
	
	// Menus
	private final HColorRequesterButton	menuBackgroundColor			= new HColorRequesterButton();
	private final HColorRequesterButton	menuTextColor					= new HColorRequesterButton();
	private final HColorRequesterButton	menuHighlightColor				= new HColorRequesterButton();
	private final HColorRequesterButton	disabledMenuTextColor			= new HColorRequesterButton();
	private final HFontRequesterButton	font2								= new HFontRequesterButton();
	private final HColorRequesterButton	acceleratorColor					= new HColorRequesterButton();
	
	// ScrollBar
	private final HColorRequesterButton	scrollBarTrackColor				= new HColorRequesterButton();
	private final HColorRequesterButton	scrollBarThumbColor				= new HColorRequesterButton();
	
	// Slider
	private final HColorRequesterButton	sliderTrackColor					= new HColorRequesterButton();
	private final HColorRequesterButton	sliderThumbColor					= new HColorRequesterButton();
	
	// TabbedPane
	private final HColorRequesterButton	tabbedPaneBackgroundColor	= new HColorRequesterButton();
	
	// TextComponents
	private final HColorRequesterButton	textFieldBackgroundColor		= new HColorRequesterButton();
	private final HColorRequesterButton	caretColor							= new HColorRequesterButton();
	private final JComboBox					caretBlinkRate					= new JComboBox(new String [] { "0", "200", "400", "600", "800", "1000" });
	
	// TitledBorders
	private final HFontRequesterButton	font3								= new HFontRequesterButton();
	private final HColorRequesterButton	titledBorderTextColor			= new HColorRequesterButton();
	
	// ToolTips
	private final HColorRequesterButton	toolTipBackgroundColor			= new HColorRequesterButton();
	
	public ThreeDPrefsFrame()  {
		super("Prefs for 3D Look & Feel");
		
		// create and set panel
		setContentPane(createPanel());
		
		// listeners
		addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { appPrefs.save(); System.exit(0); }});
		save.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { setVisible(false); test(); ThreeDPrefs.save(); appPrefs.save(); System.exit(0); }});
		test.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { test(); }});
		cancel.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { appPrefs.save(); System.exit(0); }});
		
		// Menus
		setJMenuBar(createMenuBar());
		
		// Komponenten einrichten
		updateComponents();
		
		// Anzeigen
		pack();
		Dimension size = getSize();
		if (size.width < 500) { size.width = 500; setSize(size); }
		setLocation(100,100);
		
		// Prefs
		appPrefs.add("ThreeDPrefsFrame", this);
		appPrefs.add("ThreeDPrefsFrame.TabbedPane", tabbedPane);
		appPrefs.add("ThreeDPrefsFrame.FileRequester", fileRequester);
	}

	private void about()  {
		JOptionPane.showMessageDialog(this, UIManager.getLookAndFeel().getDescription());
	}

	private JMenuBar createMenuBar()  {
		JMenuBar 		result 	= new JMenuBar();
		JMenu 			menu	= null;
		JMenuItem	item	= null;
		
		// Program
		menu = result.add(new JMenu("Program"));
		
		item = menu.add(new JMenuItem("Open ..."));
		item.setAccelerator(KeyStroke.getKeyStroke("control O"));
		item.setIcon(HIconFactory.getMenuIcon("fileopen"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { load(); }});
		
		item = menu.add(new JMenuItem("Save as ..."));
		item.setAccelerator(KeyStroke.getKeyStroke("control S"));
		item.setIcon(HIconFactory.getMenuIcon("filesave"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { saveAs(); }});
		
		menu.addSeparator();
		item = menu.add(new JMenuItem("About"));
		item.setAccelerator(KeyStroke.getKeyStroke("control A"));
		item.setIcon(HIconFactory.getMenuIcon("info"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { about(); }});
		
		menu.addSeparator();
		item = menu.add(new JMenuItem("Quit"));
		item.setAccelerator(KeyStroke.getKeyStroke("control Q"));
		item.setIcon(HIconFactory.getMenuIcon("exit"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { System.exit(0); }});
		
		// Edit
		menu = result.add(new JMenu("Edit"));
		item = menu.add(new JMenuItem("Reset to Default Values"));
		item.setAccelerator(KeyStroke.getKeyStroke("control D"));
		item.setIcon(HIconFactory.getMenuIcon("fileexport"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { defaultValues(); }});
		
		item = menu.add(new JMenuItem("Reset to Last Saved"));
		item.setAccelerator(KeyStroke.getKeyStroke("control L"));
		item.setIcon(HIconFactory.getMenuIcon("fileimport"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { lastSaved(); }});
		
		return result;
	}

	private JPanel createPanel()  {
		// create prefs panels
		tabbedPane.addTab("System"				, createPanel_System());
		tabbedPane.addTab("Borders"				, createPanel_Borders());
		tabbedPane.addTab("Desktops"			, createPanel_Desktop());
		tabbedPane.addTab("Menus"				, createPanel_Menus());
		tabbedPane.addTab("ScrollBars"			, createPanel_ScrollBar());
		tabbedPane.addTab("Sliders"				, createPanel_Slider());
		tabbedPane.addTab("TabbedPanes"		, createPanel_TabbedPane());
		tabbedPane.addTab("TextComponents"	, createPanel_TextComponents());
		tabbedPane.addTab("TitledBorders"		, createPanel_TitledBorder());
		tabbedPane.addTab("ToolTips"				, createPanel_ToolTips());
		
		// create result panel
		final JPanel panel = new JPanel();
		panel.setBorder(UIPreferences.getFrameBorder());
		panel.setLayout(new GridBagLayout());
		
		// create constraints
		final GridBagConstraints c = UIPreferences.getGridBagConstraints();
		
		// add components
		c.gridx = 0; c.gridy=0; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(tabbedPane, c);
		c.gridx = 0; c.gridy=1; c.gridwidth=4; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(new HSeparator(), c);
		c.gridx = 0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(new JPanel(), c);
		c.gridx = 1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(save, c);
		c.gridx = 2; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(test, c);
		c.gridx = 3; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0; c.fill = GridBagConstraints.HORIZONTAL;
		panel.add(cancel, c);
		
		// return panel
		return panel;
	}

	private JPanel createPanel_Borders()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Light Edge Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(lightEdgeColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Dark Edge Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(darkEdgeColor, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Button (1):"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(buttonBorderType1, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Button (2):"), c);
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(buttonBorderType2, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("CheckBox (1):"), c);
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(checkBoxBorderType1, c);
		
		c.gridx=0; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("CheckBox (2):"), c);
		c.gridx=1; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(checkBoxBorderType2, c);
		
		c.gridx=0; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ComboBox:"), c);
		c.gridx=1; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(comboBoxBorderType, c);
		
		c.gridx=0; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("List:"), c);
		c.gridx=1; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(listBorderType, c);
		
		// Zweite Spalte
		
		c.gridx=2; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("MenuBar:"), c);
		c.gridx=3; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuBarBorderType, c);
		
		c.gridx=2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Menu:"), c);
		c.gridx=3; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuBorderType, c);
		
		c.gridx=2; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("MenuItem:"), c);
		c.gridx=3; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuItemBorderType, c);
		
		c.gridx=2; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("PopupMenu:"), c);
		c.gridx=3; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(popupMenuBorderType, c);
		
		c.gridx=2; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ProgressBar:"), c);
		c.gridx=3; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(progressBarBorderType, c);
		
		c.gridx=2; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("RadioButton (1):"), c);
		c.gridx=3; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(radioButtonBorderType1, c);
		
		c.gridx=2; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("RadioButton (2):"), c);
		c.gridx=3; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(radioButtonBorderType2, c);
		
		c.gridx=2; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ScrollBar Thumb:"), c);
		c.gridx=3; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarThumbBorderType, c);
		
		// Dritte Reihe
		
		c.gridx=4; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ScrollBar Track:"), c);
		c.gridx=5; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarTrackBorderType, c);
		
		c.gridx=4; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Slider Thumb:"), c);
		c.gridx=5; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderThumbBorderType, c);
		
		c.gridx=4; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Slider Track:"), c);
		c.gridx=5; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderTrackBorderType, c);
		
		c.gridx=4; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Table:"), c);
		c.gridx=5; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(tableBorderType, c);
		
		c.gridx=4; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Table Header:"), c);
		c.gridx=5; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(tableHeaderBorderType, c);
		
		c.gridx=4; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("TextComponent (1):"), c);
		c.gridx=5; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textComponentBorderType1, c);
		
		c.gridx=4; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("TextComponent (2):"), c);
		c.gridx=5; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textComponentBorderType2, c);
		
		c.gridx=4; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ToggleButton (1):"), c);
		c.gridx=5; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(toggleButtonBorderType1, c);
		
		// Vierte Zeile
		
		c.gridx=6; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("ToggleButton (2):"), c);
		c.gridx=7; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(toggleButtonBorderType2, c);
		
		c.gridx=6; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Tree:"), c);
		c.gridx=7; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(treeBorderType, c);
		
		return result;
	}

	private JPanel createPanel_Desktop()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Desktop Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(desktopColor, c);
		
		return result;
	}

	private JPanel createPanel_Menus()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuBackgroundColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Text Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuTextColor, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Highlight Color:"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(menuHighlightColor, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Disabled Text Color:"), c);
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(disabledMenuTextColor, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Accelerator Font:"), c);
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(font2, c);
		
		c.gridx=0; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Accelerator Color:"), c);
		c.gridx=1; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(acceleratorColor, c);
		
		return result;
	}

	private JPanel createPanel_ScrollBar()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Track Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarTrackColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Thumb Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(scrollBarThumbColor, c);
		
		return result;
	}

	private JPanel createPanel_Slider()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Track Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderTrackColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Thumb Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(sliderThumbColor, c);
		
		return result;
	}

	private JPanel createPanel_System()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Font for Components:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(font1, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Use Font Antialiasing:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(useAntiAliasing, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(backgroundColor, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Text Color:"), c);
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textColor, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Highlight Color:"), c);
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(highlightColor, c);
		
		c.gridx=0; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Focus Color:"), c);
		c.gridx=1; c.gridy=5; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(focusColor, c);
		
		c.gridx=0; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Disable Color:"), c);
		c.gridx=1; c.gridy=6; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(disableColor, c);
		
		c.gridx=0; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Disabled Text Color:"), c);
		c.gridx=1; c.gridy=7; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(disabledTextColor, c);
		
		return result;
	}

	private JPanel createPanel_TabbedPane()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Color for unselected Tabs:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(tabbedPaneBackgroundColor, c);
		
		return result;
	}

	private JPanel createPanel_TextComponents()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textFieldBackgroundColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Caret Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(caretColor, c);
		
		caretBlinkRate.setEditable(true);
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Caret Blink Rate:"), c);
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(caretBlinkRate, c);
		
		return result;
	}

	private JPanel createPanel_TitledBorder()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Titled Border Font:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(font3, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Titled Border Text Color:"), c);
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(titledBorderTextColor, c);
		
		return result;
	}

	private JPanel createPanel_ToolTips()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel("Background Color:"), c);
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(toolTipBackgroundColor, c);
		
		return result;
	}

	private void defaultValues()  {
		ThreeDPrefs.resetToDefaultValues();
		updateComponents();
		updateLookAndFeel();
	}

	private void lastSaved()  {
		try
			{
			ThreeDPrefs.resetToLastSaved();
			}
		catch (IOException x) { ThreeDPrefs.resetToDefaultValues(); }
		
		updateComponents();
		updateLookAndFeel();
	}

	private void load()  {
		// init requester
		fileRequester.setLoadMode();
		
		// request
		if (fileRequester.request(this) != FileRequester.OK) return;
		
		// and load the properties
		final File file = fileRequester.getFile();
		try
			{
			ThreeDPrefs.load(file.getAbsolutePath());
			}
		catch (Exception x) 
			{ 
			JReqTools.showException(this, x);
			ThreeDPrefs.resetToDefaultValues();
			}
		
		updateComponents();
		updateLookAndFeel();
	}

	public static void main(String [] args)  {
		// prefs
		UIPreferences.init();
		
		// only english
		Locale.setDefault(Locale.ENGLISH);
		
		try 
			{
			UIManager.installLookAndFeel("3D Look & Feel", "de.hillenbrand.swing.plaf.threeD.ThreeDLookAndFeel");
			UIManager.setLookAndFeel("de.hillenbrand.swing.plaf.threeD.ThreeDLookAndFeel");
			} 
		catch (Exception exc) 
			{
			System.err.println("Error loading L&F: " + exc);
			}
		
		final HFrame frame = new ThreeDPrefsFrame();
		frame.setVisible(true);
	}

	private void saveAs()  {
		// init requester
		fileRequester.setSaveMode();
		
		// request
		if (fileRequester.request(this) != FileRequester.OK) return;
		
		// and save properties
		final File file = fileRequester.getFile();
		try
			{
			ThreeDPrefs.save(file.getAbsolutePath());
			}
		catch (Exception x) { JReqTools.showException(this, x); } 
	}

	private void test()  {
		// System
		ThreeDPrefs.normalFont 					= new FontUIResource(font1.getFont());
		ThreeDPrefs.useAntiAliasing				= useAntiAliasing.isSelected();
		ThreeDPrefs.backgroundColor				= new ColorUIResource(backgroundColor.getColor());
		ThreeDPrefs.textColor						= new ColorUIResource(textColor.getColor());
		ThreeDPrefs.highlightColor					= new ColorUIResource(highlightColor.getColor());
		ThreeDPrefs.focusColor						= new ColorUIResource(focusColor.getColor());
		ThreeDPrefs.disableColor					= new ColorUIResource(disableColor.getColor());
		ThreeDPrefs.disabledTextColor				= new ColorUIResource(disabledTextColor.getColor());
		
		// Borders
		ThreeDPrefs.lightEdgeColor					= new ColorUIResource(lightEdgeColor.getColor());
		ThreeDPrefs.darkEdgeColor					= new ColorUIResource(darkEdgeColor.getColor());
		ThreeDPrefs.buttonBorderType1			= buttonBorderType1.getBorderType();
		ThreeDPrefs.buttonBorderType2			= buttonBorderType2.getBorderType();
		ThreeDPrefs.checkBoxBorderType1			= checkBoxBorderType1.getBorderType();
		ThreeDPrefs.checkBoxBorderType2			= checkBoxBorderType2.getBorderType();
		ThreeDPrefs.comboBoxBorderType			= comboBoxBorderType.getBorderType();
		ThreeDPrefs.listBorderType					= listBorderType.getBorderType();
		ThreeDPrefs.menuBorderType				= menuBorderType.getBorderType();
		ThreeDPrefs.menuBarBorderType			= menuBarBorderType.getBorderType();
		ThreeDPrefs.menuItemBorderType			= menuItemBorderType.getBorderType();
		ThreeDPrefs.popupMenuBorderType		= popupMenuBorderType.getBorderType();
		ThreeDPrefs.progressBarBorderType		= progressBarBorderType.getBorderType();
		ThreeDPrefs.radioButtonBorderType1		= radioButtonBorderType1.getBorderType();
		ThreeDPrefs.radioButtonBorderType2		= radioButtonBorderType2.getBorderType();
		ThreeDPrefs.scrollBarThumbBorderType	= scrollBarThumbBorderType.getBorderType();
		ThreeDPrefs.scrollBarTrackBorderType		= scrollBarTrackBorderType.getBorderType();
		ThreeDPrefs.sliderThumbBorderType		= sliderThumbBorderType.getBorderType();
		ThreeDPrefs.sliderTrackBorderType		= sliderTrackBorderType.getBorderType();
		ThreeDPrefs.tableBorderType				= tableBorderType.getBorderType();
		ThreeDPrefs.tableHeaderBorderType		= tableHeaderBorderType.getBorderType();
		ThreeDPrefs.textComponentBorderType1	= textComponentBorderType1.getBorderType();
		ThreeDPrefs.textComponentBorderType2	= textComponentBorderType2.getBorderType();
		ThreeDPrefs.toggleButtonBorderType1	= toggleButtonBorderType1.getBorderType();
		ThreeDPrefs.toggleButtonBorderType2	= toggleButtonBorderType2.getBorderType();
		ThreeDPrefs.treeBorderType				= treeBorderType.getBorderType();
		
		// Desktop
		ThreeDPrefs.desktopColor					= new ColorUIResource(desktopColor.getColor());
		
		// Menus
		ThreeDPrefs.menuBackgroundColor		= new ColorUIResource(menuBackgroundColor.getColor());
		ThreeDPrefs.menuTextColor				= new ColorUIResource(menuTextColor.getColor());
		ThreeDPrefs.menuHighlightColor			= new ColorUIResource(menuHighlightColor.getColor());
		ThreeDPrefs.disabledMenuTextColor		= new ColorUIResource(disabledMenuTextColor.getColor());
		ThreeDPrefs.acceleratorFont 				= new FontUIResource(font2.getFont());
		ThreeDPrefs.acceleratorColor				= new ColorUIResource(acceleratorColor.getColor());
		
		// ScrollBar
		ThreeDPrefs.scrollBarTrackColor			= new ColorUIResource(scrollBarTrackColor.getColor());
		ThreeDPrefs.scrollBarThumbColor			= new ColorUIResource(scrollBarThumbColor.getColor());
		
		// Slider
		ThreeDPrefs.sliderTrackColor				= new ColorUIResource(sliderTrackColor.getColor());
		ThreeDPrefs.sliderThumbColor				= new ColorUIResource(sliderThumbColor.getColor());
		
		// TabbedPane
		ThreeDPrefs.tabbedPaneBackgroundColor	= new ColorUIResource(tabbedPaneBackgroundColor.getColor());
		
		// TextComponents
		ThreeDPrefs.textFieldBackgroundColor	= new ColorUIResource(textFieldBackgroundColor.getColor());
		ThreeDPrefs.caretColor						= new ColorUIResource(caretColor.getColor());
		ThreeDPrefs.caretBlinkRate					= Integer.parseInt((String)caretBlinkRate.getSelectedItem());
		
		// TitledBorders
		ThreeDPrefs.titledBorderFont 				= new FontUIResource(font3.getFont());
		ThreeDPrefs.titledBorderTextColor			= new ColorUIResource(titledBorderTextColor.getColor());
		
		// ToolTips
		ThreeDPrefs.toolTipBackgroundColor		= new ColorUIResource(toolTipBackgroundColor.getColor());
		
		// Setzen
		updateLookAndFeel();
	}

	private void updateComponents()  {
		// System
		font1.setFont(ThreeDPrefs.normalFont);
		useAntiAliasing.setSelected(ThreeDPrefs.useAntiAliasing);
		backgroundColor.setColor(ThreeDPrefs.backgroundColor);
		textColor.setColor(ThreeDPrefs.textColor);
		highlightColor.setColor(ThreeDPrefs.highlightColor);
		focusColor.setColor(ThreeDPrefs.focusColor);
		disableColor.setColor(ThreeDPrefs.disableColor);
		disabledTextColor.setColor(ThreeDPrefs.disabledTextColor);
		
		// Borders
		lightEdgeColor.setColor(ThreeDPrefs.lightEdgeColor);
		darkEdgeColor.setColor(ThreeDPrefs.darkEdgeColor);
		buttonBorderType1.setBorderType(ThreeDPrefs.buttonBorderType1);
		buttonBorderType2.setBorderType(ThreeDPrefs.buttonBorderType2);
		checkBoxBorderType1.setBorderType(ThreeDPrefs.checkBoxBorderType1);
		checkBoxBorderType2.setBorderType(ThreeDPrefs.checkBoxBorderType2);
		comboBoxBorderType.setBorderType(ThreeDPrefs.comboBoxBorderType);
		listBorderType.setBorderType(ThreeDPrefs.listBorderType);
		menuBorderType.setBorderType(ThreeDPrefs.menuBorderType);
		menuBarBorderType.setBorderType(ThreeDPrefs.menuBarBorderType);
		menuItemBorderType.setBorderType(ThreeDPrefs.menuItemBorderType);
		popupMenuBorderType.setBorderType(ThreeDPrefs.popupMenuBorderType);
		progressBarBorderType.setBorderType(ThreeDPrefs.progressBarBorderType);
		radioButtonBorderType1.setBorderType(ThreeDPrefs.radioButtonBorderType1);
		radioButtonBorderType2.setBorderType(ThreeDPrefs.radioButtonBorderType2);
		scrollBarThumbBorderType.setBorderType(ThreeDPrefs.scrollBarThumbBorderType);
		scrollBarTrackBorderType.setBorderType(ThreeDPrefs.scrollBarTrackBorderType);
		sliderThumbBorderType.setBorderType(ThreeDPrefs.sliderThumbBorderType);
		sliderTrackBorderType.setBorderType(ThreeDPrefs.sliderTrackBorderType);
		tableBorderType.setBorderType(ThreeDPrefs.tableBorderType);
		tableHeaderBorderType.setBorderType(ThreeDPrefs.tableHeaderBorderType);
		textComponentBorderType1.setBorderType(ThreeDPrefs.textComponentBorderType1);
		textComponentBorderType2.setBorderType(ThreeDPrefs.textComponentBorderType2);
		toggleButtonBorderType1.setBorderType(ThreeDPrefs.toggleButtonBorderType1);
		toggleButtonBorderType2.setBorderType(ThreeDPrefs.toggleButtonBorderType2);
		treeBorderType.setBorderType(ThreeDPrefs.treeBorderType);
		
		// Desktop
		desktopColor.setColor(ThreeDPrefs.desktopColor);
		
		// Menus
		menuBackgroundColor.setColor(ThreeDPrefs.menuBackgroundColor);
		menuTextColor.setColor(ThreeDPrefs.menuTextColor);
		menuHighlightColor.setColor(ThreeDPrefs.menuHighlightColor);
		disabledMenuTextColor.setColor(ThreeDPrefs.disabledMenuTextColor);
		font2.setFont(ThreeDPrefs.acceleratorFont);
		acceleratorColor.setColor(ThreeDPrefs.acceleratorColor);
		
		// Slider
		scrollBarTrackColor	.setColor(ThreeDPrefs.scrollBarTrackColor);
		scrollBarThumbColor.setColor(ThreeDPrefs.scrollBarThumbColor);
		
		// Slider
		sliderTrackColor	.setColor(ThreeDPrefs.sliderTrackColor);
		sliderThumbColor.setColor(ThreeDPrefs.sliderThumbColor);
		
		// TabbedPane
		tabbedPaneBackgroundColor.setColor(ThreeDPrefs.tabbedPaneBackgroundColor);
		
		// TextComponents
		textFieldBackgroundColor.setColor(ThreeDPrefs.textFieldBackgroundColor);
		caretColor.setColor(ThreeDPrefs.caretColor);
		caretBlinkRate.setSelectedItem(""+ThreeDPrefs.caretBlinkRate);
		caretBlinkRate.getEditor().setItem(""+ThreeDPrefs.caretBlinkRate);
		
		// TitledBorders
		font3.setFont(ThreeDPrefs.titledBorderFont);
		titledBorderTextColor.setColor(ThreeDPrefs.titledBorderTextColor);
		
		// ToolTips
		toolTipBackgroundColor.setColor(ThreeDPrefs.toolTipBackgroundColor);
	}

	private void updateLookAndFeel()  {
		try 
			{
			UIManager.setLookAndFeel("de.hillenbrand.swing.plaf.threeD.ThreeDLookAndFeel");
			SwingUtilities.updateComponentTreeUI(SwingUtilities.getRoot(this));
			Dimension preferred = getPreferredSize();
			Dimension size = getSize();
			if (preferred.width > size.width) size.width = preferred.width;
			if (preferred.height > size.height) size.height = preferred.height;
			setSize(size);
			} 
		catch (Exception exc) { System.err.println(exc); }
	}

}
