/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.swing.renderer;

import de.hillenbrand.swing.*;
import java.lang.reflect.*;
import javax.swing.JList;

public class HJavaMethodListCellRenderer extends MultiLineListCellRenderer<Method>
   {
   public HJavaMethodListCellRenderer()
	{
	super(HIconFactory.getMenuIcon("palette_color"), Helper.GRS("HJavaMethodListCellRenderer.Signature"), Helper.GRS("HJavaMethodListCellRenderer.DeclaringClass"));
	}

   private String createParameters(Class[] p)
	{
	try
	   {
	   String result = "";
	   for (int i = 0; i < p.length; ++i)
		{
		result += p[i].getCanonicalName();
		if (i + 1 < p.length)
		   {
		   result += ", ";
		   }
		}
	   return result;
	   }
	catch (java.lang.NoClassDefFoundError e)
	   {
	   return "class could not be loaded";
	   }
	}

   /**
    * The title (i.e. the topmost line) of the entry.
    *
    * @param	value the value currently displayed
    */
   @Override
   protected String getTitle(Method value)
	{
	return value.getName();
	}

   /**
    * The tag's value on position pos.
    *
    * @param	pos tag position
    * @param	value the currently selected value
    */
   @Override
   protected String getValue(int pos, Method value)
	{
	switch (pos)
	   {
	   case 0:
		return Modifier.toString(value.getModifiers())
			  + " " + value.getReturnType().getCanonicalName()
			  + " " + value.getName()
			  + "(" + createParameters(value.getParameterTypes()) + ")";
	   case 1:
		return value.getDeclaringClass().getName();
	   }
	return "n/a";
	}

   @Override
   protected void formatListCellRendererComponent(JList list, Method value, int index, boolean isSelected, boolean cellHasFocus)
	{
	}
   }
