/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package de.hillenbrand.swing.text;

import javax.swing.text.*;

/**
* This kit supports a fairly minimal handling of
* editing java text content.  It supports syntax
* highlighting and produces the lexical structure
* of the document as best it can.
* 
* @author  Timothy Prinzing
* @version 1.2 05/04/98
*/

public class JavaEditorKit extends StyledEditorKit {
	JavaContext preferences;
	
	public JavaEditorKit()  {
		super();
	}

	/**
	* Create a copy of the editor kit.  This
	* allows an implementation to serve as a prototype
	* for others, so that they can be quickly created.
	*/

	public Object clone()  {
		JavaEditorKit kit = new JavaEditorKit();
		kit.preferences = preferences;
		return kit;
	}

	/**
	* Creates an uninitialized text storage model
	* that is appropriate for this type of editor.
	* 
	* @return the model
	*/

	public Document createDefaultDocument()  {
		return new JavaDocument();
	}

	/**
	* Get the MIME type of the data that this
	* kit represents support for.  This kit supports
	* the type <code>text/java</code>.
	*/

	public String getContentType()  {
		return "text/java";
	}

	public JavaContext getStylePreferences()  {
		if (preferences == null) preferences = new JavaContext();
		return preferences;
	}

	/**
	* Fetches a factory that is suitable for producing
	* views of any models that are produced by this
	* kit.  The default is to have the UI produce the
	* factory, so this method has no implementation.
	* 
	* @return the view factory
	*/

	public final ViewFactory getViewFactory()  {
		return getStylePreferences();
	}

	public void setStylePreferences(JavaContext prefs)  {
		preferences = prefs;
	}

}
