/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.hillenbrand.swing.text;

import java.text.ParseException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;
import javax.swing.text.DefaultFormatter;

/**
 * A regular expression based implementation of <code>AbstractFormatter</code>.
 */
public class RegexFormatter extends DefaultFormatter
   {
   protected Pattern pattern;

   /**
    * Creates a regular expression based <code>AbstractFormatter</code>.
    * <code>pattern</code> specifies the regular expression that will
    * be used to determine if a value is legal.
    * @param pattern 
    * @throws PatternSyntaxException
    */
   public RegexFormatter(String pattern) throws PatternSyntaxException
	{
	setPattern(Pattern.compile(pattern));
	setOverwriteMode(false);
	}

   /**
    * Creates a regular expression based <code>AbstractFormatter</code>.
    * <code>pattern</code> specifies the regular expression that will
    * be used to determine if a value is legal.
    * @param pattern
    */
   public RegexFormatter(Pattern pattern)
	{
	setPattern(pattern);
	setOverwriteMode(false);
	}

   /**
    * Sets the pattern that will be used to determine if a value is
    * legal.
    * @param pattern
    */
   public void setPattern(Pattern pattern)
	{
	this.pattern = pattern;
	}

   /**
    * Returns the <code>Pattern</code> used to determine if a value is
    * legal.
    * @return
    */
   public Pattern getPattern()
	{
	return pattern;
	}

   /**
    * Parses <code>text</code> returning an arbitrary Object. Some
    * formatters may return null.
    *
    * @throws ParseException if there is an error in the conversion
    * @param text String to convert
    * @return Object representation of text
    */
   public Object stringToValue(String text) throws ParseException
	{
	if (pattern != null)
	   {
	   final Matcher m = pattern.matcher(text);
	   if (m.matches())
		{
		return super.stringToValue(text);
		}

	   throw new ParseException("Pattern did not match", 0);
	   }
	else
	   {
	   return text;
	   }
	}
   }
