/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.system;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;

public final class ApplicationPreferences
   {
   private static final boolean DEBUG = false;
   // Variables
   private final Properties properties = new Properties();
   private final Hashtable<String, Object> id2object = new Hashtable<String, Object>();
   // Property: applicationName
   private String applicationName;

   public ApplicationPreferences(String applicationName)
	{
	this.applicationName = applicationName;
	load();
	}

   public void add(String id, Object object)
	{
	if (object == null || id == null)
	   {
	   return;
	   }

	// In die Tabelle eintragen
	id2object.put(id, object);

	try
	   {
	   // Nachschauen, ob Daten vorliegen
	   if (object instanceof JFrame)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JFrame with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((JFrame) object).setBounds(bounds);
		   }
		if (object instanceof TipOfTheDay)
		   {
		   if (DEBUG)
			{
			Logger.log("ApplicationPreferences.add(): setting values for TipOfTheDay with id " + id);
			}
		   Boolean next = getBoolean(id, "showNextTime");
		   if (next != null)
			{
			((TipOfTheDay) object).setShowNextTime(next.booleanValue());
			}
		   Integer tip = getInteger(id, "tipNumber");
		   if (tip != null)
			{
			((TipOfTheDay) object).setTip(tip.intValue());
			}
		   }
		}
	   else if (object instanceof JInternalFrame)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JInternalFrame with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((JInternalFrame) object).setBounds(bounds);
		   }
		//Boolean visible = getBoolean(id, "visible");
		//if (visible != null && visible.booleanValue()) ((JInternalFrame)object).setVisible(true);
		}
	   else if (object instanceof JDialog)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JDialog with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((JDialog) object).setBounds(bounds);
		   }
		}
	   else if (object instanceof JTextComponent)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JTextComponent with id " + id);
		   }
		String text = getText(id, "text");
		if (text != null)
		   {
		   ((JTextComponent) object).setText(text);
		   }
		}
	   else if (object instanceof JCheckBox)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JCheckBox with id " + id);
		   }
		Boolean selected = getBoolean(id, "selected");
		if (selected != null)
		   {
		   ((JCheckBox) object).setSelected(selected.booleanValue());
		   }
		}
	   else if (object instanceof JCheckBoxMenuItem)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JCheckBoxMenuItem with id " + id);
		   }
		Boolean selected = getBoolean(id, "selected");
		if (selected != null)
		   {
		   ((JCheckBoxMenuItem) object).setSelected(selected.booleanValue());
		   }
		}
	   else if (object instanceof JToggleButton)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JToggleButton with id " + id);
		   }
		Boolean selected = getBoolean(id, "selected");
		if (selected != null)
		   {
		   ((JToggleButton) object).setSelected(selected.booleanValue());
		   }
		}
	   else if (object instanceof HFileChooser)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for HFileChooser with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((HFileChooser) object).setDialogBounds(bounds);
		   }
		File file = getFile(id, "file");
		if (file != null)
		   {
		   ((HFileChooser) object).setSelectedFile(file);
		   }
		}
	   else if (object instanceof HFileString)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for HFileString with id " + id);
		   }
		File file = getFile(id, "file");
		if (file != null)
		   {
		   ((HFileString) object).setFile(file);
		   }
		add(id + ".FileRequester", ((HFileString) object).getFileRequester());
		}
	   else if (object instanceof JFileChooser)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JFileChooser with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((JFileChooser) object).setBounds(bounds);
		   }
		File file = getFile(id, "file");
		if (file != null)
		   {
		   ((JFileChooser) object).setSelectedFile(file);
		   }
		}
	   else if (object instanceof JSplitPane)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JSplitPane with id " + id);
		   }
		Integer location = getInteger(id, "deviderLocation");
		if (location != null)
		   {
		   ((JSplitPane) object).setDividerLocation(location.intValue());
		   }
		}
	   else if (object instanceof JSlider)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JSlider with id " + id);
		   }
		Integer value = getInteger(id, "value");
		if (value != null)
		   {
		   ((JSlider) object).setValue(value.intValue());
		   }
		}
	   else if (object instanceof JTabbedPane)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JTabbedPane with id " + id);
		   }
		Integer selected = getInteger(id, "selectedTab");
		if (selected != null && ((JTabbedPane) object).getTabCount() > selected.intValue())
		   {
		   ((JTabbedPane) object).setSelectedIndex(selected.intValue());
		   }
		}
	   else if (object instanceof JComboBox)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JComboBox with id " + id);
		   }
		final JComboBox cb = (JComboBox) object;
		if (cb.isEditable())
		   {
		   cb.setSelectedItem(getText(id, "selectedItem"));
		   }
		else
		   {
		   cb.setSelectedIndex(getInteger(id, "selectedIndex"));
		   }
		}
	   else if (object instanceof JList)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for JList with id " + id);
		   }
		final int[] select = getIntArray(id + ".selectedIndices");
		((JList) object).setSelectedIndices(select);
		}
	   else if (object instanceof HEditableList)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for HEditableList with id " + id);
		   }
		final String[] data = getStringArray(id, "items");
		((HEditableList) object).set(data);
		}
	   else if (object instanceof HItemList)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for HItemList with id " + id);
		   }
		final String[] data = getStringArray(id, "items");
		((HItemList) object).setItemsAsStringArray(data);
		}
	   else if (object instanceof HTree)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for HTree with id " + id);
		   }
		final String s = getText(id, "selected");
		if (s != null)
		   {
		   ((HTree) object).selectNode(s);
		   }
		}
	   else if (object instanceof FileRequester)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for FileRequester with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((FileRequester) object).setBounds(bounds);
		   }
		File file = getFile(id, "file");
		if (file != null)
		   {
		   ((FileRequester) object).setFile(file);
		   }
		Boolean fileHidingEnabled = getBoolean(id, "fileHidingEnabled");
		if (fileHidingEnabled != null)
		   {
		   ((FileRequester) object).setFileHidingEnabled(fileHidingEnabled.booleanValue());
		   }
		}
	   else if (object instanceof AbstractRequester)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for AbstractRequester with id " + id);
		   }
		Rectangle bounds = getBounds(id, "bounds");
		if (bounds != null)
		   {
		   ((AbstractRequester) object).setBounds(bounds);
		   }
		}
	   else if (object instanceof Hashtable)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for Hashtable with id " + id);
		   }
		Hashtable ht = getHashtable(id);
		if (ht != null)
		   {
		   ((Hashtable) object).clear();
		   for (Enumeration enm = ht.keys(); enm.hasMoreElements();)
			{
			final Object key = enm.nextElement();
			((Hashtable) object).put(key, ht.get(key));
			}
		   }
		}
	   else if (object instanceof Vector)
		{
		if (DEBUG)
		   {
		   Logger.log("ApplicationPreferences.add(): setting values for Vector with id " + id);
		   }
		Vector v = getVector(id);
		if (v != null)
		   {
		   ((Vector) object).clear();
		   for (int i = 0; i < v.size(); ++i)
			{
			((Vector) object).addElement(v.elementAt(i));
			}
		   }
		}
	   else
		{
		System.err.println("ApplicationPreferences.add(): not supported: " + object.getClass().getName());
		}
	   }
	catch (RuntimeException x)
	   {
	   x.printStackTrace();
	   }
	}

   private String createBinaryPrefsFilename(String id)
	{
	final String dirname = System.getProperty("user.home") + File.separator + "." + applicationName;
	final String filename = dirname + File.separator + id + ".serialized";
	final File directory = new File(dirname);

	if (directory.isFile())
	   {
	   directory.delete();
	   }
	directory.mkdirs();

	return filename;
	}

   private String createPrefsFilename()
	{
	final String dirname = System.getProperty("user.home") + File.separator + "." + applicationName;
	final String filename = dirname + File.separator + "prefs.properties";
	final File directory = new File(dirname);

	if (directory.isFile())
	   {
	   directory.delete();
	   }
	directory.mkdirs();

	return filename;
	}

   private Object deserializeObject(String id)
	{
	Object result = null;

	try
	   {
	   final String filename = createBinaryPrefsFilename(id);
	   final ObjectInputStream in = new ObjectInputStream(new FileInputStream(filename));
	   result = in.readObject();
	   in.close();
	   }
	catch (FileNotFoundException x1)
	   {
	   }
	catch (ClassNotFoundException x2)
	   {
	   System.err.println(x2);
	   }
	catch (IOException x3)
	   {
	   System.err.println(x3);
	   }

	return result;
	}

   /**
    * Returns the value of the applicationName property.
    * @return 
    */
   public String getApplicationName()
	{
	return applicationName;
	}

   Boolean getBoolean(String id, String name)
	{
	String s = properties.getProperty(id + "." + name);
	return s != null ? new Boolean(s) : null;
	}

   Rectangle getBounds(String id, String name)
	{
	String s = properties.getProperty(id + "." + name);
	if (s == null)
	   {
	   return null;
	   }

	StringTokenizer tok = new StringTokenizer(s, ",");

	int x = Integer.parseInt(tok.nextToken());
	int y = Integer.parseInt(tok.nextToken());
	int w = Integer.parseInt(tok.nextToken());
	int h = Integer.parseInt(tok.nextToken());

	String version = System.getProperty("java.fullversion");
	if (version != null && version.startsWith("J2RE 1.3.0 IBM build cx130-20010207"))
	   {
	   // Fehlerkorrektur bei der Fensterpositionierung
	   x -= 5;
	   y -= 44;
	   }
	else if (version != null && version.startsWith("J2RE 1.3.0 IBM build cx130-20010626"))
	   {
	   // Fehlerkorrektur bei der Fensterpositionierung
	   x -= 5;
	   y -= 44;
	   }

	return new Rectangle(x, y, w, h);
	}

   Color getColor(String id, String name)
	{
	String s = properties.getProperty(id + "." + name);
	if (s == null)
	   {
	   return null;
	   }

	final StringTokenizer tok = new StringTokenizer(s, ",");

	final int r = Integer.parseInt(tok.nextToken());
	final int g = Integer.parseInt(tok.nextToken());
	final int b = Integer.parseInt(tok.nextToken());

	return new Color(r, g, b);
	}

   File getFile(String id, String name)
	{
	String s = properties.getProperty(id + "." + name);
	return s != null ? new File(s) : null;
	}

   Font getFont(String id, String name)
	{
	String s = properties.getProperty(id + "." + name);
	if (s == null)
	   {
	   return null;
	   }

	final StringTokenizer tok = new StringTokenizer(s, ",");

	final String fname = tok.nextToken();
	final int size = Integer.parseInt(tok.nextToken());

	int style = Font.PLAIN;
	if (tok.hasMoreTokens())
	   {
	   s = tok.nextToken();
	   if (s.equals("bi"))
		{
		style = Font.ITALIC + Font.BOLD;
		}
	   else if (s.equals("b"))
		{
		style = Font.BOLD;
		}
	   else if (s.equals("i"))
		{
		style = Font.ITALIC;
		}
	   }

	return new Font(fname, style, size);
	}

   Hashtable getHashtable(String id)
	{
	return (Hashtable) deserializeObject(id);
	}

   public String[] getStringArray(String id, String name)
	{
	final String s = properties.getProperty(id + "." + name);
	if (s == null)
	   {
	   return new String[0];
	   }

	final StringTokenizer tok = new StringTokenizer(s, "\0");
	final String[] result = new String[tok.countTokens()];
	for (int i = 0; i < result.length; ++i)
	   {
	   result[i] = tok.nextToken();
	   }

	return result;
	}

   public int[] getIntArray(String id)
	{
	final String s = properties.getProperty(id);
	if (s == null)
	   {
	   return new int[0];
	   }

	final StringTokenizer tok = new StringTokenizer(s, ",|");
	final int[] result = new int[tok.countTokens()];
	for (int i = 0; i < result.length; ++i)
	   {
	   result[i] = Integer.parseInt(tok.nextToken());
	   }

	return result;
	}

   Integer getInteger(String id, String name)
	{
	String s = properties.getProperty(id + "." + name);
	return s != null ? new Integer(s) : null;
	}

   public String getProperty(String id, String defaultValue)
	{
	return properties.getProperty(id, defaultValue);
	}

   Rectangle getRectangle(String id, String name)
	{
	return getBounds(id, name);
	}

   String getText(String id, String name)
	{
	return properties.getProperty(id + "." + name);
	}

   Vector getVector(String id)
	{
	return (Vector) deserializeObject(id);
	}

   private void load()
	{
	try
	   {
	   final String filename = createPrefsFilename();
	   if (DEBUG)
		{
		Logger.log("reading prefs from " + filename);
		}
	   final InputStream in = new FileInputStream(filename);
	   properties.load(in);
	   in.close();
	   }
	catch (FileNotFoundException x1)
	   {
	   }
	catch (IOException x2)
	   {
	   System.err.println(x2);
	   }
	}

   public void save()
	{
	// Daten in die Properties übernehmen
	storeData();

	// Abspeichern
	try
	   {
	   final String filename = createPrefsFilename();
	   if (DEBUG)
		{
		Logger.log("saving prefs to " + filename);
		}
	   final OutputStream out = new FileOutputStream(filename);
	   properties.store(out, "");
	   out.close();
	   }
	catch (IOException x2)
	   {
	   System.err.println(x2);
	   }
	}

   private void serializeObject(String id, Object object)
	{
	if (DEBUG)
	   {
	   System.out.println("[ApplicationPreferences.serializeObject(): serializing '" + id + "' (" + object.getClass().getName() + ")]");
	   }

	try
	   {
	   final String filename = createBinaryPrefsFilename(id);
	   if (DEBUG)
		{
		System.out.println("[ApplicationPreferences.serializeObject(): serializing to '" + filename + "']");
		}
	   final ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(filename));
	   os.writeObject(object);
	   os.close();
	   }
	catch (IOException x)
	   {
	   System.err.println(x);
	   }
	}

   void setBoolean(String id, String name, Boolean value)
	{
	if (value == null)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   properties.put(id + "." + name, value.toString());
	   }
	}

   void setBoolean(String id, String name, boolean value)
	{
	setBoolean(id, name, value ? Boolean.TRUE : Boolean.FALSE);
	}

   void setBounds(String id, String name, Rectangle bounds)
	{
	if (bounds == null)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   properties.put(id + "." + name, bounds.x + "," + bounds.y + "," + bounds.width + "," + bounds.height);
	   }
	}

   void setColor(String id, String name, Color color)
	{
	if (color == null)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   properties.put(id + "." + name, color.getRed() + "," + color.getGreen() + "," + color.getBlue());
	   }
	}

   void setFile(String id, String name, File file)
	{
	if (file == null)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   properties.put(id + "." + name, file.getAbsolutePath());
	   }
	}

   void setFiles(String id, String name, File[] files)
	{
	if (files == null || files.length == 0)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   final String s = ArrayUtils.toString(files);
	   properties.put(id + "." + name, s);
	   }
	}

   void setFont(String id, String name, Font font)
	{
	if (font == null)
	   {
	   properties.remove(id + "." + name);
	   return;
	   }

	String style = "";
	if (font.getStyle() == Font.BOLD + Font.ITALIC)
	   {
	   style = ",bi";
	   }
	else if (font.getStyle() == Font.BOLD)
	   {
	   style = ",b";
	   }
	else if (font.getStyle() == Font.ITALIC)
	   {
	   style = ",i";
	   }

	properties.put(id + "." + name, font.getName() + "," + font.getSize() + style);
	}

   void setHashtable(String id, Hashtable ht)
	{
	serializeObject(id, ht);
	}

   public void setIntArray(String id, String name, int[] values)
	{
	if (values == null || values.length == 0)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   String set = "";
	   for (int i = 0; i < values.length; ++i)
		{
		if (i > 0)
		   {
		   set += ",";
		   }
		set += values[i];
		}
	   properties.put(id + "." + name, set);
	   }
	}

   public void setStringArray(String id, String name, String[] values)
	{
	if (values == null || values.length == 0)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   String set = "";
	   for (int i = 0; i < values.length; ++i)
		{
		if (i > 0)
		   {
		   set += "\0";
		   }
		set += values[i];
		}
	   properties.put(id + "." + name, set);
	   }
	}

   void setInteger(String id, String name, Integer value)
	{
	if (value == null)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   properties.put(id + "." + name, value.toString());
	   }
	}

   public void setProperty(String id, String value)
	{
	properties.setProperty(id, value);
	}

   void setRectangle(String id, String name, Rectangle bounds)
	{
	setBounds(id, name, bounds);
	}

   void setText(String id, String name, String text)
	{
	if (text == null)
	   {
	   properties.remove(id + "." + name);
	   }
	else
	   {
	   properties.put(id + "." + name, text);
	   }
	}

   void setVector(String id, Vector v)
	{
	serializeObject(id, v);
	}

   public void snapshot()
	{
	if (DEBUG)
	   {
	   Logger.log("ApplicationPreferences.snapshot()");
	   }
	storeData();
	}

   private void storeData()
	{
	if (DEBUG)
	   {
	   Logger.log("ApplicationPreferences.storeData()");
	   }
	for (Enumeration enm = id2object.keys(); enm.hasMoreElements();)
	   {
	   String id = (String) enm.nextElement();
	   storeData(id);
	   }
	}

   private void storeData(String id)
	{
	final Object object = id2object.get(id);

	if (DEBUG)
	   {
	   Logger.log("ApplicationPreferences.storeData(): id=" + id);
	   }

	if (object instanceof JFrame)
	   {
	   final Rectangle bounds = ((JFrame) object).getBounds();
	   setBounds(id, "bounds", bounds);
	   final boolean visible = ((JFrame) object).isVisible();
	   setBoolean(id, "visible", new Boolean(visible));
	   if (object instanceof TipOfTheDay)
		{
		final Boolean next = new Boolean(((TipOfTheDay) object).getShowNextTime());
		setBoolean(id, "showNextTime", next);
		final Integer tip = new Integer(((TipOfTheDay) object).getTip());
		setInteger(id, "tipNumber", tip);
		}
	   }
	else if (object instanceof JInternalFrame)
	   {
	   Rectangle bounds = ((JInternalFrame) object).getBounds();
	   setBounds(id, "bounds", bounds);
	   boolean visible = ((JInternalFrame) object).isVisible();
	   setBoolean(id, "visible", new Boolean(visible));
	   }
	else if (object instanceof JDialog)
	   {
	   Rectangle bounds = ((JDialog) object).getBounds();
	   setBounds(id, "bounds", bounds);
	   }
	else if (object instanceof JTextComponent)
	   {
	   String text = ((JTextComponent) object).getText();
	   setText(id, "text", text);
	   }
	else if (object instanceof JCheckBox)
	   {
	   boolean selected = ((JCheckBox) object).isSelected();
	   setBoolean(id, "selected", new Boolean(selected));
	   }
	else if (object instanceof JCheckBoxMenuItem)
	   {
	   boolean selected = ((JCheckBoxMenuItem) object).isSelected();
	   setBoolean(id, "selected", new Boolean(selected));
	   }
	else if (object instanceof JToggleButton)
	   {
	   boolean selected = ((JToggleButton) object).isSelected();
	   setBoolean(id, "selected", new Boolean(selected));
	   }
	else if (object instanceof HFileChooser)
	   {
	   Rectangle bounds = ((HFileChooser) object).getDialogBounds();
	   setBounds(id, "bounds", bounds);
	   File file = ((HFileChooser) object).getSelectedFile();
	   setFile(id, "file", file);
	   }
	else if (object instanceof HFileString)
	   {
	   final HFileString fs = (HFileString) object;
	   if (fs.getPolicy() == HFileString.MULTIPLE_FILES)
		{
		setFiles(id, "file", fs.getFiles());
		storeData(id + ".FileRequester");
		}
	   else
		{
		final File file = fs.getFile();
		setFile(id, "file", file);
		storeData(id + ".FileRequester");
		}
	   }
	else if (object instanceof JFileChooser)
	   {
	   File file = ((JFileChooser) object).getSelectedFile();
	   setFile(id, "file", file);
	   }
	else if (object instanceof JSplitPane)
	   {
	   int location = ((JSplitPane) object).getDividerLocation();
	   setInteger(id, "deviderLocation", new Integer(location));
	   }
	else if (object instanceof JSlider)
	   {
	   int value = ((JSlider) object).getValue();
	   setInteger(id, "value", new Integer(value));
	   }
	else if (object instanceof JTabbedPane)
	   {
	   int selected = ((JTabbedPane) object).getSelectedIndex();
	   setInteger(id, "selectedTab", new Integer(selected));
	   }
	else if (object instanceof JComboBox)
	   {
	   final JComboBox b = (JComboBox) object;
	   if (b.isEditable())
		{
		setText(id, "selectedItem", b.getSelectedItem() != null ? b.getSelectedItem().toString() : null);
		}
	   else
		{
		setInteger(id, "selectedIndex", b.getSelectedIndex() >= 0 ? b.getSelectedIndex() : null);
		}
	   }
	else if (object instanceof JList)
	   {
	   final int[] selected = ((JList) object).getSelectedIndices();
	   setIntArray(id, "selectedIndices", selected);
	   }
	else if (object instanceof HEditableList)
	   {
	   final String[] data = ((HEditableList) object).get();
	   setStringArray(id, "items", data);
	   }
	else if (object instanceof HItemList)
	   {
	   final String[] data = ((HItemList) object).getItemsAsStringArray();
	   setStringArray(id, "items", data);
	   }
	else if (object instanceof HTree)
	   {
	   final String selected = ((HTree) object).getSelectedItem();
	   setText(id, "selected", selected);
	   }
	else if (object instanceof FileRequester)
	   {
	   Rectangle bounds = ((FileRequester) object).getBounds();
	   setBounds(id, "bounds", bounds);
	   File file = ((FileRequester) object).getFile();
	   setFile(id, "file", file);
	   boolean fileHidingEnabled = ((FileRequester) object).isFileHidingEnabled();
	   setBoolean(id, "fileHidingEnabled", fileHidingEnabled);
	   }
	else if (object instanceof AbstractRequester)
	   {
	   Rectangle bounds = ((AbstractRequester) object).getBounds();
	   setBounds(id, "bounds", bounds);
	   }
	else if (object instanceof Hashtable)
	   {
	   setHashtable(id, (Hashtable) object);
	   }
	else if (object instanceof Vector)
	   {
	   setVector(id, (Vector) object);
	   }
	else
	   {
	   System.err.println("ApplicationPreferences.storeData(): not supported: " + object.getClass().getName());
	   }
	}
   }
