/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.system;

import java.io.*;

public final class SystemUtils
   {
   private static StreamRedirectionTarget STDOUT = new StreamRedirectionTarget()
	{
	public void println(String s)
	   {
	   System.out.println(s);
	   }
	};

   public static void printCallHierarchy(final PrintStream out)
	{
	try
	   {
	   throw new RuntimeException("Dummy");
	   }
	catch (Exception x)
	   {
	   for (int j = 2; j < x.getStackTrace().length; ++j)
		{
		out.println("stacktrace=" + x.getStackTrace()[j]);
		}
	   }
	}

   public static String getCallingOperationName(int offset)
	{
	try
	   {
	   throw new RuntimeException("Dummy");
	   }
	catch (Exception x)
	   {
	   return x.getStackTrace()[2 + offset].getMethodName();
	   }
	}

   public static void redirectErrorStream(final Process process, final PrintStream target)
	{
	final Thread thread = new Thread()
	   {
	   public void run()
		{
		setName("JavaGPE-ErrorStreamRedirection");
		setPriority(MAX_PRIORITY);
		try
		   {
		   final BufferedReader in = new BufferedReader(new InputStreamReader(process.getErrorStream()));
		   while (true)
			{
			final String line = in.readLine();
			if (line == null)
			   {
			   break;
			   }
			target.println(line);
			}
		   }
		catch (IOException x)
		   {
		   System.err.println(x);
		   }
		}
	   };
	thread.start();
	}

   public static void redirectErrorStream(final Process process, final StreamRedirectionTarget target)
	{
	final Thread thread = new Thread()
	   {
	   public void run()
		{
		setName("JavaGPE-ErrorStreamRedirection");
		setPriority(MAX_PRIORITY);
		try
		   {
		   final BufferedReader in = new BufferedReader(new InputStreamReader(process.getErrorStream()));
		   while (true)
			{
			final String line = in.readLine();
			if (line == null)
			   {
			   break;
			   }
			target.println(line);
			}
		   }
		catch (IOException x)
		   {
		   System.err.println(x);
		   }
		}
	   };
	thread.start();
	}

   public static void redirectOutputStream(final Process process, final PrintStream target)
	{
	final Thread thread = new Thread()
	   {
	   public void run()
		{
		setName("JavaGPE-OutputStreamRedirection");
		setPriority(MAX_PRIORITY);
		try
		   {
		   final BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
		   while (true)
			{
			final String line = in.readLine();
			if (line == null)
			   {
			   break;
			   }
			target.println(line);
			}
		   }
		catch (IOException x)
		   {
		   System.err.println(x);
		   }
		}
	   };
	thread.start();
	}

   public static void redirectOutputStream(final Process process, final StreamRedirectionTarget target)
	{
	final Thread thread = new Thread()
	   {
	   public void run()
		{
		setName("JavaGPE-OutputStreamRedirection");
		setPriority(MAX_PRIORITY);
		try
		   {
		   final BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
		   while (true)
			{
			final String line = in.readLine();
			if (line == null)
			   {
			   break;
			   }
			target.println(line);
			}
		   }
		catch (IOException x)
		   {
		   System.err.println(x);
		   }
		}
	   };
	thread.start();
	}

   public static int execute(StreamRedirectionTarget target, boolean waitfor, File directory, String... cmd) throws IOException, InterruptedException
	{
	final ProcessBuilder builder = new ProcessBuilder(cmd);
	builder.directory(directory);

	final Process p = builder.start();
	redirectOutputStream(p, target != null ? target : STDOUT);
	redirectErrorStream(p, target != null ? target : STDOUT);

	if (waitfor)
	   {
	   p.waitFor();
	   return p.exitValue();
	   }

	return 0;
	}
   }
