/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.system;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.*;

public final class UIPreferences
   {
   // App Prefs
   private static final ApplicationPreferences prefs = new ApplicationPreferences("JavaUI");
   // constants
   static final String DEFAULT_CONTEXT = "<default>";
   // Look & Feels that need special care
   public static final String METAL_LOOK_AND_FEEL = "javax.swing.plaf.metal.MetalLookAndFeel";
   public static final String METAL_LOOK_AND_FEEL_JAVAGPE = "de.hillenbrand.swing.plaf.metal.MetalLookAndFeel";
   public static final String SKIN_LOOK_AND_FEEL = "com.l2fprod.gui.plaf.skin.SkinLookAndFeel";
   // UI manager keys
   public static final String CONTROL_VERY_LIGHT_SHADOW_KEY = "de.hillenbrand.swing.controlVeryLightShadowColor";
   public static final String CONTROL_LIGHT_SHADOW_KEY = "de.hillenbrand.swing.controlLightShadowColor";
   public static final String CONTROL_MID_SHADOW_KEY = "de.hillenbrand.swing.controlMidShadowColor";
   public static final String CONTROL_VERY_DARK_SHADOW_KEY = "de.hillenbrand.swing.controlVeryDarkShadowColor";
   public static final String CONTROL_DARK_SHADOW_KEY = "de.hillenbrand.swing.controlDarkShadowColor";
   public static final String TITLE_GRADIENT_COLOR1_KEY = "de.hillenbrand.swing.titleGradientColor1";
   public static final String TITLE_GRADIENT_COLOR2_KEY = "de.hillenbrand.swing.titleGradientColor2";
   public static final String TITLE_FOREGROUND_KEY = "de.hillenbrand.swing.titleForegroundColor";
   public static final String TITLE_FONT_KEY = "de.hillenbrand.swing.titleFont";
   public static final String SUB_PANEL_BACKGROUND_KEY = "de.hillenbrand.swing.subPanelBackgroundColor";
   public static final String BUTTON_ICON_ADD = "de.hillenbrand.icon.button.add";
   public static final String BUTTON_ICON_APPLY = "de.hillenbrand.icon.button.apply";
   public static final String BUTTON_ICON_CANCEL = "de.hillenbrand.icon.button.cancel";
   public static final String BUTTON_ICON_CLEAR = "de.hillenbrand.icon.button.clear";
   public static final String BUTTON_ICON_CLOSE = "de.hillenbrand.icon.button.close";
   public static final String BUTTON_ICON_CONFIGURE = "de.hillenbrand.icon.button.configure";
   public static final String BUTTON_ICON_COPY = "de.hillenbrand.icon.button.copy";
   public static final String BUTTON_ICON_CUT = "de.hillenbrand.icon.button.cut";
   public static final String BUTTON_ICON_DELETE = "de.hillenbrand.icon.button.delete";
   public static final String BUTTON_ICON_DISMISS = "de.hillenbrand.icon.button.dismiss";
   public static final String BUTTON_ICON_EDIT = "de.hillenbrand.icon.button.edit";
   public static final String BUTTON_ICON_EXECUTE = "de.hillenbrand.icon.button.execute";
   public static final String BUTTON_ICON_LOAD = "de.hillenbrand.icon.button.load";
   public static final String BUTTON_ICON_MOVEDOWN = "de.hillenbrand.icon.button.movedown";
   public static final String BUTTON_ICON_MOVEUP = "de.hillenbrand.icon.button.moveup";
   public static final String BUTTON_ICON_NEW = "de.hillenbrand.icon.button.new";
   public static final String BUTTON_ICON_OK = "de.hillenbrand.icon.button.ok";
   public static final String BUTTON_ICON_OPEN = "de.hillenbrand.icon.button.open";
   public static final String BUTTON_ICON_PASTE = "de.hillenbrand.icon.button.paste";
   public static final String BUTTON_ICON_PRINT = "de.hillenbrand.icon.button.print";
   public static final String BUTTON_ICON_RELOAD = "de.hillenbrand.icon.button.reload";
   public static final String BUTTON_ICON_RESET = "de.hillenbrand.icon.button.reload";
   public static final String BUTTON_ICON_REMOVE = "de.hillenbrand.icon.button.remove";
   public static final String BUTTON_ICON_SAVE = "de.hillenbrand.icon.button.save";
   public static final String BUTTON_ICON_SAVEAS = "de.hillenbrand.icon.button.saveas";
   public static final String BUTTON_ICON_SEARCH = "de.hillenbrand.icon.button.search";
   public static final String BUTTON_ICON_SORT = "de.hillenbrand.icon.button.sort";
   public static final String BUTTON_ICON_MAIL_GET = "de.hillenbrand.icon.button.mail.get";
   public static final String BUTTON_ICON_MAIL_NEW = "de.hillenbrand.icon.button.mail.new";
   public static final String BUTTON_ICON_MAIL_REPLY = "de.hillenbrand.icon.button.mail.reply";
   public static final String BUTTON_ICON_MAIL_REPLYALL = "de.hillenbrand.icon.button.mail.replyall";
   public static final String BUTTON_ICON_MAIL_SEND = "de.hillenbrand.icon.button.mail.send";
   public static final String BUTTON_ICON_MAIL_FIND = "de.hillenbrand.icon.button.mail.find";
   public static final String BUTTON_ICON_MAIL_FORWARD = "de.hillenbrand.icon.button.mail.forward";
   public static final String BUTTON_ICON_WWW = "de.hillenbrand.icon.button.www";
   public static final String BUTTON_ICON_LOCK = "de.hillenbrand.icon.button.lock";
   public static final String BUTTON_ICON_UNLOCK = "de.hillenbrand.icon.button.unlock";
   public static final String MENU_ICON_ABOUT = "de.hillenbrand.icon.menu.about";
   public static final String MENU_ICON_CLOSE = "de.hillenbrand.icon.menu.close";
   public static final String MENU_ICON_CONFIGURE = "de.hillenbrand.icon.menu.configure";
   public static final String MENU_ICON_COPY = "de.hillenbrand.icon.menu.copy";
   public static final String MENU_ICON_CUT = "de.hillenbrand.icon.menu.cut";
   public static final String MENU_ICON_DELETE = "de.hillenbrand.icon.menu.delete";
   public static final String MENU_ICON_DOWNLOAD = "de.hillenbrand.icon.menu.download";
   public static final String MENU_ICON_EDIT = "de.hillenbrand.icon.menu.edit";
   public static final String MENU_ICON_EXIT = "de.hillenbrand.icon.menu.exit";
   public static final String MENU_ICON_MAIL_FIND = "de.hillenbrand.icon.menu.mail.find";
   public static final String MENU_ICON_MAIL_FORWARD = "de.hillenbrand.icon.menu.mail.forward";
   public static final String MENU_ICON_MAIL_GET = "de.hillenbrand.icon.menu.mail.get";
   public static final String MENU_ICON_MAIL_NEW = "de.hillenbrand.icon.menu.mail.new";
   public static final String MENU_ICON_MAIL_REPLY = "de.hillenbrand.icon.menu.mail.reply";
   public static final String MENU_ICON_MAIL_REPLYALL = "de.hillenbrand.icon.menu.mail.replyall";
   public static final String MENU_ICON_MAIL_SEND = "de.hillenbrand.icon.menu.mail.send";
   public static final String MENU_ICON_NEW = "de.hillenbrand.icon.menu.new";
   public static final String MENU_ICON_OPEN = "de.hillenbrand.icon.menu.open";
   public static final String MENU_ICON_PASTE = "de.hillenbrand.icon.menu.paste";
   public static final String MENU_ICON_PREFS = "de.hillenbrand.icon.menu.prefs";
   public static final String MENU_ICON_PRINT = "de.hillenbrand.icon.menu.print";
   public static final String MENU_ICON_REDO = "de.hillenbrand.icon.menu.redo";
   public static final String MENU_ICON_RELOAD = "de.hillenbrand.icon.menu.reload";
   public static final String MENU_ICON_RESET = "de.hillenbrand.icon.menu.reload";
   public static final String MENU_ICON_SAVE = "de.hillenbrand.icon.menu.save";
   public static final String MENU_ICON_SAVEAS = "de.hillenbrand.icon.menu.saveas";
   public static final String MENU_ICON_SEARCH = "de.hillenbrand.icon.menu.search";
   public static final String MENU_ICON_SELECTALL = "de.hillenbrand.icon.menu.selectall";
   public static final String MENU_ICON_UNDO = "de.hillenbrand.icon.menu.undo";
   public static final String MENU_ICON_SORT = "de.hillenbrand.icon.menu.sort";
   public static final String MENU_ICON_CLEAR = "de.hillenbrand.icon.menu.clear";
   public static final String MENU_ICON_WWW = "de.hillenbrand.icon.menu.www";
   public static final String MENU_ICON_LOCK = "de.hillenbrand.icon.menu.locked";
   public static final String MENU_ICON_UNLOCK = "de.hillenbrand.icon.menu.unlocked";
   public static final String DIALOG_ICON_ABOUT = "de.hillenbrand.icon.about";
   public static final String DIALOG_ICON_ENTERDATA = "de.hillenbrand.icon.enterdata";
   public static final String DIALOG_ICON_ERROR = "de.hillenbrand.icon.error";
   public static final String DIALOG_ICON_EXCEPTION = "de.hillenbrand.icon.exception";
   public static final String DIALOG_ICON_MESSAGE = "de.hillenbrand.icon.message";
   public static final String DIALOG_ICON_NOTYETIMPLEMENTED = "de.hillenbrand.icon.notyetimplemented";
   public static final String DIALOG_ICON_QUESTION = "de.hillenbrand.icon.question";
   public static final String DIALOG_ICON_SELECTDATA = "de.hillenbrand.icon.enterdata";
   public static final String DIALOG_ICON_TIP = "de.hillenbrand.icon.tip";
   public static final String DIALOG_ICON_WORKING = "de.hillenbrand.icon.working";
   public static final String DIALOG_ICON_REQUEST_COLOR = "de.hillenbrand.icon.request.color";
   public static final String DIALOG_ICON_REQUEST_FILE = "de.hillenbrand.icon.request.file";
   public static final String DIALOG_ICON_REQUEST_FONT = "de.hillenbrand.icon.request.font";
   public static final String DIALOG_ICON_REQUEST_LOGIN = "de.hillenbrand.icon.request.login";
   public static final String DIALOG_ICON_REQUEST_PASSWORD = "de.hillenbrand.icon.request.password";
   // variables
   private static boolean doInit = true;
   private static String CONTEXT = "";

   private UIPreferences()
	{
	}

   static
	{
	// Property must be set EARLY due to Apple Bug#3909714
	if (System.getProperty("os.name").equals("Mac OS X"))
	   {
	   System.setProperty("apple.laf.useScreenMenuBar", "true");
	   }

	// some require extra handling
	try
	   {
	   // metouia installs itself
	   Class.forName("net.sourceforge.mlf.metouia.MetouiaLookAndFeel").newInstance();
	   }
	catch (Exception x)
	   {
	   }

	try
	   {
	   UIManager.put(com.jgoodies.looks.Options.USE_SYSTEM_FONTS_APP_KEY, Boolean.TRUE);
	   com.jgoodies.looks.Options.setDefaultIconSize(new Dimension(18, 18));
	   com.jgoodies.looks.Options.setUseSystemFonts(true);
	   com.jgoodies.looks.Options.setPopupDropShadowEnabled(true);
	   com.jgoodies.looks.Options.setTabIconsEnabled(true);
	   com.jgoodies.looks.Options.setUseNarrowButtons(false);
	   }
	catch (NoClassDefFoundError x)
	   {
	   }

	// init
	UIManager.put("Synthetica.window.decoration", Boolean.FALSE);
	UIManager.put("Synthetica.text.antialias", Boolean.TRUE);

	Color controlColor = UIManager.getColor("control");
	if (controlColor == null)
	   {
	   controlColor = Color.red;
	   System.err.println("ERROR: UIManager.getColor(\"control\") returns a null value ?!?");
	   }
	UIManager.put(CONTROL_VERY_LIGHT_SHADOW_KEY, ColorUtils.deriveColorHSB(controlColor, 0, 0, -0.02f));
	UIManager.put(CONTROL_LIGHT_SHADOW_KEY, ColorUtils.deriveColorHSB(controlColor, 0, 0, -0.06f));
	UIManager.put(CONTROL_MID_SHADOW_KEY, ColorUtils.deriveColorHSB(controlColor, 0, 0, -0.16f));
	UIManager.put(CONTROL_VERY_DARK_SHADOW_KEY, ColorUtils.deriveColorHSB(controlColor, 0, 0, -0.5f));
	UIManager.put(CONTROL_DARK_SHADOW_KEY, ColorUtils.deriveColorHSB(controlColor, 0, 0, -0.32f));

	Color titleColor = UIManager.getColor(isNimbusLookAndFeel() ? "nimbusBase" : "activeCaption");
	if (titleColor == null)
	   {
	   titleColor = controlColor;
	   }

	final float hsb[] = Color.RGBtoHSB(titleColor.getRed(), titleColor.getGreen(), titleColor.getBlue(), null);
	UIManager.put(TITLE_GRADIENT_COLOR1_KEY, Color.getHSBColor(hsb[0] - .013f, .15f, .85f));
	UIManager.put(TITLE_GRADIENT_COLOR2_KEY, Color.getHSBColor(hsb[0] - .005f, .24f, .80f));
	UIManager.put(TITLE_FOREGROUND_KEY, Color.getHSBColor(hsb[0], .54f, .40f));

	Font labelFont = UIManager.getFont("Label.font");
	if (labelFont == null)
	   {
	   labelFont = new Font("SansSerif", Font.PLAIN, 12);
	   System.err.println("ERROR: UIManager.getFont(\"Label.font\") returns a null value ?!?");
	   }
	UIManager.put(TITLE_FONT_KEY, labelFont.deriveFont(Font.BOLD, labelFont.getSize() + 4f));

	Color panelColor = UIManager.getColor("Panel.background");
	if (panelColor == null)
	   {
	   panelColor = Color.red;
	   System.err.println("ERROR: UIManager.getColor(\"Panel.background\") returns a null value ?!?");
	   }
	UIManager.put(SUB_PANEL_BACKGROUND_KEY, ColorUtils.deriveColorHSB(panelColor, 0, 0, -.06f));

	// icons
	UIManager.put(BUTTON_ICON_ADD, HIconFactory.getMenuIcon_UIResource("hotlistadd"));
	UIManager.put(BUTTON_ICON_APPLY, HIconFactory.getButtonIcon_UIResource("apply"));
	UIManager.put(BUTTON_ICON_CANCEL, HIconFactory.getButtonIcon_UIResource("cancel"));
	UIManager.put(BUTTON_ICON_CLEAR, HIconFactory.getMenuIcon_UIResource("editshred"));
	UIManager.put(BUTTON_ICON_CLOSE, HIconFactory.getMenuIcon_UIResource("fileclose"));
	UIManager.put(BUTTON_ICON_CONFIGURE, HIconFactory.getMenuIcon_UIResource("configure"));
	UIManager.put(BUTTON_ICON_COPY, HIconFactory.getMenuIcon_UIResource("editcopy"));
	UIManager.put(BUTTON_ICON_CUT, HIconFactory.getMenuIcon_UIResource("editcut"));
	UIManager.put(BUTTON_ICON_DELETE, HIconFactory.getMenuIcon_UIResource("edittrash"));
	UIManager.put(BUTTON_ICON_DISMISS, HIconFactory.getButtonIcon_UIResource("dismiss"));
	UIManager.put(BUTTON_ICON_EDIT, HIconFactory.getMenuIcon_UIResource("edit"));
	UIManager.put(BUTTON_ICON_EXECUTE, HIconFactory.getMenuIcon_UIResource("gear"));
	UIManager.put(BUTTON_ICON_LOAD, HIconFactory.getMenuIcon_UIResource("fileopen"));
	UIManager.put(BUTTON_ICON_MOVEDOWN, HIconFactory.getMenuIcon_UIResource("down"));
	UIManager.put(BUTTON_ICON_MOVEUP, HIconFactory.getMenuIcon_UIResource("up"));
	UIManager.put(BUTTON_ICON_NEW, HIconFactory.getMenuIcon_UIResource("filenew"));
	UIManager.put(BUTTON_ICON_OK, HIconFactory.getButtonIcon_UIResource("ok"));
	UIManager.put(BUTTON_ICON_OPEN, HIconFactory.getMenuIcon_UIResource("fileopen"));
	UIManager.put(BUTTON_ICON_PASTE, HIconFactory.getMenuIcon_UIResource("editpaste"));
	UIManager.put(BUTTON_ICON_PRINT, HIconFactory.getMenuIcon_UIResource("fileprint"));
	UIManager.put(BUTTON_ICON_RELOAD, HIconFactory.getMenuIcon_UIResource("reload"));
	UIManager.put(BUTTON_ICON_REMOVE, HIconFactory.getMenuIcon_UIResource("hotlistdel"));
	UIManager.put(BUTTON_ICON_SAVE, HIconFactory.getMenuIcon_UIResource("filesave"));
	UIManager.put(BUTTON_ICON_SAVEAS, HIconFactory.getMenuIcon_UIResource("filesaveas"));
	UIManager.put(BUTTON_ICON_SEARCH, HIconFactory.getMenuIcon_UIResource("find"));
	UIManager.put(BUTTON_ICON_SORT, HIconFactory.getMenuIcon_UIResource("sortinc"));
	UIManager.put(BUTTON_ICON_MAIL_GET, HIconFactory.getMenuIcon_UIResource("mail_get"));
	UIManager.put(BUTTON_ICON_MAIL_NEW, HIconFactory.getMenuIcon_UIResource("mail_new"));
	UIManager.put(BUTTON_ICON_MAIL_REPLY, HIconFactory.getMenuIcon_UIResource("mail_reply"));
	UIManager.put(BUTTON_ICON_MAIL_REPLYALL, HIconFactory.getMenuIcon_UIResource("mail_replyall"));
	UIManager.put(BUTTON_ICON_MAIL_SEND, HIconFactory.getMenuIcon_UIResource("mail_send"));
	UIManager.put(BUTTON_ICON_MAIL_FIND, HIconFactory.getMenuIcon_UIResource("mail_find"));
	UIManager.put(BUTTON_ICON_MAIL_FORWARD, HIconFactory.getMenuIcon_UIResource("mail_forward"));
	UIManager.put(BUTTON_ICON_WWW, HIconFactory.getMenuIcon_UIResource("htmlvalidator"));
	UIManager.put(BUTTON_ICON_LOCK, HIconFactory.getMenuIcon_UIResource("lock"));
	UIManager.put(BUTTON_ICON_UNLOCK, HIconFactory.getMenuIcon_UIResource("unlock"));

	UIManager.put(MENU_ICON_ABOUT, HIconFactory.getMenuIcon_UIResource("info"));
	UIManager.put(MENU_ICON_CLOSE, HIconFactory.getMenuIcon_UIResource("fileclose"));
	UIManager.put(MENU_ICON_CONFIGURE, HIconFactory.getMenuIcon_UIResource("configure"));
	UIManager.put(MENU_ICON_COPY, HIconFactory.getMenuIcon_UIResource("editcopy"));
	UIManager.put(MENU_ICON_CUT, HIconFactory.getMenuIcon_UIResource("editcut"));
	UIManager.put(MENU_ICON_EXIT, HIconFactory.getMenuIcon_UIResource("exit"));
	UIManager.put(MENU_ICON_NEW, HIconFactory.getMenuIcon_UIResource("filenew"));
	UIManager.put(MENU_ICON_OPEN, HIconFactory.getMenuIcon_UIResource("fileopen"));
	UIManager.put(MENU_ICON_PASTE, HIconFactory.getMenuIcon_UIResource("editpaste"));
	UIManager.put(MENU_ICON_PREFS, HIconFactory.getMenuIcon_UIResource("configure"));
	UIManager.put(MENU_ICON_PRINT, HIconFactory.getMenuIcon_UIResource("fileprint"));
	UIManager.put(MENU_ICON_RELOAD, HIconFactory.getMenuIcon_UIResource("reload"));
	UIManager.put(MENU_ICON_SAVE, HIconFactory.getMenuIcon_UIResource("filesave"));
	UIManager.put(MENU_ICON_SAVEAS, HIconFactory.getMenuIcon_UIResource("filesaveas"));
	UIManager.put(MENU_ICON_SEARCH, HIconFactory.getMenuIcon_UIResource("find"));
	UIManager.put(MENU_ICON_MAIL_GET, HIconFactory.getMenuIcon_UIResource("mail_get"));
	UIManager.put(MENU_ICON_MAIL_NEW, HIconFactory.getMenuIcon_UIResource("mail_new"));
	UIManager.put(MENU_ICON_MAIL_REPLY, HIconFactory.getMenuIcon_UIResource("mail_reply"));
	UIManager.put(MENU_ICON_MAIL_REPLYALL, HIconFactory.getMenuIcon_UIResource("mail_replyall"));
	UIManager.put(MENU_ICON_MAIL_SEND, HIconFactory.getMenuIcon_UIResource("mail_send"));
	UIManager.put(MENU_ICON_MAIL_FIND, HIconFactory.getMenuIcon_UIResource("mail_find"));
	UIManager.put(MENU_ICON_MAIL_FORWARD, HIconFactory.getMenuIcon_UIResource("mail_forward"));
	UIManager.put(MENU_ICON_EDIT, HIconFactory.getMenuIcon_UIResource("edit"));
	UIManager.put(MENU_ICON_DOWNLOAD, HIconFactory.getMenuIcon_UIResource("bottom"));
	UIManager.put(MENU_ICON_DELETE, HIconFactory.getMenuIcon_UIResource("edittrash"));
	UIManager.put(MENU_ICON_UNDO, HIconFactory.getMenuIcon_UIResource("undo"));
	UIManager.put(MENU_ICON_REDO, HIconFactory.getMenuIcon_UIResource("redo"));
	UIManager.put(MENU_ICON_SELECTALL, HIconFactory.getMenuIcon_UIResource("frame_edit"));
	UIManager.put(MENU_ICON_SORT, HIconFactory.getMenuIcon_UIResource("sortinc"));
	UIManager.put(MENU_ICON_CLEAR, HIconFactory.getMenuIcon_UIResource("editshred"));
	UIManager.put(MENU_ICON_WWW, HIconFactory.getMenuIcon_UIResource("htmlvalidator"));
	UIManager.put(MENU_ICON_LOCK, HIconFactory.getMenuIcon_UIResource("lock"));
	UIManager.put(MENU_ICON_UNLOCK, HIconFactory.getMenuIcon_UIResource("unlock"));

	UIManager.put(DIALOG_ICON_ABOUT, HIconFactory.getIcon_UIResource("about"));
	UIManager.put(DIALOG_ICON_ENTERDATA, HIconFactory.getIcon_UIResource("enterdata"));
	UIManager.put(DIALOG_ICON_ERROR, HIconFactory.getIcon_UIResource("error"));
	UIManager.put(DIALOG_ICON_EXCEPTION, HIconFactory.getIcon_UIResource("exception"));
	UIManager.put(DIALOG_ICON_MESSAGE, HIconFactory.getIcon_UIResource("message"));
	UIManager.put(DIALOG_ICON_NOTYETIMPLEMENTED, HIconFactory.getIcon_UIResource("notyetimplemented"));
	UIManager.put(DIALOG_ICON_QUESTION, HIconFactory.getIcon_UIResource("question"));
	UIManager.put(DIALOG_ICON_REQUEST_COLOR, HIconFactory.getIcon_UIResource("selectcolor"));
	UIManager.put(DIALOG_ICON_REQUEST_FILE, HIconFactory.getIcon_UIResource("folder_yellow_open"));
	UIManager.put(DIALOG_ICON_REQUEST_FONT, HIconFactory.getIcon_UIResource("selectfont"));
	UIManager.put(DIALOG_ICON_REQUEST_LOGIN, HIconFactory.getIcon_UIResource("login"));
	UIManager.put(DIALOG_ICON_REQUEST_PASSWORD, HIconFactory.getIcon_UIResource("password"));
	UIManager.put(DIALOG_ICON_SELECTDATA, HIconFactory.getIcon_UIResource("selectdata"));

	// these will be installed the usual way
	final String[] toInstall =
	   {
	   "com.sun.java.swing.plaf.gtk.GTKLookAndFeel",
	   "de.hillenbrand.swing.plaf.threeD.ThreeDLookAndFeel",
	   UIPreferences.METAL_LOOK_AND_FEEL,
	   UIPreferences.METAL_LOOK_AND_FEEL_JAVAGPE,
	   "de.hillenbrand.swing.plaf.windows.WindowsLookAndFeel",
	   "de.hillenbrand.swing.plaf.kde3.KDE3LookAndFeel",
	   "swing.addon.plaf.themelf.ThemeLookAndFeel",
	   UIPreferences.SKIN_LOOK_AND_FEEL,
	   "com.jgoodies.looks.plastic.Plastic3DLookAndFeel",
	   "com.jgoodies.looks.plastic.PlasticLookAndFeel",
	   "com.jgoodies.looks.plastic.PlasticXPLookAndFeel",
	   "com.jgoodies.looks.windows.WindowsLookAndFeel",
	   "com.digitprop.tonic.TonicLookAndFeel",
	   "de.javasoft.plaf.synthetica.SyntheticaStandardLookAndFeel",
	   };

	// try to load all look and feels
	for (int i = 0; i < toInstall.length; ++i)
	   {
	   try
		{
		// try to load the class
		final Class c = Class.forName(toInstall[i]);
		// check whether this l&f is arleay installed
		final UIManager.LookAndFeelInfo[] lfs = UIManager.getInstalledLookAndFeels();
		for (int j = 0; j < lfs.length; ++j)
		   {
		   if (lfs[j].getClassName().equals(toInstall[i]))
			{
			continue;
			}
		   }
		// create an instance of the l&f and install it into the UIManager
		final LookAndFeel lf = (LookAndFeel) c.newInstance();
		UIManager.installLookAndFeel(lf.getName(), toInstall[i]);
		}
	   catch (ClassNotFoundException x)
		{
		}
	   catch (Exception x)
		{
		System.err.println(x);
		}
	   }
	}

   private static final void apply_LookAndFeel()
	{
	final String lookAndFeel = UIPreferences.getLookAndFeel();

	try
	   {
	   if (lookAndFeel.equals(UIPreferences.SKIN_LOOK_AND_FEEL))
		{
		final String pack = UIPreferences.getSkinLFThemePack();
		if (pack.length() > 0)
		   {
		   if (pack.indexOf("://") == -1)
			{
			final File f = new File(pack);
			if (f.exists())
			   {
			   com.l2fprod.gui.plaf.skin.SkinLookAndFeel.setSkin(com.l2fprod.gui.plaf.skin.SkinLookAndFeel.loadThemePack(f.toURI().toURL()));
			   }
			}
		   else
			{
			com.l2fprod.gui.plaf.skin.SkinLookAndFeel.setSkin(com.l2fprod.gui.plaf.skin.SkinLookAndFeel.loadThemePack(new java.net.URL(pack)));
			}
		   }
		}
	   else if (lookAndFeel.equals(METAL_LOOK_AND_FEEL) || lookAndFeel.equals(METAL_LOOK_AND_FEEL_JAVAGPE))
		{
		if (getSetMetalTheme())
		   {
		   final String theme = UIPreferences.getMetalTheme();
		   try
			{
			final Class c = Class.forName(theme);
			MetalLookAndFeel.setCurrentTheme((MetalTheme) c.newInstance());
			}
		   catch (Exception x)
			{
			JReqTools.showException((JFrame) null, x);
			}
		   }
		}
	   UIManager.setLookAndFeel(lookAndFeel);
	   }
	catch (ClassNotFoundException x1)
	   {
	   try
		{
		UIManager.setLookAndFeel(METAL_LOOK_AND_FEEL_JAVAGPE);
		}
	   catch (Exception e)
		{
		}
	   JReqTools.showError((JFrame) null, Helper.GRS("Errors.SetLookAndFeel"), Helper.GRS("Errors.CantFindLookAndFeel", lookAndFeel));
	   }
	catch (Exception x2)
	   {
	   String msg = x2.getLocalizedMessage();
	   if (msg != null && msg.length() > 0)
		{
		JReqTools.showError((JFrame) null, Helper.GRS("Errors.SetLookAndFeel"), StringUtils.insertNewline(msg, 60));
		}
	   else
		{
		JReqTools.showException(new JFrame(), x2);
		}
	   }
	catch (Error e1)
	   {
	   String msg = e1.getLocalizedMessage();
	   if (msg != null && msg.length() > 0)
		{
		JReqTools.showError((JFrame) null, Helper.GRS("Errors.SetLookAndFeel"), StringUtils.insertNewline(msg, 60));
		}
	   else
		{
		JReqTools.showException(new JFrame(), e1);
		}
	   }
	}

   public static final Font getAcceleratorFont()
	{
	return getAcceleratorFont(CONTEXT);
	}

   static final Font getAcceleratorFont(String context)
	{
	Font result = prefs.getFont("AcceleratorFont." + context, "font");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getFont("AcceleratorFont", "font");
	if (result != null)
	   {
	   return result;
	   }

	return new Font("Verdana", Font.PLAIN, 10);
	}

   public static int getCaretBlinkRate()
	{
	return getCaretBlinkRate(CONTEXT);
	}

   static int getCaretBlinkRate(String context)
	{
	Integer result = prefs.getInteger("CaretBlinkRate." + context, "value");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getInteger("CaretBlinkRate", "value");
	if (result != null)
	   {
	   return result;
	   }

	return 500;
	}

   public static Color getCaretColor()
	{
	return getCaretColor(CONTEXT);
	}

   static Color getCaretColor(String context)
	{
	Color result = prefs.getColor("CaretColor." + context, "color");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getColor("CaretColor", "color");
	if (result != null)
	   {
	   return result;
	   }

	return Color.red;
	}

   public static final String getErrorSound()
	{
	return getErrorSound(CONTEXT);
	}

   static final String getErrorSound(String context)
	{
	String result = prefs.getText("ErrorSound." + context, "filename");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getText("ErrorSound", "filename");
	if (result != null)
	   {
	   return result;
	   }

	return "";
	}

   public static final Font getFixedWidthFont()
	{
	return getFixedWidthFont(CONTEXT);
	}

   static final Font getFixedWidthFont(String context)
	{
	Font result = prefs.getFont("FixedWidthFont." + context, "font");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getFont("FixedWidthFont", "font");
	if (result != null)
	   {
	   return result;
	   }

	return new Font("Courier", Font.PLAIN, 12);
	}

   public static final Font getFont()
	{
	return getFont(CONTEXT);
	}

   static final Font getFont(String context)
	{
	Font result = prefs.getFont("Font." + context, "font");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getFont("Font", "font");
	if (result != null)
	   {
	   return result;
	   }

	return new Font("Verdana", Font.PLAIN, 12);
	}

   public static final Border getFrameBorder()
	{
	return BorderFactory.createEmptyBorder(8, 8, 8, 8);
	}

   public static final HGridBagConstraints getGridBagConstraints()
	{
	final HGridBagConstraints result = new HGridBagConstraints();

	result.insets = HGridBagConstraints.DEFAULT_INSETS;
	result.fill = HGridBagConstraints.BOTH;

	return result;
	}

   public static final Border getInnerTabbedPaneBorder()
	{
	return BorderFactory.createEmptyBorder(5, 5, 5, 5);
	}

   public static final String getLocale()
	{
	return getLocale(CONTEXT);
	}

   static final String getLocale(String context)
	{
	String result = prefs.getText("Locale." + context, "name");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getText("Locale", "name");
	if (result != null)
	   {
	   return result;
	   }

	return Locale.getDefault().toString();
	}

   public static final String getLookAndFeel()
	{
	return getLookAndFeel(CONTEXT);
	}

   static final String getLookAndFeel(String context)
	{
	String result = prefs.getText("LookAndFeel." + context, "className");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getText("LookAndFeel", "className");
	if (result != null)
	   {
	   return result;
	   }

	return UIManager.getSystemLookAndFeelClassName();
	}

   public static final Font getMessageFont()
	{
	return getMessageFont(CONTEXT);
	}

   static final Font getMessageFont(String context)
	{
	Font result = prefs.getFont("MessageFont." + context, "font");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getFont("MessageFont", "font");
	if (result != null)
	   {
	   return result;
	   }

	return new Font("Verdana", Font.PLAIN, 16);
	}

   public static final String getMetalTheme()
	{
	return getMetalTheme(CONTEXT);
	}

   static final String getMetalTheme(String context)
	{
	String result = prefs.getText("MetalTheme." + context, "class");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getText("MetalTheme", "class");
	if (result != null)
	   {
	   return result;
	   }

	return "javax.swing.plaf.metal.OceanTheme";
	}

   public static final GraphicsDevice getPrimaryGraphicsDevice()
	{
	final Integer i = prefs.getInteger("PrimaryGraphicsDevice." + CONTEXT, "number");
	final int index = i == null ? -1 : i;

	final GraphicsDevice[] devices = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
	if (0 <= index && index < devices.length)
	   {
	   return devices[index];
	   }
	else
	   {
	   return GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
	   }
	}

   static final int getPrimaryGraphicsDevice(String context)
	{
	final Integer i = prefs.getInteger("PrimaryGraphicsDevice." + context, "number");
	return i == null ? -1 : i;
	}

   public static final boolean getSetAcceleratorFont()
	{
	return getSetAcceleratorFont(CONTEXT);
	}

   static final boolean getSetAcceleratorFont(String context)
	{
	Boolean result = prefs.getBoolean("AcceleratorFont." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("AcceleratorFont", "set");
	if (result != null)
	   {
	   return result;
	   }

	return true;
	}

   public static final boolean getSetCaretBlinkRate()
	{
	return getSetCaretBlinkRate(CONTEXT);
	}

   static final boolean getSetCaretBlinkRate(String context)
	{
	Boolean result = prefs.getBoolean("CaretBlinkRate." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("CaretBlinkRate", "set");
	if (result != null)
	   {
	   return result;
	   }

	return false;
	}

   public static final boolean getSetCaretColor()
	{
	return getSetCaretColor(CONTEXT);
	}

   static final boolean getSetCaretColor(String context)
	{
	Boolean result = prefs.getBoolean("CaretColor." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("CaretColor", "set");
	if (result != null)
	   {
	   return result;
	   }

	return false;
	}

   public static final boolean getSetFont()
	{
	return getSetFont(CONTEXT);
	}

   static final boolean getSetFont(String context)
	{
	Boolean result = prefs.getBoolean("Font." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("Font", "set");
	if (result != null)
	   {
	   return result;
	   }

	return true;
	}

   public static final boolean getSetLocale()
	{
	return getSetLocale(CONTEXT);
	}

   static final boolean getSetLocale(String context)
	{
	Boolean result = prefs.getBoolean("Locale." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("Locale", "set");
	if (result != null)
	   {
	   return result;
	   }

	return false;
	}

   public static final boolean getSetMetalTheme()
	{
	return getSetMetalTheme(CONTEXT);
	}

   static final boolean getSetMetalTheme(String context)
	{
	Boolean result = prefs.getBoolean("MetalTheme." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("MetalTheme", "set");
	if (result != null)
	   {
	   return result;
	   }

	return false;
	}

   public static final boolean getSetTabFont()
	{
	return getSetTabFont(CONTEXT);
	}

   static final boolean getSetTabFont(String context)
	{
	Boolean result = prefs.getBoolean("TabFont." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("TabFont", "set");
	if (result != null)
	   {
	   return result;
	   }

	return true;
	}

   public static final boolean getSetTextSelectionColor()
	{
	return getSetTextSelectionColor(CONTEXT);
	}

   static final boolean getSetTextSelectionColor(String context)
	{
	Boolean result = prefs.getBoolean("TextSelectionColor." + context, "set");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getBoolean("TextSelectionColor", "set");
	if (result != null)
	   {
	   return result;
	   }

	return false;
	}

   public static final String getSkinLFThemePack()
	{
	return getSkinLFThemePack(CONTEXT);
	}

   static final String getSkinLFThemePack(String context)
	{
	String result = prefs.getText("SkinLF." + context, "ThemePack");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getText("SkinLF", "ThemePack");
	if (result != null)
	   {
	   return result;
	   }

	return "http://www.l2fprod.com/javootoo.com/plaf/skinlf/themepacks/aquathemepack.zip";
	}

   public static final Font getTabFont()
	{
	return getTabFont(CONTEXT);
	}

   static final Font getTabFont(String context)
	{
	Font result = prefs.getFont("TabFont." + context, "font");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getFont("TabFont", "font");
	if (result != null)
	   {
	   return result;
	   }

	return new Font("Verdana", Font.PLAIN, 13);
	}

   public static Color getTextSelectionColor()
	{
	return getTextSelectionColor(CONTEXT);
	}

   static Color getTextSelectionColor(String context)
	{
	Color result = prefs.getColor("TextSelectionColor." + context, "color");
	if (result != null)
	   {
	   return result;
	   }

	result = prefs.getColor("TextSelectionColor", "color");
	if (result != null)
	   {
	   return result;
	   }

	return new Color(255, 220, 0);
	}

   public static final synchronized void init()
	{
	init(DEFAULT_CONTEXT);
	}

   public static final synchronized void init(String context)
	{
	if (context == null)
	   {
	   throw new IllegalArgumentException("Context can't be null.");
	   }
	registerContext(context);

	if (doInit)
	   {
	   CONTEXT = isContextActive(context) ? context : DEFAULT_CONTEXT;
	   // Locale
	   if (UIPreferences.getSetLocale())
		{
		final Locale l = new Locale(UIPreferences.getLocale());
		Locale.setDefault(l);
		}
	   // Look & Feel
	   apply_LookAndFeel();

	   final UIDefaults defaults = UIManager.getDefaults();

	   // Other values
	   if (UIPreferences.getSetFont())
		{
		final Font font = UIPreferences.getFont();
		defaults.put("Button.font", font);
		defaults.put("CheckBox.font", font);
		defaults.put("CheckBoxMenuItem.font", font);
		defaults.put("ColorChooser.font", font);
		defaults.put("ComboBox.font", font);
		defaults.put("EditorPane.font", font);
		defaults.put("InternalFrame.titleFont", font);
		defaults.put("Label.font", font);
		defaults.put("List.font", font);
		defaults.put("Menu.font", font);
		defaults.put("MenuBar.font", font);
		defaults.put("MenuItem.font", font);
		defaults.put("OptionPane.font", font);
		defaults.put("Panel.font", font);
		defaults.put("PasswordField.font", font);
		defaults.put("PopupMenu.font", font);
		defaults.put("ProgressBar.font", font);
		defaults.put("RadioButton.font", font);
		defaults.put("RadioButtonMenuItem.font", font);
		defaults.put("ScrollPane.font", font);
		defaults.put("Table.font", font);
		defaults.put("TableHeader.font", font);
		defaults.put("TextArea.font", font);
		defaults.put("TextField.font", font);
		defaults.put("TextPane.font", font);
		defaults.put("TitledBorder.font", font);
		defaults.put("ToggleButton.font", font);
		defaults.put("ToolBar.font", font);
		defaults.put("ToolTip.font", font);
		defaults.put("Tree.font", font);
		defaults.put("Viewport.font", font);
		}

	   if (UIPreferences.getSetAcceleratorFont())
		{
		final Font font = UIPreferences.getAcceleratorFont();
		defaults.put("CheckBoxMenuItem.acceleratorFont", font);
		defaults.put("Menu.acceleratorFont", font);
		defaults.put("MenuItem.acceleratorFont", font);
		defaults.put("RadioButtonMenuItem.acceleratorFont", font);
		}

	   if (UIPreferences.getSetTabFont())
		{
		final Font font = UIPreferences.getTabFont();
		defaults.put("TabbedPane.font", font);
		defaults.put("TabbedPane.selectedFont", font);
		// check insets
		final Insets i = (Insets) defaults.get("TabbedPane.tabInsets");
		if (i != null)
		   {
		   if (i.top < 2)
			{
			i.top = 2;
			}
		   if (i.right < 2)
			{
			i.right = 2;
			}
		   if (i.bottom < 2)
			{
			i.bottom = 2;
			}
		   if (i.right < 2)
			{
			i.left = 2;
			}
		   }
		}

	   if (UIPreferences.getSetCaretBlinkRate())
		{
		final Integer caretBlinkRate = new Integer(UIPreferences.getCaretBlinkRate());
		defaults.put("TextField.caretBlinkRate", caretBlinkRate);
		defaults.put("FormattedTextField.caretBlinkRate", caretBlinkRate);
		defaults.put("PasswordField.caretBlinkRate", caretBlinkRate);
		defaults.put("TextArea.caretBlinkRate", caretBlinkRate);
		defaults.put("TextPane.caretBlinkRate", caretBlinkRate);
		defaults.put("EditorPane.caretBlinkRate", caretBlinkRate);
		}

	   if (UIPreferences.getSetCaretColor())
		{
		final Color caretColor = UIPreferences.getCaretColor();
		defaults.put("EditorPane.caretForeground", caretColor);
		defaults.put("FormattedTextField.caretForeground", caretColor);
		defaults.put("PasswordField.caretForeground", caretColor);
		defaults.put("TextArea.caretForeground", caretColor);
		defaults.put("TextField.caretForeground", caretColor);
		defaults.put("TextPane.caretForeground", caretColor);
		}

	   if (UIPreferences.getSetTextSelectionColor())
		{
		final Color selectionColor = UIPreferences.getTextSelectionColor();
		defaults.put("EditorPane.selectionBackground", selectionColor);
		defaults.put("FormattedTextField.selectionBackground", selectionColor);
		defaults.put("PasswordField.selectionBackground", selectionColor);
		defaults.put("TextArea.selectionBackground", selectionColor);
		defaults.put("TextField.selectionBackground", selectionColor);
		defaults.put("TextPane.selectionBackground", selectionColor);
		}

	   // done
	   doInit = false;
	   }
	}

   static final boolean isContextActive(String context)
	{
	if (context.equals(DEFAULT_CONTEXT))
	   {
	   return true;
	   }

	final Boolean result = prefs.getBoolean("Context." + context, "isActive");
	if (result != null)
	   {
	   return result;
	   }

	return false;
	}

   public static final boolean isNimbusLookAndFeel()
	{
	return UIManager.getLookAndFeel().getName().equals("Nimbus");
	}

   private static final synchronized void registerContext(String context)
	{
	if (context == null || context.equals("") || context.equals(DEFAULT_CONTEXT))
	   {
	   return;
	   }

	// create save directory
	final File directory = new File(System.getProperty("user.home"), ".JavaUI");
	directory.mkdirs();

	// data file
	final File contexts = new File(directory, "contexts");

	// read all contexts
	final TreeSet<String> all = new TreeSet<String>();
	try
	   {
	   final BufferedReader reader = new BufferedReader(new FileReader(contexts));
	   while (true)
		{
		final String line = reader.readLine();
		if (line == null)
		   {
		   break;
		   }
		if (line.equals(context))
		   {
		   reader.close();
		   return;
		   }
		all.add(line);
		}
	   reader.close();
	   }
	catch (FileNotFoundException x)
	   {
	   }
	catch (IOException x)
	   {
	   System.err.println("Could not read registered contexts: " + x);
	   }

	// add context
	all.add(context);

	// write contexts
	PrintWriter writer = null;
	try
	   {
	   writer = new PrintWriter(new FileWriter(contexts));
	   for (String s : all)
		{
		writer.println(s);
		}
	   }
	catch (Exception x)
	   {
	   System.err.println("Could not register context: " + x);
	   }
	finally
	   {
	   if (writer != null)
		{
		writer.close();
		}
	   }
	}

   static void save()
	{
	prefs.save();
	}

   static final void setAcceleratorFont(String context, Font font)
	{
	prefs.setFont("AcceleratorFont." + context, "font", font);
	}

   static final void setCaretBlinkRate(String context, int rate)
	{
	prefs.setInteger("CaretBlinkRate." + context, "value", rate);
	}

   static final void setCaretColor(String context, Color color)
	{
	prefs.setColor("CaretColor." + context, "color", color);
	}

   static final void setContextActive(String context, boolean status)
	{
	if (context.equals(DEFAULT_CONTEXT))
	   {
	   return;
	   }
	prefs.setBoolean("Context." + context, "isActive", status);
	}

   static final void setErrorSound(String context, String filename)
	{
	prefs.setText("ErrorSound." + context, "filename", filename);
	}

   static final void setFixedWidthFont(String context, Font font)
	{
	prefs.setFont("FixedWidthFont." + context, "font", font);
	}

   static final void setFont(String context, Font font)
	{
	prefs.setFont("Font." + context, "font", font);
	}

   static final void setLocale(String context, String locale)
	{
	prefs.setText("Locale." + context, "name", locale);
	}

   /**
    * Sets the look and feel.
    */
   static final void setLookAndFeel(String context, String lookAndFeel)
	{
	prefs.setText("LookAndFeel." + context, "className", lookAndFeel);
	}

   static final void setMessageFont(String context, Font font)
	{
	prefs.setFont("MessageFont." + context, "font", font);
	}

   static final void setMetalTheme(String context, String value)
	{
	prefs.setText("MetalTheme." + context, "class", value);
	}

   static final void setPrimaryGraphicsDevice(String context, int number)
	{
	prefs.setInteger("PrimaryGraphicsDevice." + context, "number", number);
	}

   static final void setSetAcceleratorFont(String context, boolean value)
	{
	prefs.setBoolean("AcceleratorFont." + context, "set", value);
	}

   static final void setSetCaretBlinkRate(String context, boolean value)
	{
	prefs.setBoolean("CaretBlinkRate." + context, "set", value);
	}

   static final void setSetCaretColor(String context, boolean value)
	{
	prefs.setBoolean("CaretColor." + context, "set", value);
	}

   static final void setSetFont(String context, boolean value)
	{
	prefs.setBoolean("Font." + context, "set", value);
	}

   static final void setSetLocale(String context, boolean value)
	{
	prefs.setBoolean("Locale." + context, "set", value);
	}

   static final void setSetMetalTheme(String context, boolean value)
	{
	prefs.setBoolean("MetalTheme." + context, "set", value);
	}

   static final void setSetTabFont(String context, boolean value)
	{
	prefs.setBoolean("TabFont." + context, "set", value);
	}

   static final void setSetTextSelectionColor(String context, boolean value)
	{
	prefs.setBoolean("TextSelectionColor." + context, "set", value);
	}

   static final void setSkinLFThemePack(String context, String filename)
	{
	prefs.setText("SkinLF." + context, "ThemePack", filename);
	}

   static final void setTabFont(String context, Font font)
	{
	prefs.setFont("TabFont." + context, "font", font);
	}

   static final void setTextSelectionColor(String context, Color color)
	{
	prefs.setColor("TextSelectionColor." + context, "color", color);
	}
   }
