/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.system;

import de.hillenbrand.jreqtools.*;
import de.hillenbrand.swing.*;
import de.hillenbrand.swing.renderer.*;
import de.hillenbrand.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

public class UIPreferencesFrame extends OkCancelFrame
   {
   // app prefs
   private static final ApplicationPreferences prefs = new ApplicationPreferences("UIPreferencesFrame");
   // components
   private final JSplitPane splitPane = new JSplitPane();
   private final JTabbedPane tabbedPane = new JTabbedPane();
   private final JList locations = new JList(FileRequester.getLocations());
   private final JList contexts = new JList(getContexts());
   private final JButton delete = new JButton(HIconFactory.getMenuIcon("edittrash"));
   private final JButton copy = new JButton(HIconFactory.getMenuIcon("editcopy"));
   private final JButton reset = new JButton(HIconFactory.getMenuIcon("reload"));
   // Variables
   private final Hashtable<String, PrefsPanel> data = new Hashtable<String, PrefsPanel>();

   public class FileRequesterPrefs extends PrefsPanel
	{
	// components
	private final JButton add = new JButton(HIconFactory.getIcon("editablelist_plus"));
	private final JButton remove = new JButton(HIconFactory.getIcon("editablelist_minus"));
	private final JButton up = new JButton(HIconFactory.getIcon("editablelist_up"));
	private final JButton down = new JButton(HIconFactory.getIcon("editablelist_down"));
	private final HFileString directory = new HFileString("", HFileString.DIRECTORIES_ONLY);
	private final HTextField description = new HTextField();
	private final JButton icon = new JButton();

	public FileRequesterPrefs(String context)
	   {
	   super(context);

	   if (context != UIPreferences.DEFAULT_CONTEXT)
		{
		this.add(new JLabel(Helper.GRS("UIPreferencesFrame.FileRequester.NotAvailable"), JLabel.LEFT));
		return;
		}

	   // init
	   locations.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	   add.setMargin(new Insets(1, 1, 1, 1));
	   remove.setMargin(new Insets(1, 1, 1, 1));
	   up.setMargin(new Insets(1, 1, 1, 1));
	   down.setMargin(new Insets(1, 1, 1, 1));
	   reset();

	   // renderer
	   locations.setCellRenderer(new DefaultListCellRenderer()
		{
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
		   {
		   super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		   setIcon(((FileRequester.Location) value).getIcon());
		   setText(((FileRequester.Location) value).getDescription());
		   return this;
		   }
		});

	   // listeners
	   locations.addListSelectionListener(new ListSelectionListener()
		{
		public void valueChanged(ListSelectionEvent e)
		   {
		   if (e.getValueIsAdjusting())
			{
			return;
			}
		   FileRequester.Location loc = (FileRequester.Location) locations.getSelectedValue();
		   if (loc == null)
			{
			return;
			}
		   directory.setFile(loc.getDirectory());
		   icon.setIcon(loc.getIcon());
		   description.setText(loc.getDescription());
		   }
		});
	   directory.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   FileRequester.Location loc = (FileRequester.Location) locations.getSelectedValue();
		   if (loc == null)
			{
			return;
			}
		   loc.setDirectory(directory.getFile());
		   }
		});
	   description.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   FileRequester.Location loc = (FileRequester.Location) locations.getSelectedValue();
		   if (loc == null)
			{
			return;
			}
		   loc.setDescription(description.getText());
		   locations.repaint();
		   }
		});
	   icon.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final FileRequester.Location loc = (FileRequester.Location) locations.getSelectedValue();
		   if (loc == null)
			{
			return;
			}
		   String iconName = HIconFactory.requestFolderIcon(UIPreferencesFrame.this, loc.getIconName());
		   if (iconName == null)
			{
			return;
			}
		   loc.setIconName(iconName);
		   icon.setIcon(loc.getIcon());
		   locations.repaint();
		   }
		});
	   add.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final ListModel model = locations.getModel();
		   final Vector<FileRequester.Location> data = new Vector<FileRequester.Location>();
		   for (int i = 0; i < model.getSize(); ++i)
			{
			final FileRequester.Location loc = (FileRequester.Location) model.getElementAt(i);
			data.addElement(loc);
			}
		   final FileRequester.Location loc = new FileRequester.Location(System.getProperty("user.home"), "home", JReqTools.getResourceString("FileRequester.Location.Home"));
		   data.addElement(loc);
		   locations.setListData(data);
		   locations.setSelectedValue(loc, true);
		   }
		});
	   remove.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final int index = locations.getSelectedIndex();
		   final FileRequester.Location selected = (FileRequester.Location) locations.getSelectedValue();
		   final ListModel model = locations.getModel();
		   if (index < 0 || model.getSize() == 1)
			{
			return;
			}
		   final Vector<FileRequester.Location> data = new Vector<FileRequester.Location>();
		   for (int i = 0; i < model.getSize(); ++i)
			{
			final FileRequester.Location loc = (FileRequester.Location) model.getElementAt(i);
			if (i != index)
			   {
			   data.addElement(loc);
			   }
			}
		   locations.setListData(data);
		   locations.setSelectedIndex(Math.max(0, index < data.size() ? index : index - 1));
		   }
		});
	   up.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final int index = locations.getSelectedIndex();
		   final FileRequester.Location selected = (FileRequester.Location) locations.getSelectedValue();
		   final ListModel model = locations.getModel();
		   if (index < 0 || index == 0)
			{
			return;
			}
		   final Vector<FileRequester.Location> data = new Vector<FileRequester.Location>();
		   for (int i = 0; i < model.getSize(); ++i)
			{
			final FileRequester.Location loc = (FileRequester.Location) model.getElementAt(i);
			if (i != index)
			   {
			   data.addElement(loc);
			   }
			}
		   data.insertElementAt(selected, index - 1);
		   locations.setListData(data);
		   locations.setSelectedValue(selected, true);
		   }
		});
	   down.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   final int index = locations.getSelectedIndex();
		   final FileRequester.Location selected = (FileRequester.Location) locations.getSelectedValue();
		   final ListModel model = locations.getModel();
		   if (index < 0 || index == model.getSize() - 1)
			{
			return;
			}
		   final Vector<FileRequester.Location> data = new Vector<FileRequester.Location>();
		   for (int i = 0; i < model.getSize(); ++i)
			{
			final FileRequester.Location loc = (FileRequester.Location) model.getElementAt(i);
			if (i != index)
			   {
			   data.addElement(loc);
			   }
			}
		   data.insertElementAt(selected, index + 1);
		   locations.setListData(data);
		   locations.setSelectedValue(selected, true);
		   }
		});

	   // create editor panel
	   GridBagConstraints c = UIPreferences.getGridBagConstraints();
	   final JPanel editor = new JPanel();
	   editor.setLayout(new GridBagLayout());

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   editor.add(new JLabel(Helper.GRS("UIPreferencesFrame.FileRequester.Label.Directory"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   editor.add(directory, c);

	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   editor.add(new JLabel(Helper.GRS("UIPreferencesFrame.FileRequester.Label.Icon"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   editor.add(icon, c);

	   c.gridx = 0;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   editor.add(new JLabel(Helper.GRS("UIPreferencesFrame.FileRequester.Label.Description"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   editor.add(description, c);

	   c.gridx = 0;
	   c.gridy = 3;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   editor.add(new JPanel(), c);

	   // create result panel
	   c.insets = new Insets(1, 1, 1, 1);
	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 5;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   this.add(new JScrollPane(locations), c);

	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JPanel(), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.NONE;
	   this.add(add, c);
	   c.gridx = 2;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.NONE;
	   this.add(remove, c);
	   c.gridx = 3;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.NONE;
	   this.add(up, c);
	   c.gridx = 4;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.NONE;
	   this.add(down, c);

	   c = UIPreferences.getGridBagConstraints();

	   c.gridx = 5;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 2;
	   c.weightx = 2;
	   c.weighty = 1;
	   c.fill = GridBagConstraints.BOTH;
	   this.add(editor, c);

	   // select
	   locations.setSelectedIndex(0);
	   }

	private FileRequester.Location[] createLocationsArray()
	   {
	   final ListModel model = locations.getModel();
	   final FileRequester.Location[] result = new FileRequester.Location[model.getSize()];

	   for (int i = 0; i < result.length; ++i)
		{
		FileRequester.Location loc = (FileRequester.Location) model.getElementAt(i);
		result[i] = loc;
		}

	   return result;
	   }

	public void save()
	   {
	   if (CONTEXT == UIPreferences.DEFAULT_CONTEXT)
		{
		FileRequester.setLocations(createLocationsArray());
		}
	   }

	public void setFromContext(String context)
	   {
	   }
	}

   public class FontPrefs extends PrefsPanel
	{
	// components
	private final JCheckBox setFont = new JCheckBox();
	private final JCheckBox setAcceleratorFont = new JCheckBox();
	private final JCheckBox setTabFont = new JCheckBox();
	private final JButton font = new JButton();
	private final JButton acceleratorFont = new JButton();
	private final JButton tabFont = new JButton();
	private final JButton fixedWidthFont = new JButton();
	private final JButton messageFont = new JButton();
	// requesters
	private final FontRequester fontRequester = new FontRequester(
		  Helper.GRS("UIPreferencesFrame.FontRequester.Title"),
		  Helper.GRS("UIPreferencesFrame.FontRequester.Message"),
		  UIPreferences.getFont());

	public FontPrefs(String context)
	   {
	   super(context);

	   // init
	   reset();

	   // listeners
	   setFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   font.setEnabled(setFont.isSelected());
		   }
		});
	   setAcceleratorFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   acceleratorFont.setEnabled(setAcceleratorFont.isSelected());
		   }
		});
	   setTabFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   tabFont.setEnabled(setTabFont.isSelected());
		   }
		});

	   font.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   fontRequester.setFont(font.getFont());
		   fontRequester.setFixedWidthOnly(false);
		   if (fontRequester.request(UIPreferencesFrame.this) != FontRequester.OK)
			{
			return;
			}
		   font.setFont(fontRequester.getFont());
		   font.setText(createFontName(fontRequester.getFont()));
		   }
		});

	   acceleratorFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   fontRequester.setFont(acceleratorFont.getFont());
		   fontRequester.setFixedWidthOnly(false);
		   if (fontRequester.request(UIPreferencesFrame.this) != FontRequester.OK)
			{
			return;
			}
		   acceleratorFont.setFont(fontRequester.getFont());
		   acceleratorFont.setText(createFontName(fontRequester.getFont()));
		   }
		});

	   fixedWidthFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   fontRequester.setFont(fixedWidthFont.getFont());
		   fontRequester.setFixedWidthOnly(true);
		   if (fontRequester.request(UIPreferencesFrame.this) != FontRequester.OK)
			{
			return;
			}
		   fixedWidthFont.setFont(fontRequester.getFont());
		   fixedWidthFont.setText(createFontName(fontRequester.getFont()));
		   }
		});

	   tabFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   fontRequester.setFont(tabFont.getFont());
		   fontRequester.setFixedWidthOnly(false);
		   if (fontRequester.request(UIPreferencesFrame.this) != FontRequester.OK)
			{
			return;
			}
		   tabFont.setFont(fontRequester.getFont());
		   tabFont.setText(createFontName(fontRequester.getFont()));
		   }
		});

	   messageFont.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   fontRequester.setFont(messageFont.getFont());
		   fontRequester.setFixedWidthOnly(false);
		   if (fontRequester.request(UIPreferencesFrame.this) != FontRequester.OK)
			{
			return;
			}
		   messageFont.setFont(fontRequester.getFont());
		   messageFont.setText(createFontName(fontRequester.getFont()));
		   }
		});

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.StandardFont"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setFont, c);
	   c.gridx = 2;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(font, c);

	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.AcceleratorFont"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setAcceleratorFont, c);
	   c.gridx = 2;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(acceleratorFont, c);

	   c.gridx = 0;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.TabFont"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setTabFont, c);
	   c.gridx = 2;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(tabFont, c);

	   c.gridx = 0;
	   c.gridy = 3;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.FixedWidthFont"), JLabel.LEFT), c);
	   c.gridx = 2;
	   c.gridy = 3;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(fixedWidthFont, c);

	   c.gridx = 0;
	   c.gridy = 4;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.MessageFont"), JLabel.LEFT), c);
	   c.gridx = 2;
	   c.gridy = 4;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(messageFont, c);

	   // prefs
	   prefs.add("UIPreferencesFrame.FontRequester", fontRequester);
	   }

	private final String createFontName(Font f)
	   {
	   String style = "";
	   if (f.isBold() && f.isItalic())
		{
		style = ", bi";
		}
	   else if (f.isBold())
		{
		style = ", b";
		}
	   else if (f.isItalic())
		{
		style = ", i";
		}

	   return f.getName() + ", " + f.getSize() + style;
	   }

	public void save()
	   {
	   UIPreferences.setSetFont(CONTEXT, setFont.isSelected());
	   UIPreferences.setSetAcceleratorFont(CONTEXT, setAcceleratorFont.isSelected());
	   UIPreferences.setSetTabFont(CONTEXT, setTabFont.isSelected());

	   UIPreferences.setFont(CONTEXT, font.getFont());
	   UIPreferences.setAcceleratorFont(CONTEXT, acceleratorFont.getFont());
	   UIPreferences.setTabFont(CONTEXT, tabFont.getFont());
	   UIPreferences.setFixedWidthFont(CONTEXT, fixedWidthFont.getFont());
	   UIPreferences.setMessageFont(CONTEXT, messageFont.getFont());
	   }

	public void setFromContext(String context)
	   {
	   setFont.setSelected(UIPreferences.getSetFont(context));
	   setAcceleratorFont.setSelected(UIPreferences.getSetAcceleratorFont(context));
	   setTabFont.setSelected(UIPreferences.getSetTabFont(context));

	   Font f = UIPreferences.getFont(context);
	   font.setFont(f);
	   font.setText(createFontName(f));
	   font.setEnabled(UIPreferences.getSetFont(context));

	   f = UIPreferences.getAcceleratorFont(context);
	   acceleratorFont.setFont(f);
	   acceleratorFont.setText(createFontName(f));
	   acceleratorFont.setEnabled(UIPreferences.getSetAcceleratorFont(context));

	   f = UIPreferences.getTabFont(context);
	   tabFont.setFont(f);
	   tabFont.setText(createFontName(f));
	   tabFont.setEnabled(UIPreferences.getSetTabFont(context));

	   f = UIPreferences.getFixedWidthFont(context);
	   fixedWidthFont.setFont(f);
	   fixedWidthFont.setText(createFontName(f));

	   f = UIPreferences.getMessageFont(context);
	   messageFont.setFont(f);
	   messageFont.setText(createFontName(f));
	   }
	}

   public class LocalePrefs extends PrefsPanel
	{
	// Variables
	private final Locale[] locales = getLocales();
	// components
	private final JCheckBox setLocale = new JCheckBox();
	private final JComboBox locale = new JComboBox(locales);

	public LocalePrefs(String context)
	   {
	   super(context);

	   // init
	   locale.setRenderer(new HLocaleListCellRenderer());
	   reset();

	   // listeners
	   setLocale.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   locale.setEnabled(setLocale.isSelected());
		   }
		});

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.SetLocale"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setLocale, c);

	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.Locale"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(locale, c);
	   }

	private final Locale[] getLocales()
	   {
	   final Locale[] locale = Locale.getAvailableLocales();
	   final Collator collator = Collator.getInstance();

	   Arrays.sort(locale, new Comparator<Locale>()
		{
		public int compare(Locale l1, Locale l2)
		   {
		   return collator.compare(l1.getDisplayName(), l2.getDisplayName());
		   }
		});

	   return locale;
	   }

	public void save()
	   {
	   UIPreferences.setSetLocale(CONTEXT, setLocale.isSelected());
	   UIPreferences.setLocale(CONTEXT, locale.getSelectedItem().toString());
	   }

	public void setFromContext(String context)
	   {
	   setLocale.setSelected(UIPreferences.getSetLocale(context));
	   locale.setEnabled(UIPreferences.getSetLocale(context));

	   final String saved = UIPreferences.getLocale(context);
	   for (int i = 0; i < locales.length; ++i)
		{
		if (locales[i].toString().equals(saved))
		   {
		   locale.setSelectedIndex(i);
		   break;
		   }
		}
	   }
	}

   public class LookAndFeelPrefs extends PrefsPanel
	{
	// constants
	final String[] METAL_THEMES = new String[]
	   {
	   "javax.swing.plaf.metal.OceanTheme",
	   "javax.swing.plaf.metal.DefaultMetalTheme",
	   "de.hillenbrand.swing.plaf.metal.themes.AquaTheme",
	   "de.hillenbrand.swing.plaf.metal.themes.CharcoalTheme",
	   "de.hillenbrand.swing.plaf.metal.themes.ContrastTheme",
	   "de.hillenbrand.swing.plaf.metal.themes.EmeraldTheme",
	   "de.hillenbrand.swing.plaf.metal.themes.RubyTheme",
	   };
	// components
	private final JButton lookAndFeel = new JButton();
	private final JCheckBox setMetalTheme = new JCheckBox();
	// components: MetalLookAndFeel
	private final JComboBox metalTheme = new JComboBox(METAL_THEMES);
	// components: SkinLookAndFeel
	private final HFileString themePack = new HFileString();
	// requesters
	private final LookAndFeelRequester lookAndFeelRequester = new LookAndFeelRequester(
		  Helper.GRS("UIPreferencesFrame.LookAndFeelRequester.Title"),
		  Helper.GRS("UIPreferencesFrame.LookAndFeelRequester.Message"));

	public LookAndFeelPrefs(String context)
	   {
	   super(context);

	   // init
	   metalTheme.setEditable(true);
	   reset();

	   // listeners
	   lookAndFeel.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   if (lookAndFeelRequester.request(UIPreferencesFrame.this) != LookAndFeelRequester.OK)
			{
			return;
			}
		   String lf = lookAndFeelRequester.getLookAndFeel();
		   lookAndFeel.setText(lf);

		   final boolean enable = lf.equals(UIPreferences.METAL_LOOK_AND_FEEL) || lf.equals(UIPreferences.METAL_LOOK_AND_FEEL_JAVAGPE);
		   setMetalTheme.setEnabled(enable);
		   metalTheme.setEnabled(enable);

		   themePack.setEnabled(lf.equals(UIPreferences.SKIN_LOOK_AND_FEEL));
		   }
		});
	   setMetalTheme.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   metalTheme.setEnabled(setMetalTheme.isSelected());
		   }
		});

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.LookAndFeel"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 2;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(lookAndFeel, c);

	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.MetalTheme"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setMetalTheme, c);
	   c.gridx = 2;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(metalTheme, c);

	   c.gridx = 0;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.SkinLFThemePack"), JLabel.LEFT), c);
	   c.gridx = 2;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(themePack, c);

	   // prefs
	   prefs.add("UIPreferencesFrame.LookAndFeelRequester", lookAndFeelRequester);
	   }

	public void save()
	   {
	   UIPreferences.setLookAndFeel(CONTEXT, lookAndFeel.getText());
	   UIPreferences.setSetMetalTheme(CONTEXT, setMetalTheme.isSelected());
	   UIPreferences.setMetalTheme(CONTEXT, metalTheme.getSelectedItem().toString());
	   UIPreferences.setSkinLFThemePack(CONTEXT, themePack.getText());
	   }

	public void setFromContext(String context)
	   {
	   final String saved = UIPreferences.getLookAndFeel(context);
	   lookAndFeel.setText(saved);

	   setMetalTheme.setSelected(UIPreferences.getSetMetalTheme(context));
	   setMetalTheme.setEnabled(saved.equals(UIPreferences.METAL_LOOK_AND_FEEL) || saved.equals(UIPreferences.METAL_LOOK_AND_FEEL_JAVAGPE));
	   metalTheme.setSelectedItem(UIPreferences.getMetalTheme(context));
	   metalTheme.setEnabled(setMetalTheme.isSelected() && (saved.equals(UIPreferences.METAL_LOOK_AND_FEEL) || saved.equals(UIPreferences.METAL_LOOK_AND_FEEL_JAVAGPE)));

	   themePack.setText(UIPreferences.getSkinLFThemePack(context));
	   themePack.setEnabled(saved.equals(UIPreferences.SKIN_LOOK_AND_FEEL));
	   }
	}

   public abstract class PrefsPanel extends JPanel
	{
	protected final String CONTEXT;

	public PrefsPanel(String context)
	   {
	   super(new GridBagLayout());
	   this.setBorder(UIPreferences.getFrameBorder());

	   CONTEXT = context;
	   }

	public String getContext()
	   {
	   return CONTEXT;
	   }

	public void reset()
	   {
	   setFromContext(CONTEXT);
	   }

	public abstract void save();

	public abstract void setFromContext(String context);
	}

   public class ScreenPrefs extends PrefsPanel
	{
	final JComboBox screens = new JComboBox(GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices());
	final JButton identify = new JButton("identify");

	public ScreenPrefs(String context)
	   {
	   super(context);

	   // init
	   reset();

	   // listeners
	   identify.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   action_identify();
		   }
		});

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.PrimaryScreen"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(screens, c);
	   c.gridx = 2;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.BOTH;
	   this.add(identify, c);
	   }

	protected void action_identify()
	   {
	   final GraphicsDevice gd = (GraphicsDevice) screens.getSelectedItem();
	   if (gd == null)
		{
		return;
		}

	   final GraphicsConfiguration gc = gd.getDefaultConfiguration();
	   final Rectangle bounds = gc.getBounds();

	   final JLabel l = new JLabel("X", JLabel.CENTER);
	   l.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 50));

	   final JPanel p = new JPanel(new GridLayout());
	   p.setBorder(BorderFactory.createLineBorder(Color.black, 5));
	   p.add(l);

	   final Window f = new Window(null, gc);
	   f.add(p);
	   f.setBounds(bounds.x + bounds.width / 2 - 50, bounds.y + bounds.height / 2 - 50, 100, 100);
	   f.setVisible(true);

	   final Thread thread = new Thread()
		{
		public void run()
		   {
		   setName("JavaGPE-Identify");
		   try
			{
			Thread.sleep(2000);
			}
		   catch (Exception x)
			{
			System.err.println(x);
			}
		   f.setVisible(false);
		   f.dispose();
		   }
		};
	   thread.start();
	   }

	public void save()
	   {
	   UIPreferences.setPrimaryGraphicsDevice(CONTEXT, screens.getSelectedIndex());
	   }

	public void setFromContext(String context)
	   {
	   final int index = UIPreferences.getPrimaryGraphicsDevice(context);
	   if (0 <= index && index < screens.getModel().getSize())
		{
		screens.setSelectedIndex(index);
		}
	   else if (index != -1)
		{
		JReqTools.showMessage(UIPreferencesFrame.this, "Default graphics device has been reset to 0.");
		}
	   }
	}

   public class SoundPrefs extends PrefsPanel
	{
	private final HFileString errorSound = new HFileString();
	private final JButton errorSoundPlay = new JButton(getIcon("play.gif"));

	public SoundPrefs(String context)
	   {
	   super(context);

	   // init
	   errorSoundPlay.setMargin(new Insets(1, 1, 1, 1));
	   reset();

	   // listeners
	   errorSoundPlay.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   AudioUtils.playSound(errorSound.getFile().toString());
		   }
		});

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.ErrorSound"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 1;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(errorSound, c);
	   c.gridx = 2;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.BOTH;
	   this.add(errorSoundPlay, c);
	   }

	public void save()
	   {
	   final File f = errorSound.getFile();
	   if (f != null)
		{
		UIPreferences.setErrorSound(CONTEXT, f.toString());
		}
	   }

	public void setFromContext(String context)
	   {
	   errorSound.setFile(UIPreferences.getErrorSound(context));
	   }
	}

   public class TextFieldPrefs extends PrefsPanel
	{
	// components
	private final JCheckBox setCaretColor = new JCheckBox();
	private final JCheckBox setTextSelectionColor = new JCheckBox();
	private final JCheckBox setBlinkRate = new JCheckBox();
	private final JButton caretColor = new JButton();
	private final JButton textSelectionColor = new JButton();
	private final JComboBox blinkRate = new JComboBox(new String[]
		     {
		     "100", "200", "300", "400", "500", "600", "700", "800", "900", "1000"
		     });
	// requester
	private final ColorRequester colorRequester = new ColorRequester(
		  Helper.GRS("UIPreferencesFrame.ColorRequester.Title"),
		  Helper.GRS("UIPreferencesFrame.ColorRequester.Message"),
		  Color.red);

	public TextFieldPrefs(String context)
	   {
	   super(context);

	   // init
	   final Font font = UIPreferences.getFont();
	   final int buttonSize = font.getSize() + 8;
	   caretColor.setBorder(BorderFactory.createLineBorder(Color.black, 1));
	   caretColor.setPreferredSize(new Dimension(buttonSize, buttonSize));
	   textSelectionColor.setBorder(BorderFactory.createLineBorder(Color.black, 1));
	   textSelectionColor.setPreferredSize(new Dimension(buttonSize, buttonSize));
	   blinkRate.setEditable(true);

	   reset();

	   // listeners
	   setBlinkRate.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   blinkRate.setEnabled(setBlinkRate.isSelected());
		   }
		});
	   setCaretColor.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   caretColor.setEnabled(setCaretColor.isSelected());
		   }
		});
	   setTextSelectionColor.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   textSelectionColor.setEnabled(setTextSelectionColor.isSelected());
		   }
		});

	   caretColor.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   colorRequester.setColor(caretColor.getBackground());
		   if (colorRequester.request(UIPreferencesFrame.this) != ColorRequester.OK)
			{
			return;
			}
		   caretColor.setBackground(colorRequester.getColor());
		   }
		});
	   textSelectionColor.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   colorRequester.setColor(textSelectionColor.getBackground());
		   if (colorRequester.request(UIPreferencesFrame.this) != ColorRequester.OK)
			{
			return;
			}
		   textSelectionColor.setBackground(colorRequester.getColor());
		   }
		});
	   blinkRate.addActionListener(new ActionListener()
		{
		public void actionPerformed(ActionEvent e)
		   {
		   try
			{
			int caretBlinkRate = Integer.parseInt(blinkRate.getSelectedItem().toString());
			}
		   catch (Exception x)
			{
			JReqTools.showException(UIPreferencesFrame.this, x);
			}
		   }
		});

	   // create panel
	   final GridBagConstraints c = UIPreferences.getGridBagConstraints();

	   c.gridx = 0;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.CaretBlinkRate"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setBlinkRate, c);
	   c.gridx = 2;
	   c.gridy = 0;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(blinkRate, c);

	   c.gridx = 0;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.CaretColor"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setCaretColor, c);
	   c.gridx = 2;
	   c.gridy = 1;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(caretColor, c);

	   c.gridx = 0;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(new JLabel(Helper.GRS("UIPreferencesFrame.SelectionColor"), JLabel.LEFT), c);
	   c.gridx = 1;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(setTextSelectionColor, c);
	   c.gridx = 2;
	   c.gridy = 2;
	   c.gridwidth = 1;
	   c.gridheight = 1;
	   c.weightx = 0;
	   c.weighty = 0;
	   c.fill = GridBagConstraints.HORIZONTAL;
	   this.add(textSelectionColor, c);

	   // prefs
	   prefs.add("UIPreferencesFrame.ColorRequester", colorRequester);
	   }

	public void save()
	   {
	   UIPreferences.setSetCaretBlinkRate(CONTEXT, setBlinkRate.isSelected());
	   UIPreferences.setSetCaretColor(CONTEXT, setCaretColor.isSelected());
	   UIPreferences.setSetTextSelectionColor(CONTEXT, setTextSelectionColor.isSelected());

	   UIPreferences.setCaretBlinkRate(CONTEXT, Integer.parseInt(blinkRate.getSelectedItem().toString()));
	   UIPreferences.setCaretColor(CONTEXT, caretColor.getBackground());
	   UIPreferences.setTextSelectionColor(CONTEXT, textSelectionColor.getBackground());
	   }

	public void setFromContext(String context)
	   {
	   setBlinkRate.setSelected(UIPreferences.getSetCaretBlinkRate(context));
	   setCaretColor.setSelected(UIPreferences.getSetCaretColor(context));
	   setTextSelectionColor.setSelected(UIPreferences.getSetTextSelectionColor(context));

	   blinkRate.setEnabled(setBlinkRate.isSelected());
	   caretColor.setEnabled(setCaretColor.isSelected());
	   textSelectionColor.setEnabled(setTextSelectionColor.isSelected());

	   blinkRate.setSelectedItem(new Integer(UIPreferences.getCaretBlinkRate(context)));
	   caretColor.setBackground(UIPreferences.getCaretColor(context));
	   textSelectionColor.setBackground(UIPreferences.getTextSelectionColor(context));

	   repaint();
	   }
	}

   public UIPreferencesFrame()
	{
	super(Helper.GRS("UIPreferencesFrame.Title"), null);

	// default context
	data.put(UIPreferences.DEFAULT_CONTEXT + "LOOK", new LookAndFeelPrefs(UIPreferences.DEFAULT_CONTEXT));
	data.put(UIPreferences.DEFAULT_CONTEXT + "LOCALE", new LocalePrefs(UIPreferences.DEFAULT_CONTEXT));
	data.put(UIPreferences.DEFAULT_CONTEXT + "FONT", new FontPrefs(UIPreferences.DEFAULT_CONTEXT));
	data.put(UIPreferences.DEFAULT_CONTEXT + "TEXTFIELD", new TextFieldPrefs(UIPreferences.DEFAULT_CONTEXT));
	data.put(UIPreferences.DEFAULT_CONTEXT + "FILEREQ", new FileRequesterPrefs(UIPreferences.DEFAULT_CONTEXT));
	data.put(UIPreferences.DEFAULT_CONTEXT + "SOUND", new SoundPrefs(UIPreferences.DEFAULT_CONTEXT));
	data.put(UIPreferences.DEFAULT_CONTEXT + "SCREEN", new ScreenPrefs(UIPreferences.DEFAULT_CONTEXT));

	// init
	delete.setMargin(new Insets(1, 1, 1, 1));
	copy.setMargin(new Insets(1, 1, 1, 1));
	reset.setMargin(new Insets(1, 1, 1, 1));
	contexts.setSelectedIndex(0);
	contexts.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.LookAndFeel"), HIconFactory.getMenuIcon("configure"), data.get(UIPreferences.DEFAULT_CONTEXT + "LOOK"));
	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.Locale"), HIconFactory.getMenuIcon("flag"), data.get(UIPreferences.DEFAULT_CONTEXT + "LOCALE"));
	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.Fonts"), HIconFactory.getMenuIcon("charset"), data.get(UIPreferences.DEFAULT_CONTEXT + "FONT"));
	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.TextField"), HIconFactory.getMenuIcon("editclear"), data.get(UIPreferences.DEFAULT_CONTEXT + "TEXTFIELD"));
	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.FileRequester"), HIconFactory.getMenuIcon("bookmark_folder"), data.get(UIPreferences.DEFAULT_CONTEXT + "FILEREQ"));
	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.Sound"), HIconFactory.getMenuIcon("kalarm"), data.get(UIPreferences.DEFAULT_CONTEXT + "SOUND"));
	tabbedPane.addTab(Helper.GRS("UIPreferencesFrame.TabbedPane.Screen"), HIconFactory.getMenuIcon("window_new"), data.get(UIPreferences.DEFAULT_CONTEXT + "SCREEN"));

	// listeners
	delete.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_deleteContext(e);
		}
	   });
	copy.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_copyContext(e);
		}
	   });
	reset.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		action_resetContext(e);
		}
	   });
	contexts.addListSelectionListener(new ListSelectionListener()
	   {
	   public void valueChanged(ListSelectionEvent e)
		{
		if (!e.getValueIsAdjusting())
		   {
		   action_editContext(e);
		   }
		}
	   });

	// add menus
	setJMenuBar(createJMenuBar());

	// packen
	setBounds(100, 100, 640, 480);

	// prefs
	prefs.add("UIPreferencesFrame", this);
	prefs.add("UIPreferencesFrame.TabbedPane", tabbedPane);
	prefs.add("UIPreferencesFrame.SplitPane", splitPane);
	}

   public void about()
	{
	JReqTools.showMessage(this, Helper.GRS("UIPreferencesFrame.AboutMessage"));
	}

   private void action_copyContext(ActionEvent e)
	{
	// context
	final String context = (String) contexts.getSelectedValue();

	// copy from
	final ArrayList<String> data = new ArrayList<String>();
	for (int i = 0; i < contexts.getModel().getSize(); ++i)
	   {
	   data.add(contexts.getModel().getElementAt(i).toString());
	   }
	data.remove(context);
	if (data.size() == 0)
	   {
	   return;
	   }
	final String copyfrom = (String) JReqTools.requestObject(this, "Java GPE", Helper.GRS("UIPreferencesFrame.CopyFrom"), data.toArray(new String[0]));
	if (copyfrom == null)
	   {
	   return;
	   }

	// do it
	for (PrefsPanel p : this.data.values())
	   {
	   if (!p.getContext().equals(context))
		{
		continue;
		}
	   p.setFromContext(copyfrom);
	   }
	}

   private void action_deleteContext(ActionEvent e)
	{
	final String context = (String) contexts.getSelectedValue();
	if (context.equals(UIPreferences.DEFAULT_CONTEXT))
	   {
	   JReqTools.showError(this, Helper.GRS("UIPreferencesFrame.NotPossibleInContext", context));
	   return;
	   }

	final boolean ok = JReqTools.showQuestion(this, Helper.GRS("UIPreferencesFrame.DeleteContext", context));
	if (ok)
	   {
	   UIPreferences.setContextActive(context, false);
	   contexts.setSelectedIndex(0);
	   }
	}

   private void action_editContext(ListSelectionEvent e)
	{
	// context
	final String context = (String) contexts.getSelectedValue();
	boolean enabled = UIPreferences.isContextActive(context);
	if (!enabled)
	   {
	   final boolean ok = JReqTools.showQuestion(UIPreferencesFrame.this, Helper.GRS("UIPreferencesFrame.ActivateContext", context));
	   if (ok)
		{
		UIPreferences.setContextActive(context, true);
		enabled = true;
		}
	   else
		{
		contexts.setSelectedIndex(0);
		return;
		}
	   }

	// change panels
	PrefsPanel panel = data.get(context + "LOOK");
	if (panel == null)
	   {
	   data.put(context + "LOOK", panel = new LookAndFeelPrefs(context));
	   }
	tabbedPane.setComponentAt(0, panel);
	tabbedPane.repaint();

	panel = data.get(context + "LOCALE");
	if (panel == null)
	   {
	   data.put(context + "LOCALE", panel = new LocalePrefs(context));
	   }
	tabbedPane.setComponentAt(1, panel);
	tabbedPane.repaint();

	panel = data.get(context + "FONT");
	if (panel == null)
	   {
	   data.put(context + "FONT", panel = new FontPrefs(context));
	   }
	tabbedPane.setComponentAt(2, panel);
	tabbedPane.repaint();

	panel = data.get(context + "TEXTFIELD");
	if (panel == null)
	   {
	   data.put(context + "TEXTFIELD", panel = new TextFieldPrefs(context));
	   }
	tabbedPane.setComponentAt(3, panel);
	tabbedPane.repaint();

	panel = data.get(context + "FILEREQ");
	if (panel == null)
	   {
	   data.put(context + "FILEREQ", panel = new FileRequesterPrefs(context));
	   }
	tabbedPane.setComponentAt(4, panel);
	tabbedPane.repaint();

	panel = data.get(context + "SOUND");
	if (panel == null)
	   {
	   data.put(context + "SOUND", panel = new SoundPrefs(context));
	   }
	tabbedPane.setComponentAt(5, panel);
	tabbedPane.repaint();

	panel = data.get(context + "SCREEN");
	if (panel == null)
	   {
	   data.put(context + "SCREEN", panel = new ScreenPrefs(context));
	   }
	tabbedPane.setComponentAt(6, panel);
	tabbedPane.repaint();
	}

   private void action_resetContext(ActionEvent e)
	{
	// context
	final String context = (String) contexts.getSelectedValue();

	// do it
	for (PrefsPanel p : data.values())
	   {
	   if (!p.getContext().equals(context))
		{
		continue;
		}
	   p.reset();
	   }
	}

   protected void cancel()
	{
	// just quit
	quit();
	}

   private JMenuBar createJMenuBar()
	{
	final JMenuBar result = new JMenuBar();
	JMenu menu = null;
	JMenuItem item = null;

	// menu: program
	menu = new JMenu(Helper.GRS("UIPreferencesFrame.Menu.Program"));
	result.add(menu);

	item = new JMenuItem(Helper.GRS("UIPreferencesFrame.MenuItem.About"), HIconFactory.getMenuIcon("info"));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		about();
		}
	   });
	menu.add(item);

	menu.addSeparator();

	item = new JMenuItem(Helper.GRS("UIPreferencesFrame.MenuItem.Quit"), HIconFactory.getMenuIcon("exit"));
	item.addActionListener(new ActionListener()
	   {
	   public void actionPerformed(ActionEvent e)
		{
		quit();
		}
	   });
	menu.add(item);

	return result;
	}

   private String[] getContexts()
	{
	final File directory = new File(System.getProperty("user.home"), ".JavaUI");
	final File file = new File(directory, "contexts");
	final TreeSet<String> all = new TreeSet<String>();

	try
	   {
	   final BufferedReader reader = new BufferedReader(new FileReader(file));
	   while (true)
		{
		final String line = reader.readLine();
		if (line == null)
		   {
		   break;
		   }
		all.add(line);
		}
	   reader.close();
	   }
	catch (FileNotFoundException x)
	   {
	   }
	catch (IOException x)
	   {
	   System.err.println("Could not read registered contexts: " + x);
	   }

	final ArrayList<String> result = new ArrayList<String>();
	result.add("<default>");
	result.addAll(all);

	return result.toArray(new String[0]);
	}

   ImageIcon getIcon(String iconName)
	{
	try
	   {
	   final InputStream resource = getClass().getResourceAsStream("icons/" + iconName);
	   if (resource == null)
		{
		System.err.println("[ERROR in method " + getClass().getName() + ".getIcon(): Can't load icon '" + iconName + "']");
		return null;
		}
	   final BufferedInputStream in = new BufferedInputStream(resource);
	   final ByteArrayOutputStream out = new ByteArrayOutputStream(1024);
	   byte[] buffer = new byte[1024];
	   int n;
	   while ((n = in.read(buffer)) > 0)
		{
		out.write(buffer, 0, n);
		}
	   in.close();
	   out.flush();
	   buffer = out.toByteArray();
	   return new ImageIcon(buffer);
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   return null;
	   }
	}

   protected JPanel getUserPanel()
	{
	final JPanel result = new JPanel(new GridBagLayout());

	final GridBagConstraints c = UIPreferences.getGridBagConstraints();
	c.insets = new Insets(1, 1, 1, 1);

	final JPanel contextPanel = new JPanel(new GridBagLayout());
	c.gridx = 0;
	c.gridy = 0;
	c.gridwidth = 4;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.NONE;
	contextPanel.add(new JLabel(Helper.GRS("UIPreferencesFrame.Context"), JLabel.LEFT), c);
	c.gridx = 0;
	c.gridy = 1;
	c.gridwidth = 4;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 1;
	c.fill = GridBagConstraints.BOTH;
	contextPanel.add(new JScrollPane(contexts), c);

	c.gridx = 0;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.BOTH;
	contextPanel.add(new JPanel(), c);
	c.gridx = 1;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.NONE;
	contextPanel.add(delete, c);
	c.gridx = 2;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.NONE;
	contextPanel.add(copy, c);
	c.gridx = 3;
	c.gridy = 2;
	c.gridwidth = 1;
	c.gridheight = 1;
	c.weightx = 0;
	c.weighty = 0;
	c.fill = GridBagConstraints.NONE;
	contextPanel.add(reset, c);

	// content panel
	tabbedPane.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
	contextPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 5));
	splitPane.setBorder(null);
	splitPane.setLeftComponent(contextPanel);
	splitPane.setRightComponent(tabbedPane);

	c.gridx = 1;
	c.gridy = 0;
	c.gridwidth = 1;
	c.gridheight = 2;
	c.weightx = 1;
	c.weighty = 1;
	c.fill = GridBagConstraints.BOTH;
	result.add(splitPane, c);

	return result;
	}

   public static void main(String[] args)
	{
	UIPreferences.init("JavaGPE - UIPreferences");

	final UIPreferencesFrame prefsFrame = new UIPreferencesFrame();
	prefsFrame.setVisible(true);
	}

   protected void ok()
	{
	// save each
	for (PrefsPanel p : data.values())
	   {
	   p.save();
	   }

	// save prefs & quit
	UIPreferences.save();
	quit();
	}

   public void quit()
	{
	prefs.save();
	System.exit(0);
	}
   }
