/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import java.applet.*;
import java.net.*;
import java.util.*;

public final class AudioUtils
   {
   private static Hashtable<URL, AudioClip> cache = new Hashtable<URL, AudioClip>();
   private static boolean cacheEnabled = true;

   /**
    * Assigns the value of the enableSoundCaching property.
    */
   public static synchronized void disableSoundCaching()
	{
	cacheEnabled = false;
	}

   /**
    * Assigns the value of the enableSoundCaching property.
    */
   public static synchronized void enableSoundCaching()
	{
	cacheEnabled = true;
	}

   public static synchronized void playSound(String filename)
	{
	try
	   {
	   final URL url = new URL("file://" + filename);
	   AudioUtils.playSound(url);
	   }
	catch (Exception x)
	   {
	   System.err.println(x);
	   }
	}

   public static synchronized void playSound(URL url)
	{
	AudioClip audioClip = null;

	if (cacheEnabled)
	   {
	   audioClip = cache.get(url);
	   }

	if (audioClip == null)
	   {
	   audioClip = Applet.newAudioClip(url);
	   if (cacheEnabled)
		{
		cache.put(url, audioClip);
		}
	   }

	audioClip.play();
	}
   }
