/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import java.text.*;
import java.util.*;

public class DateUtils
   {
   protected final static DecimalFormat oneDigitFormat = new DecimalFormat("####0");
   protected final static DecimalFormat twoDigitFormat = new DecimalFormat("###00");
   protected final static DecimalFormat threeDigitFormat = new DecimalFormat("##000");

   private DateUtils()
	{
	}

   public static String getDate()
	{
	return getDate(0);
	}

   public static String getDate(long lastModified)
	{
	Calendar date = Calendar.getInstance();
	date.setTime(lastModified != 0 ? new Date(lastModified) : new Date());
	return date.get(Calendar.DAY_OF_MONTH) + "." + (date.get(Calendar.MONTH) + 1) + "." + date.get(Calendar.YEAR);
	}

   public static String calculateTime(final Calendar startTime, final Calendar endTime)
	{
	if (endTime != null && startTime != null)
	   {
	   long millis = endTime.getTimeInMillis() - startTime.getTimeInMillis();

	   long seconds = millis / 1000;
	   millis %= 1000;
	   long minutes = seconds / 60;
	   seconds %= 60;
	   long hours = minutes / 60;
	   minutes %= 60;

	   return oneDigitFormat.format(hours) + ":" + twoDigitFormat.format(minutes) + ":" + twoDigitFormat.format(seconds) + "." + threeDigitFormat.format(millis);
	   }
	else
	   {
	   return "n/a";
	   }
	}
   }
