/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package de.hillenbrand.util;

import com.sun.jimi.core.*;
import java.io.*;

public class ImageUtils
   {
   private static final boolean DEBUG = false;

   public static final void convertToJPG(String dirname, String filename)
	{
	convertToJPG(dirname, filename, 0);
	}

   public static final void convertToJPG(String dirname, String filename, int maxWidth)
	{
	final int index = filename.lastIndexOf(".");
	convertToJPG(dirname, filename, filename.substring(0, index) + ".jpg", maxWidth);
	}

   public static final void convertToJPG(File source, File destination, int maxWidth, int maxHeight)
	{
	java.awt.Image img = null;

	if (!source.exists() || source.lastModified() < destination.lastModified())
	   {
	   return;
	   }
	if (maxHeight < 1)
	   {
	   maxHeight = 9999;
	   }

	try
	   {
	   if (DEBUG)
		{
		System.out.print("Converting \"" + source + "\" to JPG ... ");
		}
	   if (DEBUG)
		{
		System.out.flush();
		}

	   if (maxWidth > 0)
		{
		img = JimiUtils.getThumbnail(source.toString(), maxWidth, maxHeight, Jimi.VIRTUAL_MEMORY + Jimi.SYNCHRONOUS);
		}
	   else
		{
		img = Jimi.getImage(source.toString());
		}

	   Jimi.putImage("image/jpg", img, destination.toString());
	   }
	catch (Exception x)
	   {
	   x.printStackTrace();
	   }
	finally
	   {
	   if (DEBUG)
		{
		System.out.println("done");
		}
	   }
	}

   public static final void convertToJPG(String dirname, String filename, String newFilename, int maxWidth)
	{
	convertToJPG(dirname, filename, newFilename, maxWidth, 9999);
	}

   public static final void convertToJPG(String dirname, String filename, String newFilename, int maxWidth, int maxHeight)
	{
	final File source = new File(dirname, filename);
	final File destination = new File(dirname, newFilename);

	convertToJPG(source, destination, maxWidth, maxHeight);
	}
   }
